/************************************************************************//**
 *
 * @file	snda_reg.h
 *
 * @brief	Definition of the SNDA Register.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 01 Sep. 2016    1.00
 *				- First revision.
 *
 ******************************************************************************/

#ifndef SNDA_REG_H__
#define SNDA_REG_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <c17_mcu_select.h>

/// SNDA Clock Control Register
typedef union SNDCLK_REG_TAG {
    unsigned short  reg;
    struct SNDCLK_REG_BIT_TAG {
        /**
         * These bits select the clock source of SNDA.
         * .CLKSRC[1:0]
         *  These bits select the clock source of SNDA.
         * .CLKDIV[2:0]
         *  These bits select the division ratio of the SNDA operating clock.
         * +---------------+-----------------------------------------+
         * |               |                CLKSRC[1:0]              |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * | CLKDIV[2:0]   +---------+----------+---------+----------+
         * |               | IOSCCLK |   OSC1   | OSC3CLK |  EXOSC   |
         * +---------------+---------+----------+---------+----------+
         * |      0x7      | Reserved|   1/1    | Reserved|   1/1    |
         * +---------------+         +          +         +          +
         * |      0x6      |         |          |         |          |
         * +---------------+---------+          +---------+          +
         * |      0x5      | 1/128   |          | 1/128   |          |
         * +---------------+---------+          +---------+          +
         * |      0x4      | 1/64    |          | 1/64    |          |
         * +---------------+---------+          +---------+          +
         * |      0x3      | 1/32    |          | 1/32    |          |
         * +---------------+---------+          +---------+          +
         * |      0x2      | 1/16    |          | 1/16    |          |
         * +---------------+---------+          +---------+          +
         * |      0x1      | 1/8     |          | 1/8     |          |
         * +---------------+---------+          +---------+          +
         * |      0x0      | 1/4     |          | 1/4     |          |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      CLKSRC      : 2;
        unsigned short      reserved_1  : 2;    ///< Reserved.
        /**
         * These bits select the division ratio of the SNDA operating clock.
         * .CLKSRC[1:0]
         *  These bits select the clock source of SNDA.
         * .CLKDIV[2:0]
         *  These bits select the division ratio of the SNDA operating clock.
         * +---------------+-----------------------------------------+
         * |               |                CLKSRC[1:0]              |
         * |               +---------+----------+---------+----------+
         * |               |   0x0   |   0x1    |   0x2   |   0x3    |
         * | CLKDIV[2:0]   +---------+----------+---------+----------+
         * |               | IOSCCLK |   OSC1   | OSC3CLK |  EXOSC   |
         * +---------------+---------+----------+---------+----------+
         * |      0x7      | Reserved|   1/1    | Reserved|   1/1    |
         * +---------------+         +          +         +          +
         * |      0x6      |         |          |         |          |
         * +---------------+---------+          +---------+          +
         * |      0x5      | 1/128   |          | 1/128   |          |
         * +---------------+---------+          +---------+          +
         * |      0x4      | 1/64    |          | 1/64    |          |
         * +---------------+---------+          +---------+          +
         * |      0x3      | 1/32    |          | 1/32    |          |
         * +---------------+---------+          +---------+          +
         * |      0x2      | 1/16    |          | 1/16    |          |
         * +---------------+---------+          +---------+          +
         * |      0x1      | 1/8     |          | 1/8     |          |
         * +---------------+---------+          +---------+          +
         * |      0x0      | 1/4     |          | 1/4     |          |
         * +---------------+---------+----------+--------------------+
         */
        unsigned short      CLKDIV      : 3;
        unsigned short      reserved_2  : 1;    ///< Reserved.
        /**
         * This bit sets whether the SNDA operating clock is supplied in DEBUG mode or not.
         * .DBRUN
         *  This bit sets whether the SNDA operating clock is supplied in DEBUG mode or not.
         *    1 (R/W): Clock supplied in DEBUG mode
         *    0 (R/W): No clock supplied in DEBUG mode
         */
        unsigned short      DBRUN       : 1;
        unsigned short      reserved_3  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SNDCLK_REG;

/// SNDA Select Register
typedef union SNDSEL_REG_TAG {
    unsigned short  reg;
    struct SNDSEL_REG_BIT_TAG {
        /**
         * These bits select a sound output mode.
         * .MOSEL[1:0]
         *  These bits select a sound output mode.
         * +------------+---------------------+
         * | MOSEL[1:0] | Sound output mode   |
         * +------------+---------------------+
         * |     0x3    | Reserved            |
         * +------------+---------------------+
         * |     0x2    | Melody mode         |
         * +------------+---------------------+
         * |     0x1    | One-shot buzzer mode|
         * +------------+---------------------+
         * |     0x0    | Normal buzzer mode  |
         * +------------+---------------------+
         */
        unsigned short      MOSEL       : 2;
        /**
         * This bit selects an output pin drive mode.
         * .SINV
         *  This bit selects an output pin drive mode.
         *    1 (R/W): Normal drive mode
         *    0 (R/W): Direct drive mode
         */
        unsigned short      SINV        : 1;
        unsigned short      reserved_1  : 5;    ///< Reserved.
        
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
        /**
         * These bits select a tempo or a one-shot buzzer output duration.
         * .STIM[3:0]
         * These bits select a tempo (when melody mode is selected) or a one-shot buzzer output duration (when one-shot buzzer mode is selected).
         * +---------------------+----------------------+
         * | STIM[3:0] | Tempo | One-shot buzzer output |
         * |           |       |    duration [ms]       |
         * +-------------------+------------------------+
         * |  0xf      |  30   |  250.0                 |
         * +-------------------+------------------------+
         * |  0xe      |  32   |  234.4                 |
         * +-------------------+------------------------+
         * |  0xd      |  34.3 |  218.8                 |
         * +-------------------+------------------------+
         * |  0xc      |  36.9 |  203.1                 |
         * +-------------------+------------------------+
         * |  0xb      |  40   |  187.5                 |
         * +-------------------+------------------------+
         * |  0xa      |  43.6 |  171.9                 |
         * +-------------------+------------------------+
         * |  0x9      |  48   |  156.3                 |
         * +-------------------+------------------------+
         * |  0x8      |  53.3 |  140.0                 |
         * +-------------------+------------------------+
         * |  0x7      |  60   |  125.0                 |
         * +-------------------+------------------------+
         * |  0x6      |  68.6 |  109.4                 |
         * +-------------------+------------------------+
         * |  0x5      |  80   |   93.8                 |
         * +-------------------+------------------------+
         * |  0x4      |  96   |   78.1                 |
         * +-------------------+------------------------+
         * |  0x3      |  120  |   62.5                 |
         * +-------------------+------------------------+
         * |  0x2      |  160  |   46.9                 |
         * +-------------------+------------------------+
         * |  0x1      |  240  |   31.3                 |
         * +-------------------+------------------------+
         * |  0x0      |  480  |   15.6                 |
         * +-------------------+------------------------+
         */
#else
        /**
         * These bits select a tempo or a one-shot buzzer output duration.
         * .STIM[3:0]
         * These bits select a tempo (when melody mode is selected) or a one-shot buzzer output duration (when one-shot buzzer mode is selected).
         * +---------------------+----------------------+
         * | STIM[3:0] | Tempo | One-shot buzzer output |
         * |           |       |    duration [ms]       |
         * +-------------------+------------------------+
         * |  0xf      |  30   |  240.2                 |
         * +-------------------+------------------------+
         * |  0xe      |  32   |  224.6                 |
         * +-------------------+------------------------+
         * |  0xd      |  34.3 |  208.8                 |
         * +-------------------+------------------------+
         * |  0xc      |  36.9 |  193.4                 |
         * +-------------------+------------------------+
         * |  0xb      |  40   |  177.7                 |
         * +-------------------+------------------------+
         * |  0xa      |  43.6 |  162.1                 |
         * +-------------------+------------------------+
         * |  0x9      |  48   |  146.5                 |
         * +-------------------+------------------------+
         * |  0x8      |  53.3 |  129.0                 |
         * +-------------------+------------------------+
         * |  0x7      |  60   |  115.2                 |
         * +-------------------+------------------------+
         * |  0x6      |  68.6 |   99.6                 |
         * +-------------------+------------------------+
         * |  0x5      |  80   |   84.0                 |
         * +-------------------+------------------------+
         * |  0x4      |  96   |   68.4                 |
         * +-------------------+------------------------+
         * |  0x3      |  120  |   52.7                 |
         * +-------------------+------------------------+
         * |  0x2      |  160  |   37.1                 |
         * +-------------------+------------------------+
         * |  0x1      |  240  |   21.5                 |
         * +-------------------+------------------------+
         * |  0x0      |  480  |    5.9                 |
         * +-------------------+------------------------+
         */
#endif
        unsigned short      STIM        : 4;
        unsigned short      reserved_2  : 4;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SNDSEL_REG;

/// SNDA Control Register
typedef union SNDCTL_REG_TAG {
    unsigned short  reg;
    struct SNDCTL_REG_BIT_TAG {
        /**
         * This bit enables the SNDA operations.
         * .MODEN
         *  This bit enables the SNDA operations.
         *    1 (R/W): Enable SNDA operations (The operating clock is supplied.)
         *    0 (R/W): Disable SNDA operations (The operating clock is stopped.)
         */
        unsigned short      MODEN       : 1;
        unsigned short      reserved_1  : 7;    ///< Reserved.
        /**
         * This bit stops sound output.
         * .SSTP
         *  This bit stops sound output.
         *    1 (W): Stop sound output
         *    0 (W): Ineffective
         *    1 (R): In stop process
         *    0 (R): Stop process completed/Idle
         */
        unsigned short      SSTP        : 1;
        unsigned short      reserved_2  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SNDCTL_REG;

/// SNDA Data Register
typedef union SNDDAT_REG_TAG {
    unsigned short  reg;
    struct SNDDAT_REG_BIT_TAG {
        /// These bits select a scale or a buzzer signal frequency.
        unsigned short      SFRQ        : 8;
        /// These bits select a duration or a buzzer signal duty ratio.
        unsigned short      SLEN        : 6;
        /**
         * This bit selects the output type in melody mode from a note or a rest.
         * .MDRS
         *  This bit selects the output type in melody mode from a note or a rest.
         *    1 (R/W): Rest
         *    0 (R/W): Note
         */
        unsigned short      MDRS        : 1;
        /**
         * This bit specifies a tie or slur (continuous play with the previous note) in melody mode.
         * .MDTI
         *  This bit specifies a tie or slur (continuous play with the previous note) in melody mode.
         *    1 (R/W): Enable tie/slur
         *    0 (R/W): Disable tie/slur
         */
        unsigned short      MDTI        : 1;
    } bit;
    unsigned char   byte_reg[2];
} SNDDAT_REG;

/// SNDA Interrupt Flag Register
typedef union SNDINTF_REG_TAG {
    unsigned short  reg;
    struct SNDINTF_REG_BIT_TAG {
        /**
         * Sound buffer empty interrupt status.
         * .EDIF
         *  Sound buffer empty interrupt status.
         *  This bit indicate the SNDA interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      EDIF        : 1;
        /**
         * Sound output completion interrupt status.
         * .EMIF
         *  Sound output completion interrupt status.
         *  This bit indicate the SNDA interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      EMIF        : 1;
        unsigned short      reserved_1  : 6;    ///< Reserved.
        /**
         * This bit indicates the sound output status.
         * .SBSY
         *  This bit indicates the sound output status.
         *    1 (R): Outputting
         *    0 (R): Idle
         */
        unsigned short      SBSY        : 1;
        unsigned short      reserved_2  : 7;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SNDINTF_REG;

/// SNDA Interrupt Enable Register
typedef union SNDINTE_REG_TAG {
    unsigned short  reg;
    struct SNDINTE_REG_BIT_TAG {
        /**
         * Sound output completion interrupt.
         * .EDIE
         *  Sound output completion interrupt.
         *  This bit indicate the SNDA interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      EDIE        : 1;
        /**
         * Sound buffer empty interrupt.
         * .EMIE
         *  Sound buffer empty interrupt.
         *  This bit indicate the SNDA interrupt cause occurrence status.
         *    1 (R): Cause of interrupt occurred
         *    0 (R): No cause of interrupt occurred
         *    1 (W): Clear flag
         *    0 (W): Ineffective
         */
        unsigned short      EMIE        : 1;
        unsigned short      reserved_1  :14;    ///< Reserved.
    } bit;
    unsigned char   byte_reg[2];
} SNDINTE_REG;

#ifdef __cplusplus
}
#endif

#endif // SNDA_REG_H__
