/************************************************************************//**
 *
 * @file    c17_lcd8a_1.c
 *
 * @brief   This is LCD8A driver program file.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include "crt0.h"
#include "c17_misc.h"
#include "c17_init_config.h"
#include "c17_lcd8a_1.h"

#if defined(C17_PE_LCD8A)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
#if defined(C17_MCUSEL_M01)
C17_INTERRUPT_HANDLER(_vector17_handler, c17intLcd8a);		// LCD8A interrupt
#endif
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
C17_INTERRUPT_HANDLER(_vector19_handler, c17intLcd8a);		// LCD8A interrupt
#endif
#endif /* CRT0_LINKED */

/* --- global variable --- */
volatile unsigned int c17lcd8afrmIntCount = 0;


/* --- function declaration --- */
static void initLcd8aSegRam(void);

/************************************************************************//**
 * @brief   This function initializes LCD8A driver.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initLcd8a(void)
{
	/// Disable LCD8A.
	c17offLcd8aDisplay();

	/// LCD8A Setting
	if((LCD8A_INIT & 0x20) == 0x20)
	{
		/// Initialize LCD8A.
		c17setLcd8a((LCD8A_INIT & 0x18)>>3, (LCD8A_INIT & 0x0F));

		/// Initialize the interrupt.
		ILVLCD8A = INTERRUPT_LEVEL_LCD8A;				/// - Set interrupt level.
	}
}


/************************************************************************//**
 * @brief   Enable LCD8A interrupt .
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntLcd8a(void)
{
 	LCD8INTF_REG	lcd8intf_set;
 	LCD8INTE_REG	lcd8inte_set;

	/// Clear interrupt flags.
	/* LCDINTF:LCD8A Interrupt Flag Register. */
	{
		lcd8intf_set.bit.FRMIF = 1;
		LCD8INTF = lcd8intf_set.reg;
	}

	/// Disable LCD8A frame interrupts.
	/* LCD8INTE:LCD8A Interrupt Enable Register. */
	{
		lcd8inte_set.bit.FRMIE = 1;
		LCD8INTE = lcd8inte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Disable LCD8A interrupt .
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntLcd8a(void)
{
 	LCD8INTF_REG	lcd8intf_set;
 	LCD8INTE_REG	lcd8inte_set;

	/// Clear interrupt flags.
	/* LCD8INTF:LCD8A Interrupt Flag Register. */
	{
		lcd8intf_set.bit.FRMIF = 1;
		LCD8INTF = lcd8intf_set.reg;
	}

	/// Disable LCD8A frame interrupts.
	/* LCD8INTE:LCD8A Interrupt Enable Register. */
	{
		lcd8inte_set.bit.FRMIE = 0;
		LCD8INTE = lcd8inte_set.reg;
	}
}


/************************************************************************//**
 * @brief   The display turn on.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17onLcd8aDisplay(void)
{
 	LCD8CTL_REG	lcd8ctl_set;
 	LCD8DSP_REG	lcd8dsp_set;

	/* LCD8CTL:LCD8A Control Register */
	{
		lcd8ctl_set.reg = LCD8CTL;
		lcd8ctl_set.bit.MODEN = 1;			/// - LCD8A enable
		LCD8CTL = lcd8ctl_set.reg;
	}

	/* LCD8DSP:LCD8A Display Control Register. */
	{
		lcd8dsp_set.reg = LCD8DSP;
		lcd8dsp_set.bit.DSPC = 1;			/// The LCD display state set to "normal display".
		LCD8DSP = lcd8dsp_set.reg;
	}
}


/************************************************************************//**
 * @brief   The display turn off.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17offLcd8aDisplay(void)
{
 	LCD8CTL_REG	lcd8ctl_set;
 	LCD8DSP_REG	lcd8dsp_set;

	/* LCD8DSP:LCD8A Display Control Register. */
	{
		lcd8dsp_set.reg = LCD8DSP;
		lcd8dsp_set.bit.DSPC = 0;			/// The LCD display state set to "display off".
		LCD8DSP = lcd8dsp_set.reg;
	}

	/* LCD8CTL:LCD8A Control Register */
	{
		lcd8ctl_set.reg = LCD8CTL;
		lcd8ctl_set.bit.MODEN = 0;			/// - LCD8A disable
		LCD8CTL = lcd8ctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function changes LCD driver's state.
 *
 * @param   state
 *  @li @c  C17_LCD_STATE_ALL_OFF   Mode : All off.
 *  @li @c  C17_LCD_STATE_ALL_ON    Mode : All on.
 *  @li @c  C17_LCD_STATE_NORMAL    Mode : Normal display.
 *  @li @c  C17_LCD_STATE_OFF       Mode : Display off.
 *
 * @retval  none
 ***************************************************************************/
void c17setLcd8aDisplayState(unsigned char state)
{
 	LCD8DSP_REG	lcd8dsp_set;

	/* LCD8DSP:LCD8A Display Control Register. */
	{
		lcd8dsp_set.reg = LCD8DSP;
		lcd8dsp_set.bit.DSPC = state;		/// The LCD display state set.
		LCD8DSP = lcd8dsp_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function gets LCD driver's state.
 *
 * @param   none
 *
 * @retval  C17_LCD_STATE_ALL_OFF   Mode : All off.
 * @retval  C17_LCD_STATE_ALL_ON    Mode : All on.
 * @retval  C17_LCD_STATE_NORMAL    Mode : Normal display.
 * @retval  C17_LCD_STATE_OFF       Mode : Display off.
 ***************************************************************************/
unsigned char c17getLcd8aDisplayState(void)
{
 	LCD8DSP_REG	lcd8dsp_set;

	/// LCD8A Display Control Register
 	lcd8dsp_set.reg = LCD8DSP;

	/// LCD display mode.
	return lcd8dsp_set.bit.DSPC;
}


/************************************************************************//**
 * @brief   This function changes LCD driver's display.
 *
 * @param   display
 *  @li @c  C17_LCD_DISPLAY_INVERTED    Inverted display.
 *  @li @c  C17_LCD_DISPLAY_NORMAL      Normal display.
 *
 * @retval  none
 ****************************************************************************/
void c17setLcd8aDisplayInverted(unsigned char display)
{
 	LCD8DSP_REG	lcd8dsp_set;

	/* LCD8DSP:LCD8A Display Control Register. */
	{
		lcd8dsp_set.reg = LCD8DSP;
		lcd8dsp_set.bit.DSPREV = display;		/// The LCD display black/white inversion set.
		LCD8DSP = lcd8dsp_set.reg;
	}
}


/************************************************************************//**
 * @brief   This function gets LCD driver's display.
 *
 * @param   none
 *
 * @retval  C17_LCD_DISPLAY_INVERTED    Inverted display.
 * @retval  C17_LCD_DISPLAY_NORMAL      Normal display.
 ***************************************************************************/
unsigned char c17getLcd8aDisplayInverted(void)
{
 	LCD8DSP_REG	lcd8dsp_set;

	/// LCD8A Display Control Register
 	lcd8dsp_set.reg = LCD8DSP;

	/// LCD display black/white inversion.
	return lcd8dsp_set.bit.DSPREV;
}

/************************************************************************//**
 * @brief   This function changes LCD driver's display area.
 *
 * @param   area
 *  @li @c  C17_LCD_DISPLAY_AREA0   display area 0.
 *  @li @c  C17_LCD_DISPLAY_AREA1   display area 1.
 *
 * @retval	none
 ***************************************************************************/
void c17setLcd8aDisplayArea(unsigned char area)
{
 	LCD8DSP_REG	lcd8dsp_set;

	/* LCD8DSP:LCD8A Display Control Register. */
	{
		lcd8dsp_set.reg = LCD8DSP;
		lcd8dsp_set.bit.DSPAR = area;		/// The LCD display area set.
		LCD8DSP = lcd8dsp_set.reg;
	}
}

/************************************************************************//**
 * @brief   This function gets LCD driver's display area.
 *
 * @param   none
 *
 * @retval  C17_LCD_DISPLAY_AREA0   display area 0.
 * @retval  C17_LCD_DISPLAY_AREA1   display area 1.
 ***************************************************************************/
unsigned char c17getLcd8aDisplayArea(void)
{
 	LCD8DSP_REG	lcd8dsp_set;

	/// LCD8A Display Control Register
 	lcd8dsp_set.reg = LCD8DSP;

	/// LCD display area.
	return lcd8dsp_set.bit.DSPAR;
}

/************************************************************************//**
 * @brief   This function set checker-board pattern(small) for display.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17setLcd8aCheckerboardPattern(void)
{
	unsigned short i;
	volatile unsigned char *segram = (unsigned char *)LCD8A_SEGRAM_TOP_ADDR;

	for (i = 0; i < LCD8A_SEGRAM_SIZE; i++) {
		if (i % 2 == 0) {
			segram[i] = 0x55;
		} else {
			segram[i] = 0xAA;
		}
	}
}

#if defined(C17_MCUSEL_M01)
/************************************************************************//**
 * @brief   This function sets LCD8A.
 *
 * @param   lcdClkSrc      Clock Source.
 *  @li @c  C17_LCD_CLKSRC_IOSC
 *  @li @c  C17_LCD_CLKSRC_OSC1
 *  @li @c  C17_LCD_CLKSRC_OSC3
 *  @li @c  C17_LCD_CLKSRC_EXT
 * @param   lcdClkDiv      Clock Division
 *  @li @c  C17_LCD_CLKDIV_1
 *  @li @c  C17_LCD_CLKDIV_16
 *  @li @c  C17_LCD_CLKDIV_32
 *  @li @c  C17_LCD_CLKDIV_64
 *  @li @c  C17_LCD_CLKDIV_128
 *  @li @c  C17_LCD_CLKDIV_256
 *  @li @c  C17_LCD_CLKDIV_512
 *
 * @retval  none
 ***************************************************************************/
void c17setLcd8a(unsigned short lcdClkSrc, unsigned short lcdClkDiv)
{
	P1FNCSEL_REG	p1fncsel_set;
	P1MODSEL_REG	p1modsel_set;

	P2FNCSEL_REG	p2fncsel_set;
	P2MODSEL_REG	p2modsel_set;

	P3FNCSEL_REG	p3fncsel_set;
	P3MODSEL_REG	p3modsel_set;

	P4FNCSEL_REG	p4fncsel_set;
	P4MODSEL_REG	p4modsel_set;

	P5FNCSEL_REG	p5fncsel_set;
	P5MODSEL_REG	p5modsel_set;

	LCD8CLK_REG		lcd8clk_set;

 	LCD8TIM_REG		lcd8tim_set;

 	LCD8PWR_REG		lcd8pwr_set;

 	LCD8DSP_REG		lcd8dsp_set;

 	LCD8INTF_REG	lcd8intf_set;
 	LCD8INTE_REG	lcd8inte_set;

	/// Initialize LEDC ports.

	/// - LCD8A Driver (LCD8A / SEG6)
	/* P1FNCSEL:P1 Port Function Select Register */
	{
		p1fncsel_set.bit.P13MUX = 3;		/// - P13 : UPMUX.
		P1FNCSEL = p1fncsel_set.reg;
	}

	/* P1MODSEL:P1 Port Mode Select Register */
	{
		p1modsel_set.bit.P13SEL = 1;		/// - P13 : Enable peripheral I/O function.
		P1MODSEL = p1modsel_set.reg;
	}

	/// - LCD8A Driver (SEG8 to SEG13)
	/* P2FNCSEL:P2 Port Function Select Register */
	{
		p2fncsel_set.bit.P27MUX = 3;		/// - P27 : UPMUX.
		p2fncsel_set.bit.P26MUX = 3;		/// - P26 : UPMUX.
		p2fncsel_set.bit.P25MUX = 3;		/// - P25 : UPMUX.
		p2fncsel_set.bit.P24MUX = 3;		/// - P24 : UPMUX.
		p2fncsel_set.bit.P23MUX = 3;		/// - P23 : UPMUX.
		p2fncsel_set.bit.P22MUX = 3;		/// - P22 : UPMUX.
		P2FNCSEL = p2fncsel_set.reg;
	}

	/* P2MODSEL:P2 Port Mode Select Register */
	{
		p2modsel_set.bit.P27SEL = 1;		/// - P27 : Enable peripheral I/O function.
		p2modsel_set.bit.P26SEL = 1;		/// - P26 : Enable peripheral I/O function.
		p2modsel_set.bit.P25SEL = 1;		/// - P25 : Enable peripheral I/O function.
		p2modsel_set.bit.P24SEL = 1;		/// - P24 : Enable peripheral I/O function.
		p2modsel_set.bit.P23SEL = 1;		/// - P23 : Enable peripheral I/O function.
		p2modsel_set.bit.P22SEL = 1;		/// - P22 : Enable peripheral I/O function.
		P2MODSEL = p2modsel_set.reg;
	}

	/// - LCD8A Driver (SEG14 to SEG21)
	/* P3FNCSEL:P3 Port Function Select Register */
	{
		p3fncsel_set.bit.P37MUX = 3;		/// - P37 : UPMUX.
		p3fncsel_set.bit.P36MUX = 3;		/// - P36 : UPMUX.
		p3fncsel_set.bit.P35MUX = 3;		/// - P35 : UPMUX.
		p3fncsel_set.bit.P34MUX = 3;		/// - P34 : UPMUX.
		p3fncsel_set.bit.P33MUX = 3;		/// - P33 : UPMUX.
		p3fncsel_set.bit.P32MUX = 3;		/// - P32 : UPMUX.
		p3fncsel_set.bit.P31MUX = 3;		/// - P31 : UPMUX.
		p3fncsel_set.bit.P30MUX = 3;		/// - P30 : UPMUX.
		P3FNCSEL = p3fncsel_set.reg;
	}

	/* P3MODSEL:P3 Port Mode Select Register */
	{
		p3modsel_set.bit.P37SEL = 1;		/// - P37 : Enable peripheral I/O function.
		p3modsel_set.bit.P36SEL = 1;		/// - P36 : Enable peripheral I/O function.
		p3modsel_set.bit.P35SEL = 1;		/// - P35 : Enable peripheral I/O function.
		p3modsel_set.bit.P34SEL = 1;		/// - P34 : Enable peripheral I/O function.
		p3modsel_set.bit.P33SEL = 1;		/// - P33 : Enable peripheral I/O function.
		p3modsel_set.bit.P32SEL = 1;		/// - P32 : Enable peripheral I/O function.
		p3modsel_set.bit.P31SEL = 1;		/// - P31 : Enable peripheral I/O function.
		p3modsel_set.bit.P30SEL = 1;		/// - P30 : Enable peripheral I/O function.
		P3MODSEL = p3modsel_set.reg;
	}

	/// - LCD8A Driver (SEG22 to SEG29)
	/* P4FNCSEL:P4 Port Function Select Register */
	{
		p4fncsel_set.bit.P47MUX = 3;		/// - P47 : UPMUX.
		p4fncsel_set.bit.P46MUX = 3;		/// - P46 : UPMUX.
		p4fncsel_set.bit.P45MUX = 3;		/// - P45 : UPMUX.
		p4fncsel_set.bit.P44MUX = 3;		/// - P44 : UPMUX.
		p4fncsel_set.bit.P43MUX = 3;		/// - P43 : UPMUX.
		p4fncsel_set.bit.P42MUX = 3;		/// - P42 : UPMUX.
		p4fncsel_set.bit.P41MUX = 3;		/// - P41 : UPMUX.
		p4fncsel_set.bit.P40MUX = 3;		/// - P40 : UPMUX.
		P4FNCSEL = p4fncsel_set.reg;
	}

	/* P4MODSEL:P4 Port Mode Select Register */
	{
		p4modsel_set.bit.P47SEL = 1;		/// - P47 : Enable peripheral I/O function.
		p4modsel_set.bit.P46SEL = 1;		/// - P46 : Enable peripheral I/O function.
		p4modsel_set.bit.P45SEL = 1;		/// - P45 : Enable peripheral I/O function.
		p4modsel_set.bit.P44SEL = 1;		/// - P44 : Enable peripheral I/O function.
		p4modsel_set.bit.P43SEL = 1;		/// - P43 : Enable peripheral I/O function.
		p4modsel_set.bit.P42SEL = 1;		/// - P42 : Enable peripheral I/O function.
		p4modsel_set.bit.P41SEL = 1;		/// - P41 : Enable peripheral I/O function.
		p4modsel_set.bit.P40SEL = 1;		/// - P40 : Enable peripheral I/O function.
		P4MODSEL = p4modsel_set.reg;
	}

	/// - LCD8A Driver (SEG30, SEG31 and COM0 to COM3)
	/* P5FNCSEL:P5 Port Function Select Register */
	{
		p5fncsel_set.bit.P55MUX = 3;		/// - P55 : UPMUX.
		p5fncsel_set.bit.P54MUX = 3;		/// - P54 : UPMUX.
		p5fncsel_set.bit.P53MUX = 3;		/// - P53 : UPMUX.
		p5fncsel_set.bit.P52MUX = 3;		/// - P52 : UPMUX.
		p5fncsel_set.bit.P51MUX = 3;		/// - P51 : UPMUX.
		p5fncsel_set.bit.P50MUX = 3;		/// - P50 : UPMUX.
		P5FNCSEL = p5fncsel_set.reg;
	}

	/* P5MODSEL:P5 Port Mode Select Register */
	{
		p5modsel_set.bit.P55SEL = 1;		/// - P55 : Enable peripheral I/O function.
		p5modsel_set.bit.P54SEL = 1;		/// - P54 : Enable peripheral I/O function.
		p5modsel_set.bit.P53SEL = 1;		/// - P53 : Enable peripheral I/O function.
		p5modsel_set.bit.P52SEL = 1;		/// - P52 : Enable peripheral I/O function.
		p5modsel_set.bit.P51SEL = 1;		/// - P51 : Enable peripheral I/O function.
		p5modsel_set.bit.P50SEL = 1;		/// - P50 : Enable peripheral I/O function.
		P5MODSEL = p5modsel_set.reg;
	}

	/* LCD8CLK:LCD8A Clock Control Register */
	{
		lcd8clk_set.bit.DBRUN = 1;				/// - Clock supplied in DEBUG mode.
		lcd8clk_set.bit.CLKDIV = lcdClkDiv;		/// - Select the division ratio.
		lcd8clk_set.bit.CLKSRC = lcdClkSrc;		/// - Select the clock source of the LCD8A.
		LCD8CLK = lcd8clk_set.reg;
	}
	/* LCD8TIM:LCD8A Timing Control Register. */
	{
		/**
	     * @todo Change LCD8A Timing Control Register
	     */
		lcd8tim_set.bit.BSTC = 1;			/// Booster clock frequency = fCLK_LCD8A/16 Hz.
		lcd8tim_set.bit.NLINE = 0;			/// N-line inverse AC drive function = Normal drive.
		lcd8tim_set.bit.FRMCNT = 3;			/// Frame frequency = 170.7 Hz.
		lcd8tim_set.bit.LDUTY = 2;			/// Drive duty = 1/3.
		LCD8TIM = lcd8tim_set.reg;
	}
	/* LCD8PWR:LCD8A Power Control Register. */
	{
		/**
	     * @todo Change LCD8A Power Control Register
	     */
		lcd8pwr_set.bit.LC	= 0x0;				/// LCD panel contrast = 0x00:light - 0x1F:dark.
		lcd8pwr_set.bit.BSTEN = 1;				/// LCD voltage booster = off.
		lcd8pwr_set.bit.HVLD = 0;				/// LCD voltage regulator into heavy load protection mode = Normal mode.
		lcd8pwr_set.bit.VCSEL = 1;				/// LCD voltage regulator output = VC2.
		lcd8pwr_set.bit.VCEN = 1;				/// LCD voltage regulator = on.
		LCD8PWR = lcd8pwr_set.reg;
	}

	/* LCD8DSP:LCD8A Display Control Register. */
	{
		/**
	     * @todo Change LCD8A Display Control Register
	     */
		lcd8dsp_set.bit.COM7DEN = 1;			/// Partial drive of COM7 = Normal output;
		lcd8dsp_set.bit.COM6DEN = 1;			/// Partial drive of COM6 = Normal output;
		lcd8dsp_set.bit.COM5DEN = 1;			/// Partial drive of COM5 = Normal output;
		lcd8dsp_set.bit.COM4DEN = 1;			/// Partial drive of COM4 = Normal output;
		lcd8dsp_set.bit.COM3DEN = 1;			/// Partial drive of COM3 = Normal output;
		lcd8dsp_set.bit.COM2DEN = 1;			/// Partial drive of COM2 = Normal output;
		lcd8dsp_set.bit.COM1DEN = 1;			/// Partial drive of COM1 = Normal output;
		lcd8dsp_set.bit.COM0DEN = 1;			/// Partial drive of COM0 = Normal output;
		lcd8dsp_set.bit.SEGREV	= 1;			/// Segment pin assignment direction = Normal assignment.
		lcd8dsp_set.bit.COMREV	= 1;			/// Common pin assignment direction = Normal assignment.
		lcd8dsp_set.bit.DSPREV	= 1;			/// Black/White inversion = Normal display.
		lcd8dsp_set.bit.DSPAR = 0;				/// Display area in the display data RAM = Display area 0.
		lcd8dsp_set.bit.DSPC = 0;				/// LCD display = display off.
		LCD8DSP = lcd8dsp_set.reg;
	}
	/// Clear interrupt flags.
	/* LCD8INTF:LCD8A Interrupt Flag Register. */
	{
		lcd8intf_set.bit.FRMIF = 1;
		LCD8INTF = lcd8intf_set.reg;
	}

	/// Disable LCD8A frame interrupts.
	/* LCD8INTE:LCD8A Interrupt Enable Register. */
	{
		lcd8inte_set.bit.FRMIE = 0;
		LCD8INTE = lcd8inte_set.reg;
	}

	/// initializes VRAM.
	initLcd8aSegRam();

}
#endif

#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
/************************************************************************//**
 * @brief   This function sets LCD8A.
 *
 * @param   lcdClkSrc      Clock Source.
 *  @li @c  C17_LCD_CLKSRC_IOSC
 *  @li @c  C17_LCD_CLKSRC_OSC1
 *  @li @c  C17_LCD_CLKSRC_OSC3
 *  @li @c  C17_LCD_CLKSRC_EXT
 * @param   lcdClkDiv      Clock Division
 *  @li @c  C17_LCD_CLKDIV_1
 *  @li @c  C17_LCD_CLKDIV_IOSC_4
 *  @li @c  C17_LCD_CLKDIV_IOSC_8
 *  @li @c  C17_LCD_CLKDIV_IOSC_16
 *  @li @c  C17_LCD_CLKDIV_IOSC_32
 *  @li @c  C17_LCD_CLKDIV_IOSC_64
 *  @li @c  C17_LCD_CLKDIV_IOSC_128
 *  @li @c  C17_LCD_CLKDIV_OSC3_16
 *  @li @c  C17_LCD_CLKDIV_OSC3_32
 *  @li @c  C17_LCD_CLKDIV_OSC3_64
 *  @li @c  C17_LCD_CLKDIV_OSC3_128
 *  @li @c  C17_LCD_CLKDIV_OSC3_256
 *  @li @c  C17_LCD_CLKDIV_OSC3_512
 *
 * @retval  none
 ***************************************************************************/
void c17setLcd8a(unsigned short lcdClkSrc, unsigned short lcdClkDiv)
{
	P0FNCSEL_REG	p0fncsel_set;
	P0MODSEL_REG	p0modsel_set;

	P1FNCSEL_REG	p1fncsel_set;
	P1MODSEL_REG	p1modsel_set;

	//P2FNCSEL_REG	p2fncsel_set;
	//P2MODSEL_REG	p2modsel_set;

	P3FNCSEL_REG	p3fncsel_set;
	P3MODSEL_REG	p3modsel_set;

	P4FNCSEL_REG	p4fncsel_set;
	P4MODSEL_REG	p4modsel_set;

#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	P5FNCSEL_REG	p5fncsel_set;
	P5MODSEL_REG	p5modsel_set;
#endif

	P6FNCSEL_REG	p6fncsel_set;
	P6MODSEL_REG	p6modsel_set;

	LCD8CLK_REG		lcd8clk_set;

 	LCD8TIM1_REG	lcd8tim1_set;
 	LCD8TIM2_REG	lcd8tim2_set;

 	LCD8PWR_REG		lcd8pwr_set;

 	LCD8DSP_REG		lcd8dsp_set;

 	LCD8COMC0_REG	lcd8comc0_set;

 	LCD8INTF_REG	lcd8intf_set;
 	LCD8INTE_REG	lcd8inte_set;

	/// Initialize LEDC ports.

	/// - LCD8A Driver (SEG17 to SEG24)
	/* P0FNCSEL:P0 Port Function Select Register */
	{
 		p0fncsel_set.reg = P0FNCSEL;
		//p0fncsel_set.bit.P07MUX = 3;		/// - P07 : UPMUX.
		//p0fncsel_set.bit.P06MUX = 3;		/// - P06 : UPMUX.
		//p0fncsel_set.bit.P05MUX = 3;		/// - P05 : UPMUX.
		p0fncsel_set.bit.P04MUX = 3;		/// - P04(SEG20) : UPMUX.
		p0fncsel_set.bit.P03MUX = 3;		/// - P03(SEG21) : UPMUX.
		p0fncsel_set.bit.P02MUX = 3;		/// - P02(SEG22) : UPMUX.
		p0fncsel_set.bit.P01MUX = 3;		/// - P01(SEG23) : UPMUX.
		p0fncsel_set.bit.P00MUX = 3;		/// - P00(SEG24) : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}
	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		//p0modsel_set.bit.P07SEL = 1;		/// - P07 : Enable peripheral I/O function.
		//p0modsel_set.bit.P06SEL = 1;		/// - P06 : Enable peripheral I/O function.
		//p0modsel_set.bit.P05SEL = 1;		/// - P05 : Enable peripheral I/O function.
		p0modsel_set.bit.P04SEL = 1;		/// - P04(SEG20) : Enable peripheral I/O function.
		p0modsel_set.bit.P03SEL = 1;		/// - P03(SEG21) : Enable peripheral I/O function.
		p0modsel_set.bit.P02SEL = 1;		/// - P02(SEG22) : Enable peripheral I/O function.
		p0modsel_set.bit.P01SEL = 1;		/// - P01(SEG23) : Enable peripheral I/O function.
		p0modsel_set.bit.P00SEL = 1;		/// - P00(SEG24) : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}

	/// - LCD8A Driver (SEG9 to SEG16)
	/* P1FNCSEL:P1 Port Function Select Register */
	{
		p1fncsel_set.reg = P1FNCSEL;
		p1fncsel_set.bit.P17MUX = 3;		/// - P17(SEG9)  : UPMUX.
		p1fncsel_set.bit.P16MUX = 3;		/// - P16(SEG10) : UPMUX.
		p1fncsel_set.bit.P15MUX = 3;		/// - P15(SEG11) : UPMUX.
		p1fncsel_set.bit.P14MUX = 3;		/// - P14(SEG12) : UPMUX.
		p1fncsel_set.bit.P13MUX = 3;		/// - P13(SEG13) : UPMUX.
		p1fncsel_set.bit.P12MUX = 3;		/// - P12(SEG14) : UPMUX.
		p1fncsel_set.bit.P11MUX = 3;		/// - P11(SEG15) : UPMUX.
		//p1fncsel_set.bit.P10MUX = 3;		/// - P10 : UPMUX.
		P1FNCSEL = p1fncsel_set.reg;
	}
	/* P1MODSEL:P1 Port Mode Select Register */
	{
		p1modsel_set.reg = P1MODSEL;
		p1modsel_set.bit.P17SEL = 1;		/// - P17(SEG9)  : Enable peripheral I/O function.
		p1modsel_set.bit.P16SEL = 1;		/// - P16(SEG10) : Enable peripheral I/O function.
		p1modsel_set.bit.P15SEL = 1;		/// - P15(SEG11) : Enable peripheral I/O function.
		p1modsel_set.bit.P14SEL = 1;		/// - P14(SEG12) : Enable peripheral I/O function.
		p1modsel_set.bit.P13SEL = 1;		/// - P13(SEG13) : Enable peripheral I/O function.
		p1modsel_set.bit.P12SEL = 1;		/// - P12(SEG14) : Enable peripheral I/O function.
		p1modsel_set.bit.P11SEL = 1;		/// - P11(SEG15) : Enable peripheral I/O function.
		//p1modsel_set.bit.P10SEL = 1;		/// - P10        : Enable peripheral I/O function.
		P1MODSEL = p1modsel_set.reg;
	}

	/// - LCD8A Driver (SEG41 to SEG48)
	/* P2FNCSEL:P2 Port Function Select Register */
	{
		//p2fncsel_set.reg = P2FNCSEL;
		//p2fncsel_set.bit.P27MUX = 3;		/// - P27 : UPMUX.
		//p2fncsel_set.bit.P26MUX = 3;		/// - P26 : UPMUX.
		//p2fncsel_set.bit.P25MUX = 3;		/// - P25 : UPMUX.
		//p2fncsel_set.bit.P24MUX = 3;		/// - P24 : UPMUX.
		//p2fncsel_set.bit.P23MUX = 3;		/// - P23 : UPMUX.
		//p2fncsel_set.bit.P22MUX = 3;		/// - P22 : UPMUX.
		//p2fncsel_set.bit.P21MUX = 3;		/// - P21 : UPMUX.
		//p2fncsel_set.bit.P20MUX = 3;		/// - P20 : UPMUX.
		//P2FNCSEL = p2fncsel_set.reg;
	}
	/* P2MODSEL:P2 Port Mode Select Register */
	{
		//p2modsel_set.reg = P2MODSEL;
		//p2modsel_set.bit.P27SEL = 1;		/// - P27 : Enable peripheral I/O function.
		//p2modsel_set.bit.P26SEL = 1;		/// - P26 : Enable peripheral I/O function.
		//p2modsel_set.bit.P25SEL = 1;		/// - P25 : Enable peripheral I/O function.
		//p2modsel_set.bit.P24SEL = 1;		/// - P24 : Enable peripheral I/O function.
		//p2modsel_set.bit.P23SEL = 1;		/// - P23 : Enable peripheral I/O function.
		//p2modsel_set.bit.P22SEL = 1;		/// - P22 : Enable peripheral I/O function.
		//p2modsel_set.bit.P21SEL = 1;		/// - P21 : Enable peripheral I/O function.
		//p2modsel_set.bit.P20SEL = 1;		/// - P20 : Enable peripheral I/O function.
		//P2MODSEL = p2modsel_set.reg;
	}

	/// - LCD8A Driver (SEG33 to SEG40)
	/* P3FNCSEL:P3 Port Function Select Register */
	{
		p3fncsel_set.reg = P3FNCSEL;
		p3fncsel_set.bit.P37MUX = 3;		/// - P37(SEG33) : UPMUX.
		p3fncsel_set.bit.P36MUX = 3;		/// - P36(SEG34) : UPMUX.
		p3fncsel_set.bit.P35MUX = 3;		/// - P35(SEG35) : UPMUX.
		p3fncsel_set.bit.P34MUX = 3;		/// - P34(SEG36) : UPMUX.
		//p3fncsel_set.bit.P33MUX = 3;		/// - P33 : UPMUX.
		//p3fncsel_set.bit.P32MUX = 3;		/// - P32 : UPMUX.
		//p3fncsel_set.bit.P31MUX = 3;		/// - P31 : UPMUX.
		//p3fncsel_set.bit.P30MUX = 3;		/// - P30 : UPMUX.
		P3FNCSEL = p3fncsel_set.reg;
	}
	/* P3MODSEL:P3 Port Mode Select Register */
	{
		p3modsel_set.reg = P3MODSEL;
		p3modsel_set.bit.P37SEL = 1;		/// - P37(SEG33) : Enable peripheral I/O function.
		p3modsel_set.bit.P36SEL = 1;		/// - P36(SEG34) : Enable peripheral I/O function.
		p3modsel_set.bit.P35SEL = 1;		/// - P35(SEG35) : Enable peripheral I/O function.
		p3modsel_set.bit.P34SEL = 1;		/// - P34(SEG36) : Enable peripheral I/O function.
		//p3modsel_set.bit.P33SEL = 1;		/// - P33 : Enable peripheral I/O function.
		//p3modsel_set.bit.P32SEL = 1;		/// - P32 : Enable peripheral I/O function.
		//p3modsel_set.bit.P31SEL = 1;		/// - P31 : Enable peripheral I/O function.
		//p3modsel_set.bit.P30SEL = 1;		/// - P30 : Enable peripheral I/O function.
		P3MODSEL = p3modsel_set.reg;
	}

	/// - LCD8A Driver (SEG4 to SEG8, SEG31, SEG32, SEG49)
	/* P4FNCSEL:P4 Port Function Select Register */
	{
		p4fncsel_set.reg = P4FNCSEL;
		p4fncsel_set.bit.P47MUX = 3;		/// - P47(SEG31) : UPMUX.
		p4fncsel_set.bit.P46MUX = 3;		/// - P46(SEG32) : UPMUX.
		//p4fncsel_set.bit.P45MUX = 3;		/// - P45 : UPMUX.
		p4fncsel_set.bit.P44MUX = 3;		/// - P44(SEG4) : UPMUX.
		p4fncsel_set.bit.P43MUX = 3;		/// - P43(SEG5) : UPMUX.
		p4fncsel_set.bit.P42MUX = 3;		/// - P42(SEG6) : UPMUX.
		p4fncsel_set.bit.P41MUX = 3;		/// - P41(SEG7) : UPMUX.
		p4fncsel_set.bit.P40MUX = 3;		/// - P40(SEG8) : UPMUX.
		P4FNCSEL = p4fncsel_set.reg;
	}
	/* P4MODSEL:P4 Port Mode Select Register */
	{
		p4modsel_set.reg = P4MODSEL;
		p4modsel_set.bit.P47SEL = 1;		/// - P47(SEG31) : Enable peripheral I/O function.
		p4modsel_set.bit.P46SEL = 1;		/// - P46(SEG32) : Enable peripheral I/O function.
		//p4modsel_set.bit.P45SEL = 1;		/// - P45 : Enable peripheral I/O function.
		p4modsel_set.bit.P44SEL = 1;		/// - P44(SEG4) : Enable peripheral I/O function.
		p4modsel_set.bit.P43SEL = 1;		/// - P43(SEG5) : Enable peripheral I/O function.
		p4modsel_set.bit.P42SEL = 1;		/// - P42(SEG6) : Enable peripheral I/O function.
		p4modsel_set.bit.P41SEL = 1;		/// - P41(SEG7) : Enable peripheral I/O function.
		p4modsel_set.bit.P40SEL = 1;		/// - P40(SEG8) : Enable peripheral I/O function.
		P4MODSEL = p4modsel_set.reg;
	}

#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	/// - LCD8A Driver (SEG25 to SEG30)
	/* P5FNCSEL:P5 Port Function Select Register */
	{
		p5fncsel_set.reg = P5FNCSEL;
		p5fncsel_set.bit.P55MUX = 3;		/// - P55(SEG25) : UPMUX.
		p5fncsel_set.bit.P54MUX = 3;		/// - P54(SEG26) : UPMUX.
		p5fncsel_set.bit.P53MUX = 3;		/// - P53(SEG27) : UPMUX.
		p5fncsel_set.bit.P52MUX = 3;		/// - P52(SEG28) : UPMUX.
		p5fncsel_set.bit.P51MUX = 3;		/// - P51(SEG29) : UPMUX.
		p5fncsel_set.bit.P50MUX = 3;		/// - P50(SEG30) : UPMUX.
		P5FNCSEL = p5fncsel_set.reg;
	}
	/* P5MODSEL:P5 Port Mode Select Register */
	{
		p5modsel_set.reg = P5MODSEL;
		p5modsel_set.bit.P55SEL = 1;		/// - P55(SEG25) : Enable peripheral I/O function.
		p5modsel_set.bit.P54SEL = 1;		/// - P54(SEG26) : Enable peripheral I/O function.
		p5modsel_set.bit.P53SEL = 1;		/// - P53(SEG27) : Enable peripheral I/O function.
		p5modsel_set.bit.P52SEL = 1;		/// - P52(SEG28) : Enable peripheral I/O function.
		p5modsel_set.bit.P51SEL = 1;		/// - P51(SEG29) : Enable peripheral I/O function.
		p5modsel_set.bit.P50SEL = 1;		/// - P50(SEG30) : Enable peripheral I/O function.
		P5MODSEL = p5modsel_set.reg;
	}
#endif

	/// - LCD8A Driver (COM0 to COM7)
	/* P6FNCSEL:P6 Port Function Select Register */
	{
		p6fncsel_set.reg = P6FNCSEL;
		p6fncsel_set.bit.P67MUX = 3;		/// - P67(COM0) : UPMUX.
		p6fncsel_set.bit.P66MUX = 3;		/// - P66(COM1) : UPMUX.
		p6fncsel_set.bit.P65MUX = 3;		/// - P65(COM2) : UPMUX.
		p6fncsel_set.bit.P64MUX = 3;		/// - P64(COM3) : UPMUX.
		p6fncsel_set.bit.P63MUX = 3;		/// - P63(SEG0) : UPMUX.
		p6fncsel_set.bit.P62MUX = 3;		/// - P62(SEG1) : UPMUX.
		p6fncsel_set.bit.P61MUX = 3;		/// - P61(SEG2) : UPMUX.
		p6fncsel_set.bit.P60MUX = 3;		/// - P60(SEG3) : UPMUX.
		P6FNCSEL = p6fncsel_set.reg;
	}
	/* P6MODSEL:P6 Port Mode Select Register */
	{
		p6modsel_set.reg = P6MODSEL;
		p6modsel_set.bit.P67SEL = 1;		/// - P67(COM0) : Enable peripheral I/O function.
		p6modsel_set.bit.P66SEL = 1;		/// - P66(COM1) : Enable peripheral I/O function.
		p6modsel_set.bit.P65SEL = 1;		/// - P65(COM2) : Enable peripheral I/O function.
		p6modsel_set.bit.P64SEL = 1;		/// - P64(COM3) : Enable peripheral I/O function.
		p6modsel_set.bit.P63SEL = 1;		/// - P63(SEG0) : Enable peripheral I/O function.
		p6modsel_set.bit.P62SEL = 1;		/// - P62(SEG1) : Enable peripheral I/O function.
		p6modsel_set.bit.P61SEL = 1;		/// - P61(SEG2) : Enable peripheral I/O function.
		p6modsel_set.bit.P60SEL = 1;		/// - P60(SEG3) : Enable peripheral I/O function.
		P6MODSEL = p6modsel_set.reg;
	}

	/* LCD8CLK:LCD8A Clock Control Register */
	{
		lcd8clk_set.bit.DBRUN = 1;				/// - Clock supplied in DEBUG mode.
		lcd8clk_set.bit.CLKDIV = lcdClkDiv;		/// - Select the division ratio.
		lcd8clk_set.bit.CLKSRC = lcdClkSrc;		/// - Select the clock source of the LCD8A.
		LCD8CLK = lcd8clk_set.reg;
	}
	/* LCD8TIM1:LCD8A Timing Control Register1. */
	{
		/**
	     * @todo Change LCD8A Timing Control Register1
	     */
		lcd8tim1_set.bit.FRMCNT = 7;		/// Frame frequency = fclk_LCD8A/(16*(FRMCNT+1)*(LDUTY+1))=64Hz
		lcd8tim1_set.bit.LDUTY = 3;			/// Drive duty = 1/4.
		LCD8TIM1 = lcd8tim1_set.reg;
	}
#if defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M32)
	/* LCD8TIM2:LCD8A Timing Control Register2. */
	{
		/**
	     * @todo Change LCD8A Timing Control Register2
	     */
		lcd8tim2_set.bit.NLINE = 0;			/// N-line inverse AC drive function = Normal drive.
		LCD8TIM2 = lcd8tim2_set.reg;
	}
#endif
#if defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
	/* LCD8TIM2:LCD8A Timing Control Register2. */
	{
		/**
	     * @todo Change LCD8A Timing Control Register2
	     */
		lcd8tim2_set.bit.BSTC = 1;			/// Booster clock frequency = fCLK_LCD8A/16 Hz.
		lcd8tim2_set.bit.NLINE = 0;			/// N-line inverse AC drive function = Normal drive.
		LCD8TIM2 = lcd8tim2_set.reg;
	}
#endif
	/* LCD8PWR:LCD8A Power Control Register. */
	{
		/**
	     * @todo Change LCD8A Power Control Register
	     */
		lcd8pwr_set.bit.EXVCSEL	= 0;		/// selects the LCD drive power supply mode = Internal generation mode.
		LCD8PWR = lcd8pwr_set.reg;
	}
#if defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
	/* LCD8PWR:LCD8A Power Control Register. */
	{
		/**
	     * @todo Change LCD8A Power Control Register
	     */
		lcd8pwr_set.bit.LC	= 0x0;				/// LCD panel contrast = 0x00:light - 0x1F:dark.
		lcd8pwr_set.bit.BSTEN = 1;				/// LCD voltage booster = off.
		lcd8pwr_set.bit.HVLD = 0;				/// LCD voltage regulator into heavy load protection mode = Normal mode.
		lcd8pwr_set.bit.VCSEL = 1;				/// LCD voltage regulator output = VC2.
		lcd8pwr_set.bit.VCEN = 1;				/// LCD voltage regulator = on.
		LCD8PWR = lcd8pwr_set.reg;
	}
#endif
	/* LCD8DSP:LCD8A Display Control Register. */
	{
		/**
	     * @todo Change LCD8A Display Control Register
	     */
		lcd8dsp_set.bit.SEGREV	= 1;			/// Segment pin assignment direction = Normal assignment.
		lcd8dsp_set.bit.COMREV	= 1;			/// Common pin assignment direction = Normal assignment.
		lcd8dsp_set.bit.DSPREV	= 1;			/// Black/White inversion = Normal display.
		lcd8dsp_set.bit.DSPAR = 0;				/// Display area in the display data RAM = Display area 0.
		lcd8dsp_set.bit.DSPC = 0;				/// LCD display = display off.
		LCD8DSP = lcd8dsp_set.reg;
	}
	/* LCD8COMC0:LCD8A COM Pin Control Register. */
	{
		/**
	     * @todo Change LCD8A COM Pin Control Register
	     */
		lcd8comc0_set.bit.COM7DEN = 1;			/// Partial drive of COM7 = Normal output;
		lcd8comc0_set.bit.COM6DEN = 1;			/// Partial drive of COM6 = Normal output;
		lcd8comc0_set.bit.COM5DEN = 1;			/// Partial drive of COM5 = Normal output;
		lcd8comc0_set.bit.COM4DEN = 1;			/// Partial drive of COM4 = Normal output;
		lcd8comc0_set.bit.COM3DEN = 1;			/// Partial drive of COM3 = Normal output;
		lcd8comc0_set.bit.COM2DEN = 1;			/// Partial drive of COM2 = Normal output;
		lcd8comc0_set.bit.COM1DEN = 1;			/// Partial drive of COM1 = Normal output;
		lcd8comc0_set.bit.COM0DEN = 1;			/// Partial drive of COM0 = Normal output;
		LCD8COMC0 = lcd8comc0_set.reg;
	}
	/// Clear interrupt flags.
	/* LCD8INTF:LCD8A Interrupt Flag Register. */
	{
		lcd8intf_set.bit.FRMIF = 1;
		LCD8INTF = lcd8intf_set.reg;
	}

	/// Disable LCD8A frame interrupts.
	/* LCD8INTE:LCD8A Interrupt Enable Register. */
	{
		lcd8inte_set.bit.FRMIE = 0;
		LCD8INTE = lcd8inte_set.reg;
	}

	/// initializes VRAM.
	initLcd8aSegRam();

}
#endif
/************************************************************************//**
 * @brief  clear display.
 *
 * @retval  None.
 ***************************************************************************/
void c17clearLcd8a(void)
{
	initLcd8aSegRam();
}

/************************************************************************//**
 * @brief   LCD8A Interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intLcd8a(void)
{
 	LCD8INTF_REG lcd8intf_set;

	/* LCD8INTF LCD8A Interrupt Flag Register read & clear */
	{
		lcd8intf_set.reg = LCD8INTF;
		LCD8INTF = lcd8intf_set.reg;
	}

	/// Check interrupt cause occurrence status.
	if(lcd8intf_set.bit.FRMIF == 1)
	{
	   /**
		* @todo add interrupt function of frame interrupt.
		*/
		c17lcd8afrmIntCount++;
	}

}



/* ----- static ----- */

/************************************************************************//**
 * @brief   This function initializes RAM of LCD8A Panel(7Segment).
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
static void initLcd8aSegRam(void)
{
	unsigned char	*segram = (unsigned char *)LCD8A_SEGRAM_TOP_ADDR;
	unsigned short	i;

	for(i = 0 ; i < LCD8A_SEGRAM_SIZE ; i++){
		*segram++ = 0x00;
	}
}

#endif
