/************************************************************************//**
 *
 * @file    c17_port_1.c
 *
 * @brief   This is Port program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/

#include "c17_port_1.h"

#include <c17_mcu_select.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_init_config.h"

#if defined(C17_PE_PORT)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector05_handler, c17intPortInput);		// PPORT interrupt
#endif /* CRT0_LINKED */


/* --- static const variable --- */
static const unsigned short highByteTable[8] = {0x0100,0x0200,0x0400,0x0800,0x1000,0x2000,0x4000,0x8000};
static const unsigned short lowByteTable[8]  = {0x0001,0x0002,0x0004,0x0008,0x0010,0x0020,0x0040,0x0080};

/* --- macro definition --- */

#define HIGHBYTE_BIT_SET(portno)				(highByteTable[portno])			///- portno = 0 to 7
#define LOWBYTE_BIT_SET(portno)				(lowByteTable[portno])			///- portno = 0 to 7


/* --- static  function --- */
static inline void intInputP0x(P0INTF_REG* p0nif_get);
static inline void intInputP1x(P1INTF_REG* p1nif_get);
static inline void intInputP2x(P2INTF_REG* p2nif_get);
static inline void intInputP3x(P3INTF_REG* p3nif_get);
static inline void intInputP4x(P4INTF_REG* p4nif_get);
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
static inline void intInputP5x(P5INTF_REG* p5nif_get);
#endif
static inline void intInputP6x(P6INTF_REG* p6nif_get);
static inline void intInputP7x(P7INTF_REG* p7nif_get);

static void enableIntPxx(unsigned short portNo, unsigned short portIntEdge, P0INTCTL_REG* portRegister);
static void disableIntPxx(unsigned short portNo, P0INTCTL_REG* portRegister);
static unsigned char getOutputPxx(unsigned short portNo, P0DAT_REG* portRegister);
static void setOutputPxx(unsigned short portNo, unsigned short portData, P0DAT_REG* portRegister);
static void setOutputPxxByte(unsigned char portData, P0DAT_REG* portRegister);
static unsigned char getInputPxx(unsigned short portNo, P0DAT_REG* portRegister);
static void setChattaPxx(unsigned short portNo, unsigned short portChatta, P0CHATEN_REG* portRegister);
static void setPullUpDownPxx(unsigned short portNo, unsigned short portPull, P0RCTL_REG* portRegister);
static void setDirectionPxx(unsigned short portNo, unsigned short portDir, P0IOEN_REG* portRegister);



/* --- global variable --- */

// Interrupt flag.
/**
 * @todo c17portP0xIntInputFlag = Port P0x interrupt flag
 */
volatile unsigned short c17portP0xIntInputFlag = 0;				///< Port P0x interrupt flag.
volatile unsigned short c17portP1xIntInputFlag = 0;				///< Port P1x interrupt flag.
volatile unsigned short c17portP2xIntInputFlag = 0;				///< Port P2x interrupt flag.
volatile unsigned short c17portP3xIntInputFlag = 0;				///< Port P3x interrupt flag.
volatile unsigned short c17portP4xIntInputFlag = 0;				///< Port P4x interrupt flag.
volatile unsigned short c17portP5xIntInputFlag = 0;				///< Port P5x interrupt flag.
volatile unsigned short c17portP6xIntInputFlag = 0;				///< Port P6x interrupt flag.
volatile unsigned short c17portP7xIntInputFlag = 0;				///< Port P7x interrupt flag.

/************************************************************************//**
 * @brief   Initialization of the PORT Register.
 *
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initPort(void)
{
	PCLK_REG		pclk_set;

	P0INTCTL_REG	p0intctl_set;
	P0IOEN_REG		p0ioen_set;
	P0CHATEN_REG	p0chaten_set;
	P0RCTL_REG		p0rctl_set;
	P0MODSEL_REG	p0modsel_set;

	P1INTCTL_REG	p1intctl_set;
	P1IOEN_REG		p1ioen_set;
	P1CHATEN_REG	p1chaten_set;
	P1RCTL_REG		p1rctl_set;
	P1MODSEL_REG	p1modsel_set;

	P2INTCTL_REG	p2intctl_set;
	P2IOEN_REG		p2ioen_set;
	P2CHATEN_REG	p2chaten_set;
	P2RCTL_REG		p2rctl_set;
	P2MODSEL_REG	p2modsel_set;

	P3INTCTL_REG	p3intctl_set;
	P3IOEN_REG		p3ioen_set;
	P3CHATEN_REG	p3chaten_set;
	P3RCTL_REG		p3rctl_set;
	P3MODSEL_REG	p3modsel_set;

	P4INTCTL_REG	p4intctl_set;
	P4IOEN_REG		p4ioen_set;
	P4CHATEN_REG	p4chaten_set;
	P4RCTL_REG		p4rctl_set;
	P4MODSEL_REG	p4modsel_set;

#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	P5INTCTL_REG	p5intctl_set;
	P5IOEN_REG		p5ioen_set;
	P5CHATEN_REG	p5chaten_set;
	P5RCTL_REG		p5rctl_set;
	P5MODSEL_REG	p5modsel_set;
#endif

	P6INTCTL_REG	p6intctl_set;
	P6IOEN_REG		p6ioen_set;
	P6CHATEN_REG	p6chaten_set;
	P6RCTL_REG		p6rctl_set;
	P6MODSEL_REG	p6modsel_set;

	P7INTCTL_REG	p7intctl_set;
	P7IOEN_REG		p7ioen_set;
	P7CHATEN_REG	p7chaten_set;
	P7RCTL_REG		p7rctl_set;
	P7MODSEL_REG	p7modsel_set;

	PDIOEN_REG		pdioen_set;
	PDRCTL_REG		pdrctl_set;
	//PDMODSEL_REG	pdmodsel_set;

	/* P0INTCTL: disable port input interrupt. */
	{
		p0intctl_set.reg = 0;
		P0INTCTL = p0intctl_set.reg;
	}
	/* P1INTCTL: disable port input interrupt. */
	{
		p1intctl_set.reg = 0;
		P1INTCTL = p1intctl_set.reg;
	}
	/* P2INTCTL: disable port input interrupt. */
	{
		p2intctl_set.reg = 0;
		P2INTCTL = p2intctl_set.reg;
	}
	/* P3INTCTL: disable port input interrupt. */
	{
		p3intctl_set.reg = 0;
		P3INTCTL = p3intctl_set.reg;
	}
	/* P4INTCTL: disable port input interrupt. */
	{
		p4intctl_set.reg = 0;
		P4INTCTL = p4intctl_set.reg;
	}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	/* P5INTCTL: disable port input interrupt. */
	{
		p5intctl_set.reg = 0;
		P5INTCTL = p5intctl_set.reg;
	}
#endif
	/* P6INTCTL: disable port input interrupt. */
	{
		p6intctl_set.reg = 0;
		P6INTCTL = p6intctl_set.reg;
	}
	/* P7INTCTL: disable port input interrupt. */
	{
		p7intctl_set.reg = 0;
		P7INTCTL = p7intctl_set.reg;
	}

	c17protectMiscOff();

	/* PCLK: P Port Clock Control Register */
	{
      /**
       * @todo Change CLKSRC & CLKDIV
       */
		// When using the chattering filter, configure the PPORT operating clock
		{
			pclk_set.bit.DBRUN = 1;			/// - Clock supplied in DEBUG mode.
			pclk_set.bit.CLKDIV = 4;		/// - Set division ratio for clock source(1/16).
			pclk_set.bit.KRSTCFG = 0;		/// - Configure the key-entry reset function.
			pclk_set.bit.CLKSRC = 0;		/// - Set clock source for chattering filter(IOSC).
			PCLK = pclk_set.reg;
		}
	}

	c17protectMiscOn();

	/* P0CHATEN: P0 Port Chattering Filter Enable Register */
	{
		p0chaten_set.reg = 0;
		P0CHATEN = p0chaten_set.reg;
	}
	/* P1CHATEN: P1 Port Chattering Filter Enable Register */
	{
		p1chaten_set.reg = 0;
		P1CHATEN = p1chaten_set.reg;
	}
	/* P2CHATEN: P2 Port Chattering Filter Enable Register */
	{
		p2chaten_set.reg = 0;
		P2CHATEN = p2chaten_set.reg;
	}
	/* P3CHATEN: P3 Port Chattering Filter Enable Register */
	{
		p3chaten_set.reg = 0;
		P3CHATEN = p3chaten_set.reg;
	}
	/* P4CHATEN: P4 Port Chattering Filter Enable Register */
	{
		p4chaten_set.reg = 0;
		P4CHATEN = p4chaten_set.reg;
	}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	/* P5CHATEN: P5 Port Chattering Filter Enable Register */
	{
		p5chaten_set.reg = 0;
		P5CHATEN = p5chaten_set.reg;
	}
#endif
	/* P6CHATEN: P6 Port Chattering Filter Enable Register */
	{
		p6chaten_set.reg = 0;
		P6CHATEN = p6chaten_set.reg;
	}
	/* P7CHATEN: P7 Port Chattering Filter Enable Register */
	{
		p7chaten_set.reg = 0;
		P7CHATEN = p7chaten_set.reg;
	}

	/* P0IOEN:P0 Port Enable Register */
	{
		p0ioen_set.reg = 0;
		P0IOEN = p0ioen_set.reg;
	}
	/* P1IOEN:P1 Port Enable Register */
	{
		p1ioen_set.reg = 0;
		P1IOEN = p1ioen_set.reg;
	}
	/* P2IOEN:P2 Port Enable Register */
	{
		p2ioen_set.reg = 0;
		P2IOEN = p2ioen_set.reg;
	}
	/* P3IOEN:P3 Port Enable Register */
	{
		p3ioen_set.reg = 0;
		P3IOEN = p3ioen_set.reg;
	}
	/* P4IOEN:P4 Port Enable Register */
	{
		p4ioen_set.reg = 0;
		P4IOEN = p4ioen_set.reg;
	}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	/* P5IOEN:P5 Port Enable Register */
	{
		p5ioen_set.reg = 0;
		P5IOEN = p5ioen_set.reg;
	}
#endif
	/* P6IOEN:P6 Port Enable Register */
	{
		p6ioen_set.reg = 0;
		P6IOEN = p6ioen_set.reg;
	}
	/* P7IOEN:P7 Port Enable Register */
	{
		p7ioen_set.reg = 0;
		P7IOEN = p7ioen_set.reg;
	}
	/* PDIOEN:PD Port Enable Register */
	{
		pdioen_set.reg = 0;
		PDIOEN = pdioen_set.reg;
	}

	/* P0RCTL_REG:P0 Port Pull-up/down Control Register */
	{
		p0rctl_set.reg = 0;
		P0RCTL = p0rctl_set.reg;
	}
	/* P1RCTL_REG:P1 Port Pull-up/down Control Register */
	{
		p1rctl_set.reg = 0;
		P1RCTL = p1rctl_set.reg;
	}
	/* P2RCTL_REG:P2 Port Pull-up/down Control Register */
	{
		p2rctl_set.reg = 0;
		P2RCTL = p2rctl_set.reg;
	}
	/* P3RCTL_REG:P3 Port Pull-up/down Control Register */
	{
		p3rctl_set.reg = 0;
		P3RCTL = p3rctl_set.reg;
	}
	/* P4RCTL_REG:P4 Port Pull-up/down Control Register */
	{
		p4rctl_set.reg = 0;
		P4RCTL = p4rctl_set.reg;
	}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	/* P5RCTL_REG:P5 Port Pull-up/down Control Register */
	{
		p5rctl_set.reg = 0;
		P5RCTL = p5rctl_set.reg;
	}
#endif
	/* P6RCTL_REG:P6 Port Pull-up/down Control Register */
	{
		p6rctl_set.reg = 0;
		P6RCTL = p6rctl_set.reg;
	}
	/* P7RCTL_REG:P7 Port Pull-up/down Control Register */
	{
		p7rctl_set.reg = 0;
		P7RCTL = p7rctl_set.reg;
	}
	/* PDRCTL_REG:PD Port Pull-up/down Control Register */
	{
		pdrctl_set.reg = 0;
		PDRCTL = pdrctl_set.reg;
	}

	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = 0;
		P0MODSEL = p0modsel_set.reg;
	}
	/* P1MODSEL:P1 Port Mode Select Register */
	{
		p1modsel_set.reg = 0;
		P1MODSEL = p1modsel_set.reg;
	}
	/* P2MODSEL:P2 Port Mode Select Register */
	{
		p2modsel_set.reg = 0;
		P2MODSEL = p2modsel_set.reg;
	}
	/* P3MODSEL:P3 Port Mode Select Register */
	{
		p3modsel_set.reg = 0;
		P3MODSEL = p3modsel_set.reg;
	}
	/* P4MODSEL:P4 Port Mode Select Register */
	{
		p4modsel_set.reg = 0;
		P4MODSEL = p4modsel_set.reg;
	}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	/* P5MODSEL:P5 Port Mode Select Register */
	{
		p5modsel_set.reg = 0;
		P5MODSEL = p5modsel_set.reg;
	}
#endif
	/* P6MODSEL:P6 Port Mode Select Register */
	{
		p6modsel_set.reg = 0;
		P6MODSEL = p6modsel_set.reg;
	}
	/* P7MODSEL:P7 Port Mode Select Register */
	{
		p7modsel_set.reg = 0;
		P7MODSEL = p7modsel_set.reg;
	}
#if 0
	/* PDMODSEL:PD Port Mode Select Register */
	{
		pdmodsel_set.reg = 0;
		PDMODSEL = pdmodsel_set.reg;
	}
#endif
	/// Initialize the interrupt.
	ILVPPORT = INTERRUPT_LEVEL_PORT;

}

/************************************************************************//**
 * @brief   PORT input interrupt function.(Register in the vector table.)
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intPortInput(void)
{
	P0INTF_REG	p0nif_get;
	P1INTF_REG	p1nif_get;
	P2INTF_REG	p2nif_get;
	P3INTF_REG	p3nif_get;
	P4INTF_REG	p4nif_get;
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	P5INTF_REG	p5nif_get;
#endif
	P6INTF_REG	p6nif_get;
	P7INTF_REG	p7nif_get;

	/* P0INTF:P0 Port Interrupt Flag Register read & clear */
	{
		p0nif_get.reg = P0INTF;
		P0INTF = p0nif_get.reg;
	}
	intInputP0x(&p0nif_get);

	/* P1INTF:P1 Port Interrupt Flag Register read & clear */
	{
		p1nif_get.reg = P1INTF;
		P1INTF = p1nif_get.reg;
	}
	intInputP1x(&p1nif_get);

	/* P2INTF:P2 Port Interrupt Flag Register read & clear */
	{
		p2nif_get.reg = P2INTF;
		P2INTF = p2nif_get.reg;
	}
	intInputP2x(&p2nif_get);

	/* P3INTF:P3 Port Interrupt Flag Register read & clear */
	{
		p3nif_get.reg = P3INTF;
		P3INTF = p3nif_get.reg;
	}
	intInputP3x(&p3nif_get);

	/* P4INTF:P4 Port Interrupt Flag Register read & clear */
	{
		p4nif_get.reg = P4INTF;
		P4INTF = p4nif_get.reg;
	}
	intInputP4x(&p4nif_get);
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	/* P5INTF:P5 Port Interrupt Flag Register read & clear */
	{
		p5nif_get.reg = P5INTF;
		P5INTF = p5nif_get.reg;
	}
	intInputP5x(&p5nif_get);
#endif
	/* P6INTF:P6 Port Interrupt Flag Register read & clear */
	{
		p6nif_get.reg = P6INTF;
		P6INTF = p6nif_get.reg;
	}
	intInputP6x(&p6nif_get);

	/* P7INTF:P7 Port Interrupt Flag Register read & clear */
	{
		p7nif_get.reg = P7INTF;
		P7INTF = p7nif_get.reg;
	}
	intInputP7x(&p7nif_get);

}

static inline void intInputP0x(P0INTF_REG* p0nif_get)
{
	/* P0INTF:P0 Port Interrupt Flag Register */
	{
		if(p0nif_get->bit.P00IF == 1)
		{
		   /**
			* @todo add interrupt function of P00
			*/
			c17portP0xIntInputFlag |= 0x01;
		}
		if(p0nif_get->bit.P01IF == 1)
		{
		   /**
			* @todo add interrupt function of P01
			*/
			c17portP0xIntInputFlag |= 0x02;
		}
		if(p0nif_get->bit.P02IF == 1)
		{
		   /**
			* @todo add interrupt function of P02
			*/
			c17portP0xIntInputFlag |= 0x04;
		}
		if(p0nif_get->bit.P03IF == 1)
		{
		   /**
			* @todo add interrupt function of P03
			*/
			c17portP0xIntInputFlag |= 0x08;
		}
		if(p0nif_get->bit.P04IF == 1)
		{
		   /**
			* @todo add interrupt function of P04
			*/
			c17portP0xIntInputFlag |= 0x10;
		}
		if(p0nif_get->bit.P05IF == 1)
		{
		   /**
			* @todo add interrupt function of P05
			*/
			c17portP0xIntInputFlag |= 0x20;
		}
		if(p0nif_get->bit.P06IF == 1)
		{
		   /**
			* @todo add interrupt function of P06
			*/
			c17portP0xIntInputFlag |= 0x40;
		}
		if(p0nif_get->bit.P07IF == 1)
		{
		   /**
			* @todo add interrupt function of P07
			*/
			c17portP0xIntInputFlag |= 0x80;
		}
	}
}
static inline void intInputP1x(P1INTF_REG* p1nif_get)
{
	/* P1INTF:P1 Port Interrupt Flag Register */
	{
		if(p1nif_get->bit.P10IF == 1)
		{
		   /**
			* @todo add interrupt function of P10
			*/
			c17portP1xIntInputFlag |= 0x01;
		}
		if(p1nif_get->bit.P11IF == 1)
		{
		   /**
			* @todo add interrupt function of P11
			*/
			c17portP1xIntInputFlag |= 0x02;
		}
		if(p1nif_get->bit.P12IF == 1)
		{
		   /**
			* @todo add interrupt function of P12
			*/
			c17portP1xIntInputFlag |= 0x04;
		}
		if(p1nif_get->bit.P13IF == 1)
		{
		   /**
			* @todo add interrupt function of P13
			*/
			c17portP1xIntInputFlag |= 0x08;
		}
		if(p1nif_get->bit.P14IF == 1)
		{
		   /**
			* @todo add interrupt function of P14
			*/
			c17portP1xIntInputFlag |= 0x10;
		}
		if(p1nif_get->bit.P15IF == 1)
		{
		   /**
			* @todo add interrupt function of P15
			*/
			c17portP1xIntInputFlag |= 0x20;
		}
		if(p1nif_get->bit.P16IF == 1)
		{
		   /**
			* @todo add interrupt function of P16
			*/
			c17portP1xIntInputFlag |= 0x40;
		}
		if(p1nif_get->bit.P17IF == 1)
		{
		   /**
			* @todo add interrupt function of P17
			*/
			c17portP1xIntInputFlag |= 0x80;
		}
	}
}
static inline void intInputP2x(P2INTF_REG* p2nif_get)
{
	/* P2INTF:P2 Port Interrupt Flag Register */
	{
		if(p2nif_get->bit.P20IF == 1)
		{
		   /**
			* @todo add interrupt function of P20
			*/
			c17portP2xIntInputFlag |= 0x01;
		}
		if(p2nif_get->bit.P21IF == 1)
		{
		   /**
			* @todo add interrupt function of P21
			*/
			c17portP2xIntInputFlag |= 0x02;
		}
		if(p2nif_get->bit.P22IF == 1)
		{
		   /**
			* @todo add interrupt function of P22
			*/
			c17portP2xIntInputFlag |= 0x04;
		}
		if(p2nif_get->bit.P23IF == 1)
		{
		   /**
			* @todo add interrupt function of P23
			*/
			c17portP2xIntInputFlag |= 0x08;
		}
		if(p2nif_get->bit.P24IF == 1)
		{
		   /**
			* @todo add interrupt function of P24
			*/
			c17portP2xIntInputFlag |= 0x10;
		}
		if(p2nif_get->bit.P25IF == 1)
		{
		   /**
			* @todo add interrupt function of P25
			*/
			c17portP2xIntInputFlag |= 0x20;
		}
		if(p2nif_get->bit.P26IF == 1)
		{
		   /**
			* @todo add interrupt function of P26
			*/
			c17portP2xIntInputFlag |= 0x40;
		}
		if(p2nif_get->bit.P27IF == 1)
		{
		   /**
			* @todo add interrupt function of P27
			*/
			c17portP2xIntInputFlag |= 0x80;
		}

	}
}
static inline void intInputP3x(P3INTF_REG* p3nif_get)
{
	/* P3INTF:P3 Port Interrupt Flag Register */
	{
		if(p3nif_get->bit.P30IF == 1)
		{
		   /**
			* @todo add interrupt function of P30
			*/
			c17portP3xIntInputFlag |= 0x01;
		}
		if(p3nif_get->bit.P31IF == 1)
		{
		   /**
			* @todo add interrupt function of P31
			*/
			c17portP3xIntInputFlag |= 0x02;
		}
		if(p3nif_get->bit.P32IF == 1)
		{
		   /**
			* @todo add interrupt function of P32
			*/
			c17portP3xIntInputFlag |= 0x04;
		}
		if(p3nif_get->bit.P33IF == 1)
		{
		   /**
			* @todo add interrupt function of P33
			*/
			c17portP3xIntInputFlag |= 0x08;
		}
		if(p3nif_get->bit.P34IF == 1)
		{
		   /**
			* @todo add interrupt function of P34
			*/
			c17portP3xIntInputFlag |= 0x10;
		}
		if(p3nif_get->bit.P35IF == 1)
		{
		   /**
			* @todo add interrupt function of P35
			*/
			c17portP3xIntInputFlag |= 0x20;
		}
		if(p3nif_get->bit.P36IF == 1)
		{
		   /**
			* @todo add interrupt function of P36
			*/
			c17portP3xIntInputFlag |= 0x40;
		}
		if(p3nif_get->bit.P37IF == 1)
		{
		   /**
			* @todo add interrupt function of P37
			*/
			c17portP3xIntInputFlag |= 0x80;
		}

	}
}
static inline void intInputP4x(P4INTF_REG* p4nif_get)
{
	/* P4INTF:P4 Port Interrupt Flag Register */
	{
		if(p4nif_get->bit.P40IF == 1)
		{
		   /**
			* @todo add interrupt function of P40
			*/
			c17portP4xIntInputFlag |= 0x01;
		}
		if(p4nif_get->bit.P41IF == 1)
		{
		   /**
			* @todo add interrupt function of P41
			*/
			c17portP4xIntInputFlag |= 0x02;
		}
		if(p4nif_get->bit.P42IF == 1)
		{
		   /**
			* @todo add interrupt function of P42
			*/
			c17portP4xIntInputFlag |= 0x04;
		}
		if(p4nif_get->bit.P43IF == 1)
		{
		   /**
			* @todo add interrupt function of P43
			*/
			c17portP4xIntInputFlag |= 0x08;
		}
		if(p4nif_get->bit.P44IF == 1)
		{
		   /**
			* @todo add interrupt function of P44
			*/
			c17portP4xIntInputFlag |= 0x10;
		}
		if(p4nif_get->bit.P45IF == 1)
		{
		   /**
			* @todo add interrupt function of P45
			*/
			c17portP4xIntInputFlag |= 0x20;
		}
		if(p4nif_get->bit.P46IF == 1)
		{
		   /**
			* @todo add interrupt function of P46
			*/
			c17portP4xIntInputFlag |= 0x40;
		}
		if(p4nif_get->bit.P47IF == 1)
		{
		   /**
			* @todo add interrupt function of P47
			*/
			c17portP4xIntInputFlag |= 0x80;
		}
	}
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
static inline void intInputP5x(P5INTF_REG* p5nif_get)
{
	/* P5INTF:P5 Port Interrupt Flag Register */
	{
		if(p5nif_get->bit.P50IF == 1)
		{
		   /**
			* @todo add interrupt function of P50
			*/
			c17portP5xIntInputFlag |= 0x01;
		}
		if(p5nif_get->bit.P51IF == 1)
		{
		   /**
			* @todo add interrupt function of P51
			*/
			c17portP5xIntInputFlag |= 0x02;
		}
		if(p5nif_get->bit.P52IF == 1)
		{
		   /**
			* @todo add interrupt function of P52
			*/
			c17portP5xIntInputFlag |= 0x04;
		}
		if(p5nif_get->bit.P53IF == 1)
		{
		   /**
			* @todo add interrupt function of P53
			*/
			c17portP5xIntInputFlag |= 0x08;
		}
		if(p5nif_get->bit.P54IF == 1)
		{
		   /**
			* @todo add interrupt function of P54
			*/
			c17portP5xIntInputFlag |= 0x10;
		}
		if(p5nif_get->bit.P55IF == 1)
		{
		   /**
			* @todo add interrupt function of P55
			*/
			c17portP5xIntInputFlag |= 0x20;
		}
	}
}
#endif
static inline void intInputP6x(P6INTF_REG* p6nif_get)
{
	/* P6INTF:P6 Port Interrupt Flag Register */
	{
		if(p6nif_get->bit.P60IF == 1)
		{
		   /**
			* @todo add interrupt function of P60
			*/
			c17portP6xIntInputFlag |= 0x01;
		}
		if(p6nif_get->bit.P61IF == 1)
		{
		   /**
			* @todo add interrupt function of P61
			*/
			c17portP6xIntInputFlag |= 0x02;
		}
		if(p6nif_get->bit.P62IF == 1)
		{
		   /**
			* @todo add interrupt function of P62
			*/
			c17portP6xIntInputFlag |= 0x04;
		}
		if(p6nif_get->bit.P63IF == 1)
		{
		   /**
			* @todo add interrupt function of P63
			*/
			c17portP6xIntInputFlag |= 0x08;
		}
		if(p6nif_get->bit.P64IF == 1)
		{
		   /**
			* @todo add interrupt function of P64
			*/
			c17portP6xIntInputFlag |= 0x10;
		}
		if(p6nif_get->bit.P65IF == 1)
		{
		   /**
			* @todo add interrupt function of P65
			*/
			c17portP6xIntInputFlag |= 0x20;
		}
		if(p6nif_get->bit.P66IF == 1)
		{
		   /**
			* @todo add interrupt function of P66
			*/
			c17portP6xIntInputFlag |= 0x40;

		}
		if(p6nif_get->bit.P67IF == 1)
		{
		   /**
			* @todo add interrupt function of P67
			*/
			c17portP6xIntInputFlag |= 0x80;
		}
	}
}
static inline void intInputP7x(P7INTF_REG* p7nif_get)
{
	/* P7INTF:P7 Port Interrupt Flag Register */
	{
		if(p7nif_get->bit.P70IF == 1)
		{
		   /**
			* @todo add interrupt function of P70
			*/
			c17portP7xIntInputFlag |= 0x01;
		}
		if(p7nif_get->bit.P71IF == 1)
		{
		   /**
			* @todo add interrupt function of P71
			*/
			c17portP7xIntInputFlag |= 0x02;
		}
		if(p7nif_get->bit.P72IF == 1)
		{
		   /**
			* @todo add interrupt function of P72
			*/
			c17portP7xIntInputFlag |= 0x04;
		}
		if(p7nif_get->bit.P73IF == 1)
		{
		   /**
			* @todo add interrupt function of P73
			*/
			c17portP7xIntInputFlag |= 0x08;
		}
		if(p7nif_get->bit.P74IF == 1)
		{
		   /**
			* @todo add interrupt function of P74
			*/
			c17portP7xIntInputFlag |= 0x10;
		}
		if(p7nif_get->bit.P75IF == 1)
		{
		   /**
			* @todo add interrupt function of P75
			*/
			c17portP7xIntInputFlag |= 0x20;
		}
		if(p7nif_get->bit.P76IF == 1)
		{
		   /**
			* @todo add interrupt function of P76
			*/
			c17portP7xIntInputFlag |= 0x40;
		}
	}
}

/************************************************************************//**
 * @brief   Check if port interrupt occurred for each group.
 *
 *
 * @param   grpNo                         Group Number.
 *  @li @c      0 to 7                    Group Number.
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 *
 * @retval  0   port interrupt did not occur.
 * @retval  1   port interrupt occurred.
 ***************************************************************************/
int c17checkIntFlagGrp(unsigned short grpNo, unsigned short portNo)
{
	unsigned short mask = (unsigned short)1 << portNo;
	int Result = 0;
	switch(grpNo)
	{
	case C17_PORT_GRP_P0:
		Result = (c17portP0xIntInputFlag & mask) ? 1 : 0;
		break;
	case C17_PORT_GRP_P1:
		Result = (c17portP1xIntInputFlag & mask) ? 1 : 0;
		break;
	case C17_PORT_GRP_P2:
		Result = (c17portP2xIntInputFlag & mask) ? 1 : 0;
		break;
	case C17_PORT_GRP_P3:
		Result = (c17portP3xIntInputFlag & mask) ? 1 : 0;
		break;
	case C17_PORT_GRP_P4:
		Result = (c17portP4xIntInputFlag & mask) ? 1 : 0;
		break;
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	case C17_PORT_GRP_P5:
		Result = (c17portP5xIntInputFlag & mask) ? 1 : 0;
		break;
#endif
	case C17_PORT_GRP_P6:
		Result = (c17portP6xIntInputFlag & mask) ? 1 : 0;
		break;
	case C17_PORT_GRP_P7:
		Result = (c17portP7xIntInputFlag & mask) ? 1 : 0;
		break;
	}
	return Result;
}
/************************************************************************//**
 * @brief   Clear port interrupt flag for each group.
 *
 *
 * @param   grpNo                         Group Number.
 *  @li @c      0 to 7                    Group Number.
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17clrIntFlagGrp(unsigned short grpNo, unsigned short portNo)
{
	unsigned short mask = (unsigned short)1 << portNo;

	switch(grpNo)
	{
	case C17_PORT_GRP_P0:
		c17portP0xIntInputFlag &= ~mask;
		break;
	case C17_PORT_GRP_P1:
		c17portP1xIntInputFlag &= ~mask;
		break;
	case C17_PORT_GRP_P2:
		c17portP2xIntInputFlag &= ~mask;
		break;
	case C17_PORT_GRP_P3:
		c17portP3xIntInputFlag &= ~mask;
		break;
	case C17_PORT_GRP_P4:
		c17portP4xIntInputFlag &= ~mask;
		break;
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	case C17_PORT_GRP_P5:
		c17portP5xIntInputFlag &= ~mask;
		break;
#endif
	case C17_PORT_GRP_P6:
		c17portP6xIntInputFlag &= ~mask;
		break;
	case C17_PORT_GRP_P7:
		c17portP7xIntInputFlag &= ~mask;
		break;
	}
}
/************************************************************************//**
 * @brief   Set port interrupt enable for each group.
 *
 *
 * @param   grpNo                         Group Number.
 *  @li @c      0 to 7                    Group Number.
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntPortGrp(unsigned short grpNo, unsigned short portNo, unsigned short portIntEdge)
{
	switch(grpNo)
	{
	case C17_PORT_GRP_P0:
		c17enableIntP0x(portNo, portIntEdge);
		break;
	case C17_PORT_GRP_P1:
		c17enableIntP1x(portNo, portIntEdge);
		break;
	case C17_PORT_GRP_P2:
		c17enableIntP2x(portNo, portIntEdge);
		break;
	case C17_PORT_GRP_P3:
		c17enableIntP3x(portNo, portIntEdge);
		break;
	case C17_PORT_GRP_P4:
		c17enableIntP4x(portNo, portIntEdge);
		break;
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	case C17_PORT_GRP_P5:
		c17enableIntP5x(portNo, portIntEdge);
		break;
#endif
	case C17_PORT_GRP_P6:
		c17enableIntP6x(portNo, portIntEdge);
		break;
	case C17_PORT_GRP_P7:
		c17enableIntP6x(portNo, portIntEdge);
		break;
	}
}

/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP0x(unsigned short portNo, unsigned short portIntEdge)
{
	P0INTF_REG      p0intf_set;

	if(portNo > 7) {
		return;
	}

	/* P0INTF: Port Interrupt Flag Register */
	{
		p0intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P0INTF = p0intf_set.reg;
	}

	/* P0INTCTL:P0 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P0INTCTL);
	}
}



/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP1x(unsigned short portNo, unsigned short portIntEdge)
{
	P1INTF_REG      p1intf_set;

	if(portNo > 7) {
		return;
	}

	/* P1INTF: Port Interrupt Flag Register */
	{
		p1intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P1INTF = p1intf_set.reg;
	}

	/* P1INTCTL:P1 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P1INTCTL);
	}
}
/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP2x(unsigned short portNo, unsigned short portIntEdge)
{
	P2INTF_REG      p2intf_set;

	if(portNo > 7) {
		return;
	}

	/* P2INTF: Port Interrupt Flag Register */
	{
		p2intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P2INTF = p2intf_set.reg;
	}

	/* P2INTCTL:P2 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P2INTCTL);
	}
}
/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP3x(unsigned short portNo, unsigned short portIntEdge)
{
	P3INTF_REG      p3intf_set;

	if(portNo > 7) {
		return;
	}

	/* P3INTF: Port Interrupt Flag Register */
	{
		p3intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P3INTF = p3intf_set.reg;
	}

	/* P3INTCTL:P3 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P3INTCTL);
	}
}
/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP4x(unsigned short portNo, unsigned short portIntEdge)
{
	P4INTF_REG      p4intf_set;

	if(portNo > 7) {
		return;
	}

	/* P4INTF: Port Interrupt Flag Register */
	{
		p4intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P4INTF = p4intf_set.reg;
	}

	/* P4INTCTL:P4 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P4INTCTL);
	}
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP5x(unsigned short portNo, unsigned short portIntEdge)
{
	P5INTF_REG      p5intf_set;

	if(portNo > 7) {
		return;
	}

	/* P5INTF: Port Interrupt Flag Register */
	{
		p5intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P5INTF = p5intf_set.reg;
	}

	/* P5INTCTL:P5 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P5INTCTL);
	}
}
#endif
/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP6x(unsigned short portNo, unsigned short portIntEdge)
{
	P6INTF_REG      p6intf_set;

	if(portNo > 7) {
		return;
	}

	/* P6INTF: Port Interrupt Flag Register */
	{
		p6intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P6INTF = p6intf_set.reg;
	}

	/* P6INTCTL:P6 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P6INTCTL);
	}
}
/************************************************************************//**
 * @brief   Set port interrupt enable.
 *
 *
 * @param   portNo                        Port Number.
 *  @li @c      0 to 7                    Port Number.
 *  @li @c      others                    Nothing is done.
 * @param   portIntEdge                   Interrupt edge.
 *  @li @c      C17_PORT_INT_EDGE_RISING  Rising edge.
 *  @li @c      C17_PORT_INT_EDGE_FALLING Falling edge.
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntP7x(unsigned short portNo, unsigned short portIntEdge)
{
	P7INTF_REG      p7intf_set;

	if(portNo > 7) {
		return;
	}

	/* P7INTF: Port Interrupt Flag Register */
	{
		p7intf_set.reg = LOWBYTE_BIT_SET(portNo);
		P7INTF = p7intf_set.reg;
	}

	/* P7INTCTL:P7 Port Interrupt Control Register. */
	{
		enableIntPxx(portNo, portIntEdge, (P0INTCTL_REG*)&P7INTCTL);
	}
}

static void enableIntPxx(unsigned short portNo, unsigned short portIntEdge, P0INTCTL_REG* portRegister)
{
	P0INTCTL_REG	pxintctl_set;

	pxintctl_set.reg = portRegister->reg;

	if(portIntEdge == C17_PORT_INT_EDGE_FALLING) {
		/**
		 * Set PxEDGE
		 * Interrupt will occur at a falling edge.
		 */
		pxintctl_set.reg |= HIGHBYTE_BIT_SET(portNo) ;
	} else {
		/**
		 *  Clear PxEDGE
		 * Interrupt will occur at a rising edge.
		 */
		pxintctl_set.reg &= (0xFFFF ^ HIGHBYTE_BIT_SET(portNo));
	}
	/**
	 * Set PxIE
	 * Enable port input interrupts.
	 */
	pxintctl_set.reg |=  LOWBYTE_BIT_SET(portNo) ;

	portRegister->reg = pxintctl_set.reg;
}
/************************************************************************//**
 * @brief   Set port interrupt disable for each group.
 *
 *
 * @param   grpNo           Group Number.
 *  @li @c      0 to 7      Group Number
 *  @li @c      others      Nothing is done.
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntPortGrp(unsigned short grpNo, unsigned short portNo)
{
	switch(grpNo)
	{
	case C17_PORT_GRP_P0:
		c17disableIntP0x(portNo);
		break;
	case C17_PORT_GRP_P1:
		c17disableIntP1x(portNo);
		break;
	case C17_PORT_GRP_P2:
		c17disableIntP2x(portNo);
		break;
	case C17_PORT_GRP_P3:
		c17disableIntP3x(portNo);
		break;
	case C17_PORT_GRP_P4:
		c17disableIntP4x(portNo);
		break;
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	case C17_PORT_GRP_P5:
		c17disableIntP5x(portNo);
		break;
#endif
	case C17_PORT_GRP_P6:
		c17disableIntP6x(portNo);
		break;
	case C17_PORT_GRP_P7:
		c17disableIntP7x(portNo);
		break;
	}
}

/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP0x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P0INTCTL:P0 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P0INTCTL);
	}
}

/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP1x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P1INTCTL:P1 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P1INTCTL);
	}
}

/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP2x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P2INTCTL:P2 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P2INTCTL);
	}
}
/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP3x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P3INTCTL:P3 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P3INTCTL);
	}
}
/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP4x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P4INTCTL:P4 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P4INTCTL);
	}
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP5x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P5INTCTL:P5 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P5INTCTL);
	}
}
#endif
/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP6x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P6INTCTL:P6 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P6INTCTL);
	}
}
/************************************************************************//**
 * @brief   Set port interrupt disable.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntP7x(unsigned short portNo)
{
	if(portNo > 7) {
		return;
	}

	/* P7INTCTL:P7 Port Interrupt Control Register. */
	{
		// Disable interrupt.
		disableIntPxx(portNo, (P0INTCTL_REG*)&P7INTCTL);
	}
}

static void disableIntPxx(unsigned short portNo, P0INTCTL_REG* portRegister)
{
	P0INTCTL_REG	pxintctl_set;

	pxintctl_set.reg = portRegister->reg;
	/**
	 * Set PxIE
	 * Disable port input interrupts.
	 */
	pxintctl_set.reg &= (0xFFFF ^ LOWBYTE_BIT_SET(portNo)) ;

	portRegister->reg = pxintctl_set.reg;
}

/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP0xByte(void)
{
	// Get output data.
	return P0DAT_P0OUT;
}
/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP1xByte(void)
{
	// Get output data.
	return P1DAT_P1OUT;
}
/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP2xByte(void)
{
	// Get output data.
	return P2DAT_P2OUT;
}
/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP3xByte(void)
{
	// Get output data.
	return P3DAT_P3OUT;
}
/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP4xByte(void)
{
	// Get output data.
	return P4DAT_P4OUT;
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP5xByte(void)
{
	// Get output data.
	return P5DAT_P5OUT;
}
#endif
/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP6xByte(void)
{
	// Get output data.
	return P6DAT_P6OUT;
}
/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputP7xByte(void)
{
	// Get output data.
	return P7DAT_P7OUT;
}
/************************************************************************//**
 * @brief   Get output port data.(BYTE)
 *
 *
 *  @retval     0 to 0xFF
 ***************************************************************************/
unsigned char c17getOutputPDxByte(void)
{
	// Get output data.
	return PDDAT_PDOUT;
}
/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP0x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P0DAT: P0 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P0DAT);
	}
}
/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP1x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P1DAT: P1 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P1DAT);
	}
}
/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP2x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P2DAT: P2 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P2DAT);
	}
}
/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP3x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P3DAT: P3 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P3DAT);
	}
}
/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP4x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P4DAT: P4 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P4DAT);
	}
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP5x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P5DAT: P5 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P5DAT);
	}
}
#endif
/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP6x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P6DAT: P6 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P6DAT);
	}
}
/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputP7x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P7DAT: P7 Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&P7DAT);
	}
}
/************************************************************************//**
 * @brief   Get output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getOutputPDx(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*PDDAT: PD Port Data Register */
	{
		// Get output data.
		return getOutputPxx(portNo, (P0DAT_REG*)&PDDAT);
	}
}


static unsigned char getOutputPxx(unsigned short portNo, P0DAT_REG* portRegister)
{
	// Get output data.
	unsigned char result = 0;

	/**
	 * GET PxOUT
     *    1 : Output high level from the port pin
     *    0 : Output low level from the port pin
	 *
	 */
	if((portRegister->reg & HIGHBYTE_BIT_SET(portNo)) != 0)
	{
		result = 1;
	}
	return result;
}


/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP0xByte(unsigned char portData)
{
	/* P0DAT :P0 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P0DAT);
	}
}


/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP1xByte(unsigned char portData)
{
	/* P1DAT :P1 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P1DAT);
	}
}
/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP2xByte(unsigned char portData)
{
	/* P2DAT :P2 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P2DAT);
	}
}
/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP3xByte(unsigned char portData)
{
	/* P3DAT :P3 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P3DAT);
	}
}
/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP4xByte(unsigned char portData)
{
	/* P4DAT :P4 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P4DAT);
	}
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP5xByte(unsigned char portData)
{
	/* P5DAT :P5 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P5DAT);
	}

}
#endif
/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP6xByte(unsigned char portData)
{
	/* P6DAT :P6 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P6DAT);
	}

}
/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP7xByte(unsigned char portData)
{
	/* P7DAT :P7 Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&P7DAT);
	}

}
/************************************************************************//**
 * @brief   Set output port data.(BYTE)
 *
 *
 * @param   portData        output data.
 *  @li @c  0 to 0xFF
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputPDxByte(unsigned char portData)
{
	/* PDDAT :PD Port Data Register */
	{
		setOutputPxxByte(portData, (P0DAT_REG*)&PDDAT);
	}

}

static void setOutputPxxByte(unsigned char portData, P0DAT_REG* portRegister)
{
	P0DAT_REG	pxdata_set;

	/* PxDAT :Px Port Data Register */
	{
		pxdata_set.reg = portRegister->reg;
		pxdata_set.byte_reg[1] = portData;
		// Set output data.
		portRegister->reg = pxdata_set.reg;
	}
}
/************************************************************************//**
 * @brief   Set output port data for each group.
 *
 *
 * @param   grpNo           Group Number.
 *  @li @c      0 to 7      Group Number
 *  @li @c      others      Nothing is done.
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   dportDataata    output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputPortGrp(unsigned short grpNo, unsigned short portNo, unsigned short portData)
{
	switch(grpNo)
	{
	case C17_PORT_GRP_P0:
		c17setOutputP0x(portNo, portData);
		break;
	case C17_PORT_GRP_P1:
		c17setOutputP1x(portNo, portData);
		break;
	case C17_PORT_GRP_P2:
		c17setOutputP2x(portNo, portData);
		break;
	case C17_PORT_GRP_P3:
		c17setOutputP3x(portNo, portData);
		break;
	case C17_PORT_GRP_P4:
		c17setOutputP4x(portNo, portData);
		break;
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	case C17_PORT_GRP_P5:
		c17setOutputP5x(portNo, portData);
		break;
#endif
	case C17_PORT_GRP_P6:
		c17setOutputP6x(portNo, portData);
		break;
	case C17_PORT_GRP_P7:
		c17setOutputP7x(portNo, portData);
		break;
	}
}
/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   dportDataata    output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP0x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P0DAT: P0 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P0DAT);
	}
}
/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP1x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P1DAT: P1 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P1DAT);
	}
}
/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP2x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P2DAT: P2 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P2DAT);
	}
}
/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP3x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P3DAT: P3 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P3DAT);
	}
}
/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP4x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P4DAT: P4 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P4DAT);
	}
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP5x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P5DAT: P5 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P5DAT);
	}
}
#endif
/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP6x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P6DAT: P6 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P6DAT);
	}
}
/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputP7x(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*P7DAT: P7 Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&P7DAT);
	}
}
/************************************************************************//**
 * @brief   Set output port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 * @param   portData        output data.
 *  @li @c      0 or 1      LOW or HIGH.
 *
 *  @retval     none
 ***************************************************************************/
void c17setOutputPDx(unsigned short portNo, unsigned short portData)
{
	if(portNo > 7) {
		return ;
	}
	/*PDDAT: PD Port Data Register.*/
	{
		setOutputPxx(portNo, portData, (P0DAT_REG*)&PDDAT);
	}
}

static void setOutputPxx(unsigned short portNo, unsigned short portData, P0DAT_REG* portRegister)
{
	P0DAT_REG	pxdat_set;

	pxdat_set.reg =  portRegister->reg;

	if(portData == 1) {
		/**
		 * Set PxOUT
		 * Output high level from the port pin.
		 */
		pxdat_set.reg |= HIGHBYTE_BIT_SET(portNo) ;
	} else {
		/**
		 *  Clear PxOUT
		 * Output low level from the port pin.
		 */
		pxdat_set.reg &= (0xFFFF ^ HIGHBYTE_BIT_SET(portNo));
	}

	portRegister->reg = pxdat_set.reg;
}

/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP0xByte(void)
{
	// Get input data.
	return P0DAT_P0IN;
}
/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP1xByte(void)
{
	// Get input data.
	return P1DAT_P1IN;
}
/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP2xByte(void)
{
	// Get input data.
	return P2DAT_P2IN;
}
/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP3xByte(void)
{
	// Get input data.
	return P3DAT_P3IN;
}
/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP4xByte(void)
{
	// Get input data.
	return P4DAT_P4IN;
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP5xByte(void)
{
	// Get input data.
	return P5DAT_P5IN;
}
#endif
/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP6xByte(void)
{
	// Get input data.
	return P6DAT_P6IN;
}
/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputP7xByte(void)
{
	// Get input data.
	return P7DAT_P7IN;
}
/************************************************************************//**
 * @brief   Get input port data.(BYTE)
 *
 *
 *
 *  @retval     0 to 0xFF       input data.
 ***************************************************************************/
unsigned char c17getInputPDxByte(void)
{
	// Get input data.
	return PDDAT_PDIN;
}
/************************************************************************//**
 * @brief   Get input port data for each group.
 *
 *
 * @param   grpNo           Group Number.
 *  @li @c      0 to 7      Group Number
 *  @li @c      others      Nothing is done.
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputPortGrp(unsigned short grpNo, unsigned short portNo)
{
	switch(grpNo)
	{
	case C17_PORT_GRP_P0:
		return c17getInputP0x(portNo);
	case C17_PORT_GRP_P1:
		return c17getInputP1x(portNo);
	case C17_PORT_GRP_P2:
		return c17getInputP2x(portNo);
	case C17_PORT_GRP_P3:
		return c17getInputP3x(portNo);
	case C17_PORT_GRP_P4:
		return c17getInputP4x(portNo);
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	case C17_PORT_GRP_P5:
		return c17getInputP5x(portNo);
#endif
	case C17_PORT_GRP_P6:
		return c17getInputP6x(portNo);
	case C17_PORT_GRP_P7:
		return c17getInputP7x(portNo);
	}
	return 0;
}
/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP0x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P0DAT: P0 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P0DAT);
	}
}
/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP1x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P1DAT: P1 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P1DAT);
	}
}
/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP2x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P2DAT: P2 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P2DAT);
	}
}
/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP3x(unsigned short portNo)
{
	if(portNo > 7) {
		return 0;
	}

	/*P3DAT: P3 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P3DAT);
	}
}
/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP4x(unsigned short portNo)
{
	if (portNo > 7) {
		return 0;
	}

	/*P4DAT: P4 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P4DAT);
	}
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP5x(unsigned short portNo)
{
	if (portNo > 7) {
		return 0;
	}

	/*P5DAT: P5 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P5DAT);
	}
}
#endif
/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP6x(unsigned short portNo)
{
	if (portNo > 7) {
		return 0;
	}

	/*P6DAT: P6 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P6DAT);
	}
}

/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputP7x(unsigned short portNo)
{
	if (portNo > 7) {
		return 0;
	}

	/*P7DAT: P7 Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&P7DAT);
	}
}

/************************************************************************//**
 * @brief   Get input port data.
 *
 *
 * @param   portNo          Port Number.
 *  @li @c      0 to 7      Port Number.
 *  @li @c      others      Nothing is done.
 *
 *  @retval     0           Low.
 *  @retval     1           High.
 ***************************************************************************/
unsigned char c17getInputPDx(unsigned short portNo)
{
	if (portNo > 7) {
		return 0;
	}

	/*PDDAT: PD Port Data Register */
	{
		// Get output data.
		return getInputPxx(portNo, (P0DAT_REG*)&PDDAT);
	}
}

unsigned char getInputPxx(unsigned short portNo, P0DAT_REG* portRegister)
{
	// Get intput data.
	unsigned char result = 0;

	/**
	 * GET PxIN
     *    1 : Output high level from the port pin
     *    0 : Output low level from the port pin
	 *
	 */
	if((portRegister->reg & LOWBYTE_BIT_SET(portNo)) != 0)
	{
		result = 1;
	}

	return result;
}

/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control for each group
 *
 *
 * @param   grpNo                       Group Number.
 *  @li @c      0 to 7                  Group Number
 *  @li @c      others                  Nothing is done.
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionPortGrp(unsigned short grpNo, unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	switch(grpNo)
	{
	case C17_PORT_GRP_P0:
		c17setDirectionP0x(portNo, portDir, portChatta, portPull);
		break;
	case C17_PORT_GRP_P1:
		c17setDirectionP1x(portNo, portDir, portChatta, portPull);
		break;
	case C17_PORT_GRP_P2:
		c17setDirectionP2x(portNo, portDir, portChatta, portPull);
		break;
	case C17_PORT_GRP_P3:
		c17setDirectionP3x(portNo, portDir, portChatta, portPull);
		break;
	case C17_PORT_GRP_P4:
		c17setDirectionP4x(portNo, portDir, portChatta, portPull);
		break;
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
	case C17_PORT_GRP_P5:
		c17setDirectionP5x(portNo, portDir, portChatta, portPull);
		break;
#endif
	case C17_PORT_GRP_P6:
		c17setDirectionP6x(portNo, portDir, portChatta, portPull);
		break;
	case C17_PORT_GRP_P7:
		c17setDirectionP7x(portNo, portDir, portChatta, portPull);
		break;
	}
}
/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP0x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}

	/* P0IOEN:P0 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P0IOEN);
	}
	/* P0CHATEN: P0 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P0CHATEN);
	}
	/* P0RCTL_REG:P0 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P0RCTL);
	}
}

/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP1x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}

	/* P1IOEN:P1 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P1IOEN);
	}
	/* P1CHATEN: P1 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P1CHATEN);
	}
	/* P1RCTL_REG:P1 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P1RCTL);
	}
}
/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP2x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}
	/* P2IOEN:P2 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P2IOEN);
	}
	/* P2CHATEN: P2 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P2CHATEN);
	}
	/* P2RCTL_REG:P2 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P2RCTL);
	}
}
/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP3x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}
	/* P3IOEN:P3 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P3IOEN);
	}
	/* P3CHATEN: P3 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P3CHATEN);
	}
	/* P3RCTL_REG:P3 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P3RCTL);
	}
}
/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP4x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}
	/* P4IOEN:P4 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P4IOEN);
	}
	/* P4CHATEN: P4 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P4CHATEN);
	}
	/* P4RCTL_REG:P4 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P4RCTL);
	}
}
#if defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33)
/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP5x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}
	/* P5IOEN:P5 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P5IOEN);
	}
	/* P5CHATEN: P5 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P5CHATEN);
	}
	/* P5RCTL_REG:P5 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P5RCTL);
	}
}
#endif
/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP6x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}
	/* P6IOEN:P6 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P6IOEN);
	}
	/* P6CHATEN: P6 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P6CHATEN);
	}
	/* P6RCTL_REG:P6 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P6RCTL);
	}
}
/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionP7x(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}
	/* P7IOEN:P7 Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&P7IOEN);
	}
	/* P7CHATEN: P7 Port Chattering Filter Enable Register */
	{
		setChattaPxx(portNo, portChatta, (P0CHATEN_REG*)&P7CHATEN);
	}
	/* P7RCTL_REG:P7 Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&P7RCTL);
	}
}
/************************************************************************//**
 * @brief   Set port direction and Chattering Filter and PULL UP/PULL DOWN Control
 *
 *
 * @param   portNo                      Port Number.
 *  @li @c      0 to 7                  Port Number.
 *  @li @c      others                  Nothing is done.
 * @param   portDir                     GPIO port input / output
 *  @li @c      C17_PORT_DIR_INPUT      Input.
 *  @li @c      C17_PORT_DIR_OUTPUT     Output.
 * @param   portChatta                  GPIO  Chattering Filter Enable/Disable
 *  @li @c      0                       Disable.
 *  @li @c      1                       Enable.
 * @param   portPull                    GPIO  PULL UP/PULL DOWN Control
 *  @li @c      C17_PORT_PULL_DISABLE   Disable.
 *  @li @c      C17_PORT_PULL_UP        PULL UP.
 *  @li @c      C17_PORT_PULL_DOWN      PULL DOWN.
 *
 *  @retval     none
 ***************************************************************************/
void c17setDirectionPDx(unsigned short portNo, unsigned short portDir, unsigned short portChatta, unsigned short portPull)
{
	// Set Port Direction
	if (portNo > 7) {
		return ;
	}

	/* PDIOEN:PD Port Enable Register */
	{
		setDirectionPxx(portNo, portDir, (P0IOEN_REG*)&PDIOEN);
	}

	/* PDRCTL_REG:PD Port Pull-up/down Control Register */
	{
		setPullUpDownPxx(portNo, portPull, (P0RCTL_REG*)&PDRCTL);
	}
}


static void setDirectionPxx(unsigned short portNo, unsigned short portDir, P0IOEN_REG* portRegister)
{
	P0IOEN_REG	pxioen_set;

	pxioen_set.reg =  portRegister->reg;

	if(portDir == C17_PORT_DIR_OUTPUT){
		/**
		 * Set PxOEN
         *    Enable (Data is output from the port pin.)
    	 */
		pxioen_set.reg |= LOWBYTE_BIT_SET(portNo) ;

		/**
		 * Clear PxIEN
         *    0 (R/W): Disable (Input data is fixed at 0.)
		 *
		 */
		pxioen_set.reg &= (0xFFFF ^ HIGHBYTE_BIT_SET(portNo)) ;
	} else if (portDir == C17_PORT_DIR_INPUT){
		/**
		 * Claer PxOEN
         *     Disable (The port is placed into Hi-Z.)
    	 */
		pxioen_set.reg &= (0xFFFF ^ LOWBYTE_BIT_SET(portNo)) ;

		/**
		 *  Set PxIEN
         *    Enable (The port pin status is input.)
		 *
		 */
		pxioen_set.reg |= HIGHBYTE_BIT_SET(portNo) ;
	} else if (portDir == C17_PORT_DIR_DISABLE) {
		/**
		 * Claer PxOEN
		 *     Disable (The port is placed into Hi-Z.)
		 */
		pxioen_set.reg &= (0xFFFF ^ LOWBYTE_BIT_SET(portNo)) ;

		/**
		 * Clear PxIEN
		 *    0 (R/W): Disable (Input data is fixed at 0.)
		 *
		 */
		pxioen_set.reg &= (0xFFFF ^ HIGHBYTE_BIT_SET(portNo)) ;

	}
	portRegister->reg = pxioen_set.reg;

}
static void setChattaPxx(unsigned short portNo, unsigned short portChatta, P0CHATEN_REG* portRegister)
{
	P0CHATEN_REG	pxchaten_set;

	pxchaten_set.reg =  portRegister->reg;

	if (portChatta == 1) {
		/**
		 * Set PxCHATEN
		 * Enable (The chattering filter is used.)
		 */
		pxchaten_set.reg |= LOWBYTE_BIT_SET(portNo) ;
	} else {
		/**
		 * Clear PxCHATEN
		 * Disable (The chattering filter is bypassed.)
		 */
		pxchaten_set.reg &= (0xFFFF ^ LOWBYTE_BIT_SET(portNo));
	}
	portRegister->reg = pxchaten_set.reg;
}

static void setPullUpDownPxx(unsigned short portNo, unsigned short portPull, P0RCTL_REG* portRegister)
{
	P0RCTL_REG	pxrctl_set;

	pxrctl_set.reg =  portRegister->reg;
	switch(portPull)
	{
		case C17_PORT_PULL_DISABLE:
			/**
			 * Clear PxREN
			 * Disable (No pull-up/down control is performed.)
			 *
			 */
			pxrctl_set.reg &=  (0xFFFF ^ LOWBYTE_BIT_SET(portNo));
			break;
		case C17_PORT_PULL_UP:
			/**
			 * Set PxREN
			 * Enable (The built-in pull-up/down resistor is used.)
			 *
			 */
			pxrctl_set.reg |=  LOWBYTE_BIT_SET(portNo);

			/*
			 * Set PxPDPU
			 * Pull-up resistor
			 */
			pxrctl_set.reg |=  HIGHBYTE_BIT_SET(portNo);
			break;
		case C17_PORT_PULL_DOWN:
			/**
			 * Set PxREN
			 * Enable (The built-in pull-up/down resistor is used.)
			 *
			 */
			pxrctl_set.reg |=  LOWBYTE_BIT_SET(portNo);

			/**
			 * Clear PxPDPU
			 * Pull-down resistor
			 */
			pxrctl_set.reg &=  (0xFFFF ^ HIGHBYTE_BIT_SET(portNo));
			break;
	}
	portRegister->reg = pxrctl_set.reg;
}

#endif
