/************************************************************************//**
 *
 * @file    c17_remc3_1.c
 *
 * @brief   This is REMC3 driver program.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2016-2016. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include "c17_remc3_1.h"

#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_init_config.h"
#include "c17_port_1.h"

#if defined(C17_PE_REMC)
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector18_handler, c17intRemc);	// REMC interrupt.
#endif /* CRT0_LINKED */


/* --- macro definition --- */
/// NEC format
/// 1T=562us
#define CRPER_NEC			(REMC_CLOCK * 1000000 / 38000 - 1)	//38KHz	  REMCCLK /38000000 -1
#define LEADER_APLEN_NEC	(562 * REMC_CLOCK * 16 - 1)			//16T	  0.000562(1T) * 16 * REMCCLK -1
#define LEADER_DBLEN_NEC    (562 * REMC_CLOCK * 24 - 1)			//16T+8T  0.000562(1T) * 24 * REMCCLK -1
#define BIT_0_APLEN_NEC		(562 * REMC_CLOCK * 1 - 1)			//1T      0.000562(1T) *  1 * REMCCLK -1
#define BIT_0_DBLEN_NEC	 	(562 * REMC_CLOCK * 2 - 1)			//1T+1T   0.000562(1T) *  2 * REMCCLK -1
#define BIT_1_APLEN_NEC		(562 * REMC_CLOCK * 1 - 1)			//1T      0.000562(1T) *  1 * REMCCLK -1
#define BIT_1_DBLEN_NEC	 	(562 * REMC_CLOCK * 4 - 1)			//1T+3T   0.000562(1T) *  4 * REMCCLK -1

// AEHA format
// 1T=425us
#define CRPER_AEHA			(REMC_CLOCK * 1000000 / 38000 - 1)	//38KHz	  REMCCLK /38000000 -1
#define LEADER_APLEN_AEHA	(425 * REMC_CLOCK * 8 - 1)			//8T      0.000425(1T) *  8 * REMCCLK -1
#define LEADER_DBLEN_AEHA   (425 * REMC_CLOCK * 12 - 1)			//8T+4T   0.000425(1T) * 12 * REMCCLK -1
#define BIT_0_APLEN_AEHA	(425 * REMC_CLOCK * 1 - 1)			//1T      0.000425(1T) *  1 * REMCCLK -1
#define BIT_0_DBLEN_AEHA	(425 * REMC_CLOCK * 2 - 1)			//1T+1T   0.000425(1T) *  2 * REMCCLK -1
#define BIT_1_APLEN_AEHA	(425 * REMC_CLOCK * 1 - 1)			//1T      0.000425(1T) *  1 * REMCCLK -1
#define BIT_1_DBLEN_AEHA	(425 * REMC_CLOCK * 4 - 1)			//1T+3T   0.000425(1T) *  4 * REMCCLK -1

// SONY format
// 1T=600us
#define CRPER_SONY			(REMC_CLOCK * 1000000 / 40000 - 1)	//40KHz	  REMCCLK /40000000 -1
#define LEADER_APLEN_SONY	(0)									//0
#define LEADER_DBLEN_SONY   (600 * REMC_CLOCK * 4 - 1)			//4T      0.000600(1T) * 4 * REMCCLK -1
#define BIT_0_APLEN_SONY	(600 * REMC_CLOCK * 1 - 1)			//1T      0.000600(1T) * 1 * REMCCLK -1
#define BIT_0_DBLEN_SONY	(600 * REMC_CLOCK * 2 - 1)			//1T+1T   0.000600(1T) * 2 * REMCCLK -1
#define BIT_1_APLEN_SONY	(600 * REMC_CLOCK * 1 - 1)			//1T      0.000600(1T) * 1 * REMCCLK -1
#define BIT_1_DBLEN_SONY	(600 * REMC_CLOCK * 3 - 1)			//1T+2T   0.000600(1T) * 3 * REMCCLK -1

/// REMC interrupt Status
enum
{
	STATUS_NONE,
	STATUS_DATA,
	STATUS_TR,
	STATUS_END
};

typedef struct st_remc_s {
	unsigned short	 Status;
	unsigned short	 bitCount;
	unsigned short sonybitCount;
	unsigned short sendEndFlg;
	unsigned short	 sendLength;
	unsigned char*	 sendData;
	unsigned char	 data;
} ST_REMC_S;

typedef struct st_remc_wave {
	unsigned short leader_aplen;		/// Leader APLEN
	unsigned short leader_dblen;		/// Leader DBLEN
	unsigned short bit1_aplen;			/// Data = 1 APLEN
	unsigned short bit1_dblen;			/// Data = 1 DBLEN
	unsigned short bit0_aplen;			/// Data = 0 APLEN
	unsigned short bit0_dblen;			/// Data = 0 DBLEN
} ST_REMC_WAVE;

/* --- static --- */
static unsigned char  send_format;
static ST_REMC_S		remc_send;
static ST_REMC_WAVE		remc_wave;

static void necFormatInt(void);
static void sonyFormatInt(void);

/************************************************************************//**
 * @brief   Initialize Remc.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17initRemc(void)
{
	REMCLK_REG		remclk_set;
	REMDBCTL_REG	remdbctl_set;

	P0MODSEL_REG	p0modsel_set;
	P0FNCSEL_REG	p0fncsel_set;

	// REMCRST=1  Reset REMC.
	/* REMDBCTL:REMC2 Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.REMCRST = 1;
		REMDBCTL = remdbctl_set.reg;
	}

	/* REMCLK:REMC Clock Control Register */
	{
		remclk_set.bit.DBRUN = 0;
		remclk_set.bit.CLKDIV = (REMC_SET_INIT & 0x0F);
		remclk_set.bit.CLKSRC = (REMC_SET_INIT & 0xC0)>>6;
		REMCLK = remclk_set.reg;
	}

	/* REMDBCTL:REMC Data Bit Counter Control Register. */
	{
		remdbctl_set.bit.PRESET = 0;
		remdbctl_set.bit.PRUN =0;
		remdbctl_set.bit.REMOINV = 0;
		remdbctl_set.bit.BUFEN = 0;
		remdbctl_set.bit.TRMD = (REMC_SET_INIT & 0x20)>>5;
		remdbctl_set.bit.REMCRST = 0;
		remdbctl_set.bit.MODEN = 1;
		REMDBCTL = remdbctl_set.reg;
	}

	/// Initialize the REMC output function to the ports.
	/*
	 *  P07 : REMO
	 */
	/// Set Universal Port Multiplexer (UPMUX).
	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P07SEL = 1;		/// - P07 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}
	/* P0FNCSEL: P0 Port Function Select Register */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P07MUX = 0;		/// - P07 : REMO.
		P0FNCSEL = p0fncsel_set.reg;
	}

	/// Initialize the interrupt.
	ILVREMC_0 = INTERRUPT_LEVEL_REMC;
}


/************************************************************************//**
 * @brief   Remc enable.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableRemc(void)
{
	REMDBCTL_REG	remdbctl_set;

	/* REMDBCTL:REMC Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.MODEN = 1;
		REMDBCTL = remdbctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Remc disable.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableRemc(void)
{
	REMDBCTL_REG	remdbctl_set;

	/* REMDBCTL:REMC Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.PRUN = 0;
		REMDBCTL = remdbctl_set.reg;

		remdbctl_set.bit.MODEN = 0;
		REMDBCTL = remdbctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   REMC Interrupt enable
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17enableIntRemc(void)
{
	REMINTF_REG		remintf_set;
	REMINTE_REG		reminte_set;

	//Clear interrupt flag.
	/* REMINTF: REMC Status and Interrupt Flag Register. */
	{
		remintf_set.reg = 0;
		remintf_set.bit.DBIF = 1;
		remintf_set.bit.APIF = 1;
		REMINTF	= remintf_set.reg;
	}

	/**
	 * @todo Change interrupt
	 */
	// DBIF=1 APIF=0  Enable interrupt.
	/* REMINTE: REMC Interrupt Enable Register. */
	{
		reminte_set.reg = 0;
		reminte_set.bit.DBIE = 1;
		reminte_set.bit.APIE = 0;
		REMINTE = reminte_set.reg;
	}
}


/************************************************************************//**
 * @brief   REMC Interrupt disable
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17disableIntRemc(void)
{
	REMINTF_REG		remintf_set;
	REMINTE_REG		reminte_set;

	//Clear interrupt flag.
	/* REMINTF: REMC Status and Interrupt Flag Register. */
	{
		remintf_set.reg = 0;
		remintf_set.bit.DBIF = 1;
		remintf_set.bit.APIF = 1;
		REMINTF	= remintf_set.reg;
	}

	// DBIF=0 APIF=0  Disable interrupt.
	/* REMINTE: REMC Interrupt Enable Register. */
	{
		reminte_set.reg = 0;
		reminte_set.bit.DBIE = 0;
		reminte_set.bit.APIE = 0;
		REMINTE = reminte_set.reg;
	}
}


/************************************************************************//**
 * @brief   Set Carrier REMC.
 *
 * @param   remcCarr:Carrier Waveform
 *  @li @c   0 to 255
 * @param   remcDuty:DUTY
 *  @li @c   0 to 255
 *
 * @retval  none
 ***************************************************************************/
void c17setCarrierRemc(unsigned short remcCarr, unsigned short remcDuty)
{
	REMCCTL_REG		remcctl_set;
	REMCARR_REG		remcarr_set;

	/* REMCARR: REMC Carrier Waveform Register. */
	{
		remcarr_set.bit.CRDTY = remcDuty;
		remcarr_set.bit.CRPER = remcCarr;
		REMCARR		= remcarr_set.reg;					///	Set carrier waveform.
	}

	/// Enable carrier modulation.
	/* REMCCTL: REMC Carrier Modulation Control Register. */
	{
		remcctl_set.reg = 0;
		remcctl_set.bit.CARREN = 1;
		REMCCTL = remcctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Set REMO output polarity.
 *
 * @param   remoPol:REMO output polarity
 *
 ***************************************************************************/
void c17setRemoPolRemc(unsigned short remoPol)
{
	REMCCTL_REG		remcctl_set;

	/// set REMO output polarity.
	/* REMCCTL: REMC Carrier Modulation Control Register. */
	{
		remcctl_set.reg = 0;
		remcctl_set.bit.OUTINVEN = remoPol;
		REMCCTL = remcctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Remc start.
 *
 * @param   remcAplen      data bit active pulse length.
 *  @li @c   0 to 65535
 * @param   remcDblen      data bit length.
 *  @li @c   0 to 65535
 *
 * @retval  none
 ***************************************************************************/
void c17startRemc(unsigned short remcAplen, unsigned short remcDblen)
{
	REMDBCTL_REG	remdbctl_set;

	REMAPLEN = remcAplen;			/// Set data bit active pulse length.
	REMDBLEN = remcDblen;			/// Set data bit length.

	/* REMDBCTL:REMC Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.MODEN = 1;
		REMDBCTL = remdbctl_set.reg;

		remdbctl_set.bit.PRESET = 1;
		remdbctl_set.bit.PRUN = 1;
		REMDBCTL = remdbctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Remc stop.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopRemc(void)
{
	REMDBCTL_REG	remdbctl_set;

	/* REMDBCTL:REMC Data Bit Counter Control Register. */
	{
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.PRUN = 0;
		REMDBCTL = remdbctl_set.reg;
	}
}


/************************************************************************//**
 * @brief   Send Remc data.
 *
 * @param   remcData:   Send data pointer
 * @param   remcLen:    Send data length
 *
 * @retval  none
 ***************************************************************************/
void c17sendRemc(const unsigned char* remcData, unsigned char remcLen)
{
	remc_send.sendData = (unsigned char*)remcData;
	remc_send.sendLength = remcLen;
	remc_send.Status = STATUS_DATA;
	remc_send.sendEndFlg = 0;
	remc_send.bitCount = 0;
	remc_send.sonybitCount = 0;
	c17startRemc(remc_wave.leader_aplen,remc_wave.leader_dblen);
}


/************************************************************************//**
 * @brief   REMC transmit end wait.
 *
 * @param   none
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int c17sendRemcDataWait(void)
{
	if(remc_send.sendEndFlg == 1)
	{
		return EXIT_SUCCESS;
	}
	else
	{
		return EXIT_FAILURE;
	}
}


/************************************************************************//**
 * @brief   1, Set Carrier Remc.
 *
 *          2, Set Data Bit Length and,Data Bit Active Pulse Length.
 *
 * @param   format
 *
 * @retval  none
 ***************************************************************************/
void c17setFormatRemc(unsigned short remcFormat)
{
	REMDBCTL_REG	remdbctl_set;

	send_format = remcFormat;

	c17disableIntRemc();
	switch(send_format)
	{
		case C17_REMC_FORMAT_NEC:
			c17setCarrierRemc(CRPER_NEC, CRPER_NEC/3);		// Set carrier waveform for NEC format
															// 38kHz, 1/3duty
															// CRPER = REMC_CLOCK/38KHz -1 = CRPER_NEC
															// CRDTY = CRPER_NEC/3

			/* REMDBCTL:REMC Data Bit Counter Control Register. */
			{
			   /**
				* @todo change REMOINV
				*/
				remdbctl_set.reg = REMDBCTL;
				remdbctl_set.bit.REMOINV = 1;
				REMDBCTL = remdbctl_set.reg;
			}

			// set leader APLEN, DBLEN
			remc_wave.leader_aplen = LEADER_APLEN_NEC;
			remc_wave.leader_dblen = LEADER_DBLEN_NEC;
			// set bit APLEN, DBLEN
			remc_wave.bit1_aplen = BIT_1_APLEN_NEC;
			remc_wave.bit1_dblen = BIT_1_DBLEN_NEC;
			remc_wave.bit0_aplen = BIT_0_APLEN_NEC;
			remc_wave.bit0_dblen = BIT_0_DBLEN_NEC;
			break;
		case C17_REMC_FORMAT_AEHA:
			c17setCarrierRemc(CRPER_AEHA, CRPER_AEHA/3);	// Set carrier waveform for AEHA format
															// 38kHz, 1/3duty
															// CRPER = REMC_CLOCK/38KHz -1 = CRPER_AEHA
															// CRDTY = CRPER_AEHA/3
			/* REMDBCTL:REMC Data Bit Counter Control Register. */
			{
			   /**
				* @todo change REMOINV
				*/
				remdbctl_set.reg = REMDBCTL;
				remdbctl_set.bit.REMOINV = 0;
				REMDBCTL = remdbctl_set.reg;
			}

			// set leader APLEN, DBLEN
			remc_wave.leader_aplen = LEADER_APLEN_AEHA;
			remc_wave.leader_dblen = LEADER_DBLEN_AEHA;
			// set bit APLEN, DBLEN
			remc_wave.bit1_aplen = BIT_1_APLEN_AEHA;
			remc_wave.bit1_dblen = BIT_1_DBLEN_AEHA;
			remc_wave.bit0_aplen = BIT_0_APLEN_AEHA;
			remc_wave.bit0_dblen = BIT_0_DBLEN_AEHA;
			break;
		case C17_REMC_FORMAT_SONY:
			c17setCarrierRemc(CRPER_SONY, CRPER_SONY/3);	// Set carrier waveform for SONY format
															// 40kHz, 1/3duty
															// CRPER = REMC_CLOCK/40KHz -1 = CRPER_SONY
															// CRDTY = CRPER_SONY/3

			/* REMDBCTL:REMC Data Bit Counter Control Register. */
			{
			   /**
				* @todo change REMOINV
				*/
				remdbctl_set.reg = REMDBCTL;
				remdbctl_set.bit.REMOINV = 1;
				REMDBCTL = remdbctl_set.reg;
			}

			remc_send.sonybitCount = 0;

			// set leader APLEN, DBLEN
			remc_wave.leader_aplen = LEADER_APLEN_SONY;
			remc_wave.leader_dblen = LEADER_DBLEN_SONY;
			// set bit APLEN, DBLEN
			remc_wave.bit1_aplen = BIT_1_APLEN_SONY;
			remc_wave.bit1_dblen = BIT_1_DBLEN_SONY;
			remc_wave.bit0_aplen = BIT_0_APLEN_SONY;
			remc_wave.bit0_dblen = BIT_0_DBLEN_SONY;
			break;
	}

	c17enableIntRemc();
}
/************************************************************************//**
 * @brief   1, Set Carrier Remc.
 *
 *          2, Set Data Bit Length and,Data Bit Active Pulse Length.
 *
 * @param   format
 *
 * @retval  none
 ***************************************************************************/
void c17setFormatRemcDirect(const ST_REMC_PARAM *param)
{
	REMDBCTL_REG	remdbctl_set;

	unsigned short crr = (unsigned short)(REMC_CLOCK * 1000000UL / (unsigned long)param->subcreer - 1);

	send_format = param->remcFormat;
	c17disableIntRemc();
	c17setCarrierRemc(crr, crr/3);		// Set carrier waveform for NEC format
																// 38kHz, 1/3duty
																// CRPER = REMC_CLOCK/38KHz -1 = CRPER_NEC
																// CRDTY = CRPER_NEC/3

	/* REMDBCTL:REMC Data Bit Counter Control Register. */
	{
	   /**
		* @todo change REMOINV
		*/
		remdbctl_set.reg = REMDBCTL;
		remdbctl_set.bit.REMOINV = (send_format == C17_REMC_FORMAT_SONY) ? 1 : 0;
		REMDBCTL = remdbctl_set.reg;
	}

	// set leader APLEN, DBLEN
	if (send_format != C17_REMC_FORMAT_SONY)
	{
		remc_wave.leader_aplen = param->t * REMC_CLOCK * param->leader_h - 1;
		remc_wave.leader_dblen = param->t * REMC_CLOCK * (param->leader_h + param->leader_l) - 1;
	}
	else
	{
		remc_wave.leader_dblen = param->t * REMC_CLOCK * param->leader_h - 1;
		remc_wave.leader_aplen = 0;
	}
	// set bit APLEN, DBLEN
	remc_wave.bit1_aplen = param->t * REMC_CLOCK * param->bit1_h - 1;
	remc_wave.bit1_dblen = param->t * REMC_CLOCK * (param->bit1_h + param->bit1_l) - 1;
	remc_wave.bit0_aplen = param->t * REMC_CLOCK * param->bit0_h - 1;
	remc_wave.bit0_dblen = param->t * REMC_CLOCK * (param->bit0_h + param->bit0_l) - 1;

	c17enableIntRemc();
}

/************************************************************************//**
 * @brief   REMC interrupt function
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intRemc(void)
{
	REMINTF_REG	remintf_set;

	/* Capture interrupt flag */
	remintf_set.reg = REMINTF;
	/* Clear Interrupt flag. */
	REMINTF = remintf_set.reg;

	/// Check interrupt cause occurrence status.
	if (remintf_set.bit.APIF == 1) {

	   /**
		* @todo add interrupt function of APIF
		*/
	}
	/// Check interrupt cause occurrence status.
	if (remintf_set.bit.DBIF == 1) {
	   /**
		* @todo add interrupt function of DBIF
		*/
		if(send_format == C17_REMC_FORMAT_NEC || send_format == C17_REMC_FORMAT_AEHA)
		{
			necFormatInt();
		}
		else if(send_format == C17_REMC_FORMAT_SONY)
		{
			sonyFormatInt();
		}
	}
}


/************************************************************************//**
 * @brief   For NEC format & AEHA format interrupt function
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
static void necFormatInt(void)
{
	switch(remc_send.Status)
	{
		case STATUS_NONE:
			break;
		case STATUS_DATA:

			if(remc_send.bitCount == 0){
				remc_send.data = *remc_send.sendData++;
				remc_send.sendLength--;
			}
#if REMC_BITORDER == REMC_BITORDER_MSBFIRST
			if(remc_send.data & 0x80){
				c17startRemc(remc_wave.bit1_aplen,remc_wave.bit1_dblen);
			}
			else{
				c17startRemc(remc_wave.bit0_aplen,remc_wave.bit0_dblen);
			}

			remc_send.data <<= 1;
#else
			if(remc_send.data & 0x01){
				c17startRemc(remc_wave.bit1_aplen,remc_wave.bit1_dblen);
			}
			else{
				c17startRemc(remc_wave.bit0_aplen,remc_wave.bit0_dblen);
			}

			remc_send.data >>= 1;
#endif
			remc_send.bitCount++;
			if(remc_send.bitCount == 8){
				remc_send.bitCount = 0;

				if(remc_send.sendLength == 0){
					remc_send.Status = STATUS_TR;
					break;
				}
			}
			break;
		case STATUS_TR:
			c17startRemc(remc_wave.bit1_aplen,remc_wave.bit1_dblen);
			remc_send.Status = STATUS_END;
			break;
		case STATUS_END:
			remc_send.sendEndFlg = 1;
			remc_send.Status = STATUS_NONE;
			c17stopRemc();
			break;
	}
}


/************************************************************************//**
 * @brief   For SONY format interrupt function
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void sonyFormatInt(void)
{
	switch(remc_send.Status)
	{
		case STATUS_NONE:
			break;
		case STATUS_DATA:
			if(remc_send.bitCount==0){
				remc_send.data = *remc_send.sendData++;
				remc_send.sendLength--;
			}
#if REMC_BITORDER == REMC_BITORDER_MSBFIRST
			if(remc_send.data & 0x80){
				c17startRemc(remc_wave.bit1_aplen,remc_wave.bit1_dblen);
			}
			else{
				c17startRemc(remc_wave.bit0_aplen,remc_wave.bit0_dblen);
			}
			remc_send.data <<= 1;
#else
			if(remc_send.data & 0x01){
				c17startRemc(remc_wave.bit1_aplen,remc_wave.bit1_dblen);
			}
			else{
				c17startRemc(remc_wave.bit0_aplen,remc_wave.bit0_dblen);
			}
			remc_send.data >>= 1;
#endif
			remc_send.bitCount++;
			remc_send.sonybitCount++;
			if(remc_send.bitCount == 8){
				remc_send.bitCount = 0;
			}
			if(remc_send.sonybitCount == 12){
				remc_send.Status = STATUS_TR;
				break;
			}
			break;
		case STATUS_TR:
			remc_send.sendEndFlg = 1;
			remc_send.Status = STATUS_NONE;
			c17stopRemc();
			break;
	}
}
#endif

