/************************************************************************//**
 *
 * @file    c17_wdt2_1.c
 *
 * @brief   This is Watch Dog Timer program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2016-2021. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 01 Sep. 2016    1.00
 *              - First revision.
 *          - 09 Jun. 2021    1.10
 *              - Change source code.
 *
 ***************************************************************************/
#include "c17_wdt2_1.h"

#include <stdlib.h>
#include "c17_misc.h"
#include "c17_init_config.h"

#if defined(C17_PE_WDT)

/************************************************************************//**
 * @brief   Initialize watchdog Timer.
 *
 * @param   none
 *
 * @retval  EXIT_SUCCESS        Success.
 * @retval  EXIT_FAILURE        Failed.
 ***************************************************************************/
void c17initWdt(void)
{
	WDTCLK_REG	wdtclk_set;
	WDTCTL_REG	wdtctl_set;
	WDTCMP_REG	wdtcmp_set;


	/// Stop watchdog Timer.
	c17stopWdt();

	/// Disable write-protect.
	c17protectMiscOff();

	/// Set clock source for watchdog Timer.
	/* WDTCLK:WDT2 Clock Control Register */
	{
		 /**
		 * @todo Change clock source & Clock division ratio selection
		 */
		wdtclk_set.bit.DBRUN = 0;		/// - Clock supplied in DEBUG mode(0:Disable 1:Enable).
#if defined(C17_MCUSEL_W13) || \
	defined(C17_MCUSEL_W34) || defined(C17_MCUSEL_W35) || defined(C17_MCUSEL_W36)
		wdtclk_set.bit.CLKDIV = 2;		/// - Set division ratio for clock source(1/128).
#endif
#if defined(C17_MCUSEL_M12) || defined(C17_MCUSEL_M13) || \
	defined(C17_MCUSEL_M30) || defined(C17_MCUSEL_M31) || defined(C17_MCUSEL_M32) || defined(C17_MCUSEL_M33) || defined(C17_MCUSEL_M34)
		wdtclk_set.bit.CLKDIV = 0;		/// - Set division ratio for clock source(1/128).
#endif
		wdtclk_set.bit.CLKSRC = 1;		/// - Set clock source(OSC1).
		WDTCLK = wdtclk_set.reg;
	}

	/* WDTCMP:WDT2 Counter Compare Match Register. */
	{
		 /**
		 * @todo Change Counter Compare Match Register
		 */
		wdtcmp_set.bit.CMP = 0x3FF;		/// - CMP= 4sec * (32.768KHz/128) -1
		WDTCMP = wdtcmp_set.reg;
	}

	/// Set RESET mode.
	/// Reset watchdog Timer count.
	/* WDTCTL:WDT2 Control Register */
	{
		 /**
		 * @todo Change MOD Register
		 */
		wdtctl_set.bit.MOD = 0;			/// - Set RESET mode.
		wdtctl_set.bit.WDTCNTRST = 1;	/// - Reset
		wdtctl_set.bit.WDTRUN = 0xa;	/// - Stop
		WDTCTL = wdtctl_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();

}


/************************************************************************//**
 * @brief   Start watchdog Timer.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17startWdt(void)
{
	WDTCTL_REG	wdtctl_set;

	/// Disable write-protect.
	c17protectMiscOff();

	/// Start watchdog Timer.
	/// Reset watchdog Timer count.
	/* WDTCTL:WDT2 Control Register */
	{
		wdtctl_set.reg = WDTCTL;
		wdtctl_set.bit.WDTCNTRST = 1;	/// - Reset
		wdtctl_set.bit.WDTRUN = 0x0;	/// - Start
		WDTCTL = wdtctl_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();
}


/************************************************************************//**
 * @brief   Stop watchdog Timer.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17stopWdt(void)
{
	WDTCTL_REG	wdtctl_set;

	/// Disable write-protect.
	c17protectMiscOff();

	/// Stop watchdog Timer.
	/// Reset watchdog Timer count.
	/* WDTCTL:WDT2 Control Register */
	{
		wdtctl_set.reg = WDTCTL;
		wdtctl_set.bit.WDTCNTRST = 1;	/// - Reset
		wdtctl_set.bit.WDTRUN = 0xa;	/// - Stop
		WDTCTL = wdtctl_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();
}

/************************************************************************//**
 * @brief   Force reset.
 *
 *
 * @retval  none
 ***************************************************************************/
void c17forceResetWdt(void)
{
	
	WDTCMP_REG	wdtcmp_set;
	WDTCLK_REG	wdtclk_set;

	/// Stop watchdog Timer.
	c17stopWdt();

	/// Disable write-protect.
	c17protectMiscOff();

	/// Set clock source for watchdog Timer.
	/* WDTCLK:WDT2 Clock Control Register */
	{
		 /**
		 * @todo Change clock source & Clock division ratio selection
		 */
		wdtclk_set.reg = WDTCLK ;
		wdtclk_set.bit.CLKSRC = 0;
		WDTCLK = wdtclk_set.reg;
		wdtclk_set.bit.CLKSRC = 1;		/// - Set clock source(OSC1).
		WDTCLK = wdtclk_set.reg;
	}


	/* WDTCMP:WDT2 Counter Compare Match Register. */
	{
		 /**
		 * @todo Change Counter Compare Match Register
		 */
		wdtcmp_set.bit.CMP = 2;		/// - CMP= 4sec * (32.768KHz/128) -1
		WDTCMP = wdtcmp_set.reg;
	}

	/// Enable write-protect.
	c17protectMiscOn();

	/// Start watchdog Timer.
	c17startWdt();
}

#endif

