/************************************************************************//**
 *
 * @file    main.c
 *
 * @brief   This is S1C17M33 IR Remote controller sample program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018-2021. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *          - 09 Jun. 2021    1.10
 *              - Change source code.
 *
 ***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "crt0.h"
#include "c17_clg.h"
#include "c17_misc.h"
#include "c17_port_1.h"
#include "c17_t16_1.h"
#include "c17_wdt2_1.h"
#include "c17_svd3_1.h"
#include "c17_init_config.h"
#include <c17_mcu_select.h>
#include "c17_rtca_1.h"
#include "mid_dipsw.h"
#include "mid_key.h"
#include "main_config.h"
#include "main_acc.h"
#include "mid_bat.h"
#include "util.h"

/* --- external declaration --- */
#ifdef MAIN_USE_DEMO_MODE
extern int main_runDemo(void);
#endif
#ifdef MAIN_USE_TERM_MODE
extern int main_runTerm(void);
#endif
#ifdef MAIN_USE_TEST_MODE
extern int main_runTest(void);
#endif

/* --- function --- */
int main(void);
static void rtcInit(void);

/************************************************************************//**
 * main function.
 *
 * @return  EXIT_SUCCESS    Success.
 * @return  EXIT_FAILURE    Failed.
 ***************************************************************************/
int main(void)
{
	/// Clock = IOSC
	c17startClgIOSC();
	c17switchClgSystemClockIOSC();

	/// Initialize and start OSC1.
	c17selOscTypOSC1(C17_CLG_OSC1_CRYSTAL);
	c17startClgOSC1();

	/// start OSC3(internal oscillator)
	c17stopClgOSC3();
	c17startClgIntOSC3(C17_CLG_OSC3_12MHZ);

	/// the system clock set in OSC3.
	c17switchClgSystemClockOSC3();
	c17stopClgIOSC();

	/// Continue operation state before SLEEP
	c17setOsc1OpeInSleep(C17_CLG_CONTINUE);
	c17setOsc3OpeInSleep(C17_CLG_STOP);


	/// Initialize the driver.
	c17initWdt();
	c17resetWdt();
	c17initPort();
	c17initT16();
	rtcInit();
	mid_initBat();
	mid_initDipSw();
	
	/// Call the main routine for each application.
	switch(mid_getDipSwState())
	{
	case 0x0:	/// 0000: Demonstration mode
		mid_offDipSw();
#ifdef MAIN_USE_DEMO_MODE
		main_runDemo();
#endif
		break;
	case 0x5:	/// 0101: Terminal mode
		mid_offDipSw();
#ifdef MAIN_USE_TERM_MODE
		main_runTerm();
#endif
		break;
	case 0xF:	/// 1111: Test mode
#ifdef MAIN_USE_TEST_MODE
		main_runTest();
#endif
		break;
	case 0x8:	/// 1000: Remote controller mode.
	default:
		mid_offDipSw();
		main_runAcc();
		break;
	}
	return EXIT_SUCCESS;
}

static void rtcInit(void)
{
	ST_RTCA_BCD rtcval;

	rtcval.year   = util_convUShortNumToBcd(ACC_CLOCK_START_YEAR);
	rtcval.month  = 0x01;
	rtcval.day    = 0x01;
	rtcval.week   = util_getWeek(ACC_CLOCK_START_YEAR, 1, 1);
	rtcval.ampm   = C17_RTCA_AM;
	rtcval.hour   = 0x00;
	rtcval.minute = 0x00;
	rtcval.second = 0x00;

	c17initRtca();
	c17setRtcaMode(C17_RTCA_12H);
	c17setRtcaDate(&rtcval);
	c17startRtca();
	c17enableIntRtca();
}
