/************************************************************************//**
 *
 * @file    main_acc.c
 *
 * @brief   This is Remote controller mode program for S1C17M33 IR Remote controller sample.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "c17_rtca_1.h"
#include "c17_wdt2_1.h"
#include "c17_port_1.h"
#include "c17_clg.h"
#include "main_config.h"
#include "util.h"
#include "mid_key.h"
#include "mid_lcdac.h"
#include "mid_tmr.h"
#include "mid_accode.h"
#include "mid_remsnd.h"
#include "main_acc.h"
#include "mid_bat.h"

/* --- macro --- */
#define ACC_IR_REPEAT_COUNT	1	//Specify repeat count(sending count of same code).

/* --- constant variable --- */
#define ACC_WIND_HORI_ORDER_COUNT		(sizeof(AccWindHoriOrder) / sizeof(unsigned short))
const unsigned short AccWindHoriOrder[] = {0, 3, 2, 1, 2, 3, 4, 5, 4, 3};
const char *AccYearBlank         = "     ";
const char *AccTimerDisableBlank = "--:--";

/* --- function --- */
static void accExecOnOffKeyInput(KEY_STATE_S *pKeyState);
static void accExecKeyInputNormal(KEY_STATE_S *pKeyState);
static void accExecKeyInputClock(KEY_STATE_S *pKeyState);
static void accExecKeyInputTimer(KEY_STATE_S *pKeyState);
static void accReqIrSend(void);
static void accChangeOperationState(int newState);
static void accUpdateNormalClockDisp(void);
static void accSetCursorBlinkOn(void);
static void accSetCursorBlinkOff(void);
static void accSetClockDispMode(int mode);
static void accRtcBcdToWork(unsigned short rtcaMode, const ST_RTCA_BCD *psrc, ACC_DATE_TIME_S *pdst);
static void accWorkToRtcBcd(unsigned short rtcaMode, const ACC_DATE_TIME_S *psrc, ST_RTCA_BCD *pdst);
static int accMoveCursor(KEY_STATE_S *pKeyState, unsigned short *pcursor, unsigned short min, unsigned short max);

/* --- global variables ---*/
ACC_AC_SETTINGS_S	AccAcSettings;
int				AccOperationState = ACC_OPERATION_STATE_NORMAL;
int				AccClockDispMode = ACC_CLOCK_DISP_TIME;
int				AccIsIrSending = 0;

/* --- static variable --- */
static REMAC_IRCODE_S	AccIrCode;
static unsigned int		AccClockPrevCount;
static unsigned short	AccCursor;
static unsigned char	AccCursorBlinkEnable;
static unsigned char	AccCursorBlinkState;
static ACC_DATE_TIME_S	AccClockWork;
static ACC_TIMER_S		AccOnTimerWork;
static ACC_TIMER_S		AccOffTimerWork;
static unsigned short	AccRtcModeWork;
static int				AccClockAlwayDisp = ACC_ALWAYS_CLOCK_DSIP;
static int				AccIsTramsIconDisp = 0;
static int				AccDelaySend = 0;

/// Settings for the Air conditioner control.
static unsigned short		AccWindHoriIdx = 0;
static int					AccWindVertDir = ACC_WIND_VERT_DIR_DN;

/************************************************************************//**
 * main function
 * @brief  Remote control mode program.
 *
 * @retval  EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_runAcc(void)
{
	int activeState;


	/// Initialize the module.
	mid_initRemSnd();
	mid_initLcdAc();
	mid_initKey();
	mid_initTmr();
	main_initAcc();

	/// Main loop
	activeState = ACC_ACTIVE_STATE_STANDBY;
	while (1)
	{
		if (activeState == ACC_ACTIVE_STATE_STANDBY)
		{
			/// Turn off the module.
			main_offAcc();

			/// Enter to standby.
			mid_setKeyWakeup(KEY_ONOFF);
			if (AccClockAlwayDisp)
			{
				mid_onLcdAc();
				accUpdateNormalClockDisp();
			}
			while (1)
			{

				// Set sleep mode.
									// Nop is necessary when the main program is not executed after
									// sleep is	released, and it diverges to the server blade.
				asm ("slp");
				asm ("nop");

				/// Reset the WDT.
				c17resetWdt();

				/// Check the power voltage.
				mid_chkBatLowBattery(0);

				/// check if ON/OFF key is pressed.
				if (mid_chkKeyWakeup())
				{
					break;
				}

				/// Update clock display.
				if (AccClockAlwayDisp)
				{
					if (AccClockPrevCount != c17rtcaIntCount)
					{
						accUpdateNormalClockDisp();
					}
				}
			}
			/// Turn on the module.
			main_onAcc();
			activeState = ACC_ACTIVE_STATE_ACTIVE;
		}
		else if (activeState == ACC_ACTIVE_STATE_ACTIVE)
		{
			KEY_STATE_S keystate;

			/// Reset the WDT.
			c17resetWdt();

			/// Get key event.
			mid_getKeyState(&keystate);
			if (keystate.keycode != KEY_NONE)
			{
				main_execAccKeyInput(&keystate);
			}

			/// Check sending progress of IR remote control code.
			if (!main_chkAccIrSendState())
			{
				if (!AccAcSettings.AcOnOff)
				{
					activeState = ACC_ACTIVE_STATE_STANDBY;
					continue;
				}

			}

			/// Check the power voltage.
			if (mid_chkBatLowBattery(AccIsIrSending)) {
				mid_dispLcdAcBatteryLevel(LCDAC_BATTERY_LEVEL_LOW);
			} else {
				mid_dispLcdAcBatteryLevel(LCDAC_BATTERY_LEVEL_NO);
			}
			/// Update clock display.
			main_updateClockDisp();

			// Set halt mode.
					// Nop is necessary when the main program is not executed after
					// halt is	released, and it diverges to the server blade.
			asm ("halt");
			asm ("nop");
		}
	}
	return EXIT_SUCCESS;
}


/************************************************************************//**
 * @brief  Initialize the parameter for remote controller.
 *
 * @retval  none
 ***************************************************************************/
void main_initAcc(void)
{
	mid_initAcCode(&AccIrCode);
	AccAcSettings.AcOnOff = 0;
	AccAcSettings.TempUnit = ACC_TEMP_UNIT;
	AccAcSettings.Temp = ACC_TEMP_DEF;
	AccAcSettings.Mode = 0;
	AccAcSettings.WindVert = 0;
	AccAcSettings.WindHori = 0;
	AccAcSettings.WindFan = 0;
	AccAcSettings.OnTimer.enable = 0;
	AccAcSettings.OnTimer.hour = 0;	/// AM 0:00
	AccAcSettings.OnTimer.minute = 0;
	AccAcSettings.OnTimer.enable = 0;
	AccAcSettings.OffTimer.hour = 0;	/// AM 0:00
	AccAcSettings.OffTimer.minute = 0;
	AccWindHoriIdx = 0;
	AccWindVertDir = ACC_WIND_VERT_DIR_DN;

}

/************************************************************************//**
 * @brief  Turn on the remote controller.
 *
 * @retval  none
 ***************************************************************************/
void main_onAcc(void)
{
	/// Set operation state to normal.
	AccOperationState = ACC_OPERATION_STATE_NORMAL;
	AccClockDispMode  = ACC_CLOCK_DISP_TIME;

	/// Turn on interfaces.
	mid_onLcdAc();
	mid_onRemSnd();

	/// Update LCD display.
	mid_dispLcdAcSeparator();
	mid_dispLcdAcMode(AccAcSettings.Mode);
	mid_dispLcdAcTemperature(AccAcSettings.TempUnit, AccAcSettings.Temp);
	mid_dispLcdAcWindVert(AccAcSettings.WindVert);
	mid_dispLcdAcWindHori(AccAcSettings.WindHori);
	mid_dispLcdAcWindFan(AccAcSettings.WindFan);
	accUpdateNormalClockDisp();

	/// Power on the A/C
	AccAcSettings.AcOnOff = 1;
	accReqIrSend();
}
/************************************************************************//**
 * @brief  Turn off the remote controller.
 *
 * @retval  none
 ***************************************************************************/
void main_offAcc(void)
{
	/// Turn off interfaces.
	mid_stopTmr(MAIN_CURSOR_BLINK_TIMER);
	mid_stopTmr(MAIN_CLOCK_DISP_TIMER);
	mid_stopTmr(MAIN_SETTING_TIMER);
	mid_stopTmr(MAIN_TRAMS_ICON_TIMER);
	AccIsTramsIconDisp = 0;
	AccIsIrSending     = 0;
	mid_offLcdAc();
	mid_offRemSnd();
}


/************************************************************************//**
 * @brief  Process the key input.
 *
 * @param pKeyState		key state.
 *
 * @retval  none
 ***************************************************************************/
void main_execAccKeyInput(KEY_STATE_S *pKeyState)
{
	/// Set default key scan parameter.
	if (pKeyState->keyevent == KEY_EVENT_PRESS)
	{
		mid_setKeyLongPressTime(1000);
		mid_setKeyRepeateTime(200);
	}

	/// Check if ON/OFF key is input.
	if (pKeyState->keycode == KEY_ONOFF)
	{
		accExecOnOffKeyInput(pKeyState);
		return;
	}

	/// Key operation processing for each control state.
	if (!AccAcSettings.AcOnOff)
	{
		return;
	}
	switch(AccOperationState) {
	case ACC_OPERATION_STATE_NORMAL:
		/// Operation state.
		/// Normal.
		accExecKeyInputNormal(pKeyState);
		break;
	case ACC_OPERATION_STATE_SET_CLOCK:
		/// Operation state.
		/// Clock setting.
		accExecKeyInputClock(pKeyState);
		break;
	case ACC_OPERATION_STATE_SET_TIMER:
		/// Operation state.
		/// Timer setting.
		accExecKeyInputTimer(pKeyState);
		break;
	}
}
/************************************************************************//**
 * @brief  Process the ON/OFF key input.
 *
 * @param pKeyState		key state.
 *
 * @retval  none
 ***************************************************************************/
static void accExecOnOffKeyInput(KEY_STATE_S *pKeyState)
{
	if (pKeyState->keyevent == KEY_EVENT_PRESS)
	{
		if (AccAcSettings.AcOnOff)
		{
			/// Turn off the A/C and then entering standby.
			AccAcSettings.AcOnOff = 0;
			accReqIrSend();
		}
		else
		{
			/// Turn on the A/C.
			AccAcSettings.AcOnOff = 1;
			accChangeOperationState(ACC_OPERATION_STATE_NORMAL);
			accReqIrSend();
		}
	}
	else if (pKeyState->keyevent == KEY_EVENT_PRESS_LIMIT)
	{
		/// Turn off the A/C and then entering standby.
		AccAcSettings.AcOnOff = 0;
		accReqIrSend();
	}
}
/************************************************************************//**
 * @brief  Process the key input in normal state.
 *
 * @param pKeyState		key state.
 *
 * @retval  none
 ***************************************************************************/
static void accExecKeyInputNormal(KEY_STATE_S *pKeyState)
{
	switch(pKeyState->keycode)
	{
	case KEY_MODE:
		//
		// Normal key operation.
		// MODE key event.
		//
		if (pKeyState->keyevent == KEY_EVENT_PRESS)
		{
			/// switch to next mode at pressed.
			util_incUShortVal(&AccAcSettings.Mode, 0, ACC_MODE_ITEM_COUNT - 1, UTIL_WRAP_ENABLE);
			mid_dispLcdAcMode(AccAcSettings.Mode);
			accReqIrSend();
		}
		break;
	case KEY_TEMP_UP:
	case KEY_TEMP_DN:
		//
		// Normal key operation.
		// TEMP UP/DOWN key event.
		//
		if ((pKeyState->keyevent == KEY_EVENT_PRESS) ||
			(pKeyState->keyevent == KEY_EVENT_REPEAT) ||
			(pKeyState->keyevent == KEY_EVENT_LONG_PRESS))
		{
			/// Rise or lower the temperature at pressed.
			util_upDnUShortVal(
					pKeyState->keycode == KEY_TEMP_UP,
					&AccAcSettings.Temp, ACC_TEMP_MIN, ACC_TEMP_MAX, UTIL_WRAP_DISABLE);
			mid_dispLcdAcTemperature(AccAcSettings.TempUnit, AccAcSettings.Temp);
			if (pKeyState->keyevent == KEY_EVENT_PRESS)
			{
				AccDelaySend = 0;
				accReqIrSend();
			} else {
				AccDelaySend = 1;
			}
		} else if (pKeyState->keyevent == KEY_EVENT_RELEASE) {
			if (AccDelaySend)
			{
				accReqIrSend();
			}
		}
		break;
	case KEY_WIND_VERT:
		//
		// Normal key operation.
		// WIND Vertical event.
		//
		if (pKeyState->keyevent == KEY_EVENT_PRESS)
		{
			/// Switch to next position at pressed.
			if (AccWindVertDir == ACC_WIND_VERT_DIR_DN)
			{
				if (++AccAcSettings.WindVert >= (ACC_WIND_VERT_ITEM_COUNT - 1))
				{
					AccWindVertDir = ACC_WIND_VERT_DIR_UP;
				}
			}
			else
			{
				if (--AccAcSettings.WindVert == 0)
				{
					AccWindVertDir = ACC_WIND_VERT_DIR_DN;
				}
			}
			mid_dispLcdAcWindVert(AccAcSettings.WindVert);
			accReqIrSend();
		}
		break;
	case KEY_WIND_HORI:
		//
		// Normal key operation.
		// WIND Horizontal event
		//
		if (pKeyState->keyevent == KEY_EVENT_PRESS)
		{
			/// Switch to next position at pressed.
			util_incUShortVal(&AccWindHoriIdx, 0, ACC_WIND_HORI_ORDER_COUNT - 1, UTIL_WRAP_ENABLE);
			AccAcSettings.WindHori = AccWindHoriOrder[AccWindHoriIdx];
			mid_dispLcdAcWindHori(AccAcSettings.WindHori);
			accReqIrSend();
		}
		break;
	case KEY_WIND_FAN:
		//
		// Normal key operation.
		// WIND Fan event
		//
		if (pKeyState->keyevent == KEY_EVENT_PRESS)
		{
			/// Switch to next position at pressed.
			util_incUShortVal(&AccAcSettings.WindFan, 0, ACC_WIND_FAN_ITEM_COUNT - 1, UTIL_WRAP_ENABLE);
			mid_dispLcdAcWindFan(AccAcSettings.WindFan);
			accReqIrSend();
		}
		break;
	case KEY_CLOCK:
		//
		// Normal key operation.
		// Clock event
		//
		if (pKeyState->keyevent == KEY_EVENT_PRESS)
		{
			/// Switch to date display at pressed.
			accSetClockDispMode(ACC_CLOCK_DISP_DATE);
			accUpdateNormalClockDisp();
			///
			mid_setKeyLongPressTime(3000);

		}
		else if (pKeyState->keyevent == KEY_EVENT_LONG_PRESS)
		{
			/// Enter to state of clock setting at long press.
			accChangeOperationState(ACC_OPERATION_STATE_SET_CLOCK);
		}
		break;
	case KEY_TIME:
		//
		// Normal key operation.
		// TIME event
		//
		if (pKeyState->keyevent == KEY_EVENT_PRESS)
		{
			/// switch to timer reservation display at pressed.
			accSetClockDispMode(ACC_CLOCK_DISP_TIMER);
			accUpdateNormalClockDisp();
			mid_setKeyLongPressTime(2000);
		}
		else if (pKeyState->keyevent == KEY_EVENT_LONG_PRESS)
		{
			/// Enter to state of timer setting at long press.
			accChangeOperationState(ACC_OPERATION_STATE_SET_TIMER);
		}
		break;
	}
}
/************************************************************************//**
 * @brief  Process the key input in clock setting state.
 *
 * @param pKeyState		key state.
 *
 * @retval  none
 ***************************************************************************/
static void accExecKeyInputClock(KEY_STATE_S *pKeyState)
{
	unsigned short usTmp;
	int isComplete = 0;
	int isCancel = 0;

	/// Set default key scan parameter.
	if (pKeyState->keyevent == KEY_EVENT_PRESS)
	{
		mid_setKeyLongPressTime(1000);
		mid_setKeyRepeateTime(100);
	}

	/// Key input processing.
	switch(AccCursor)
	{
	case ACC_CURSOR_CLOCK_YEAR:
	case ACC_CURSOR_CLOCK_MONTH:
	case ACC_CURSOR_CLOCK_DAY:
		/// Clock setting key operation.
		/// Year input.
		switch(pKeyState->keycode)
		{
		case KEY_SETTING_UP:
		case KEY_SETTING_DN:
			switch(pKeyState->keyevent)
			{
			case KEY_EVENT_PRESS:
			case KEY_EVENT_LONG_PRESS:
			case KEY_EVENT_REPEAT:
				if (AccCursor == ACC_CURSOR_CLOCK_YEAR)
				{
					/// Increase or decrease the year.
					util_upDnUShortVal((pKeyState->keycode == KEY_SETTING_UP),
							&AccClockWork.year, ACC_CLOCK_START_YEAR, 99, UTIL_WRAP_ENABLE);
				}
				else if (AccCursor == ACC_CURSOR_CLOCK_MONTH)
				{
					/// Increase or decrease the month.
					util_upDnUShortVal(
						pKeyState->keycode == KEY_SETTING_UP, &AccClockWork.month, 1, 12, UTIL_WRAP_ENABLE);
				}
				else
				{
					/// Increase or decrease the day.
					usTmp = util_getDaysOfMonth(AccClockWork.year, AccClockWork.month);
					util_upDnUShortVal(
						pKeyState->keycode == KEY_SETTING_UP, &AccClockWork.day, 1, usTmp, UTIL_WRAP_ENABLE);
				}

				/// Check if the input date exceeds last day of month.
				usTmp = util_getDaysOfMonth(AccClockWork.year, AccClockWork.month);
				if (AccClockWork.day > usTmp)
				{
					 AccClockWork.day = usTmp;
				}
				/// Get week of the input date.
				AccClockWork.week = util_getWeek(AccClockWork.year, AccClockWork.month, AccClockWork.day);
				/// Pause blinking of cursor.
				AccCursorBlinkEnable = ACC_CURSOR_BLINK_DISABLE;
				accSetCursorBlinkOn();
				break;
			case KEY_EVENT_RELEASE:
				/// Resume blinking of cursor.
				AccCursorBlinkEnable = ACC_CURSOR_BLINK_ENABLE;
				accSetCursorBlinkOn();
				break;
			}
			break;
		default:
			isCancel = accMoveCursor(pKeyState, &AccCursor, ACC_CURSOR_CLOCK_YEAR, ACC_CURSOR_CLOCK_MINUTE);
			break;
		}
		break;
	case ACC_CURSOR_CLOCK_12h_24h:
		/// Clock setting key operation.
		/// 12/24h input.
		switch(pKeyState->keycode)
		{
		case KEY_SETTING_UP:
		case KEY_SETTING_DN:
			if(pKeyState->keyevent == KEY_EVENT_PRESS)
			{
				/// 12/24h mode is Alternately switched.
				AccRtcModeWork = (AccRtcModeWork == C17_RTCA_12H) ? C17_RTCA_24H : C17_RTCA_12H;
				accSetCursorBlinkOn();
			}
			break;
		default:
			accMoveCursor(pKeyState, &AccCursor, ACC_CURSOR_CLOCK_YEAR, ACC_CURSOR_CLOCK_MINUTE);
			break;
		}
		break;
	case ACC_CURSOR_CLOCK_HOUR:
	case ACC_CURSOR_CLOCK_MINUTE:
		/// Clock setting key operation.
		/// Hour or Minute input.
		switch(pKeyState->keycode)
		{
		case KEY_SETTING_UP:
		case KEY_SETTING_DN:
			switch(pKeyState->keyevent)
			{
			case KEY_EVENT_PRESS:
			case KEY_EVENT_LONG_PRESS:
			case KEY_EVENT_REPEAT:
				if (AccCursor == ACC_CURSOR_CLOCK_HOUR)
				{
					/// Increase or decrease the hour.
					util_upDnHour(pKeyState->keycode == KEY_SETTING_UP, &AccClockWork.hour);
				}
				else
				{
					/// Increase or decrease the minute.
					util_upDnMinute(pKeyState->keycode == KEY_SETTING_UP, &AccClockWork.minute);
				}
				/// Pause blinking of cursor.
				AccCursorBlinkEnable = ACC_CURSOR_BLINK_DISABLE;
				accSetCursorBlinkOn();
				break;
			case KEY_EVENT_RELEASE:
				/// Resume blinking of cursor.
				AccCursorBlinkEnable = ACC_CURSOR_BLINK_ENABLE;
				accSetCursorBlinkOn();
				break;
			}
			break;
		default:
			isComplete = accMoveCursor(pKeyState, &AccCursor, ACC_CURSOR_CLOCK_YEAR, ACC_CURSOR_CLOCK_MINUTE);
			break;
		}
		break;

	}
	if (isComplete)
	{
		/// Update clock setting.
		ST_RTCA_BCD tmpDate;
		if (c17getRtcaMode() != AccRtcModeWork)
		{
			c17setRtcaMode(AccRtcModeWork);
		}
		accWorkToRtcBcd(AccRtcModeWork, &AccClockWork, &tmpDate);
		c17setRtcaDate(&tmpDate);
		accReqIrSend();

		/// Return to normal control state.
		accChangeOperationState(ACC_OPERATION_STATE_NORMAL);
	}
	else if (isCancel)
	{
		/// Cancel the timer setting.
		accChangeOperationState(ACC_OPERATION_STATE_NORMAL);
	}
	else
	{
		mid_startTmr(MAIN_SETTING_TIMER);
	}
}

/************************************************************************//**
 * @brief  Process the key input in timer setting state.
 *
 * @param pKeyState		key state.
 *
 * @retval  none
 ***************************************************************************/
static void accExecKeyInputTimer(KEY_STATE_S *pKeyState)
{
	int isComplete = 0;
	int isCancel = 0;

	/// Set default key scan parameter.
	if (pKeyState->keyevent == KEY_EVENT_PRESS)
	{
		mid_setKeyLongPressTime(1000);
		mid_setKeyRepeateTime(100);
	}

	/// Key input processing.
	switch(AccCursor)
	{
	case ACC_CURSOR_TIMER_BLINK_ALL:
		/// Timer setting key operation.
		/// Wait for OK is input.
		isCancel = accMoveCursor(pKeyState, &AccCursor, ACC_CURSOR_TIMER_BLINK_ALL, ACC_CURSOR_TIMER_OFF_MINUTE);

		break;
	case ACC_CURSOR_TIMER_ON_ENABLE:
		/// Timer setting key operation.
		/// ON timer enable/disable cursor.
		switch(pKeyState->keycode)
		{
		case KEY_SETTING_UP:
		case KEY_SETTING_DN:
			if(pKeyState->keyevent == KEY_EVENT_PRESS)
			{
				AccOnTimerWork.enable = AccOnTimerWork.enable ? 0: 1;
				accSetCursorBlinkOn();
			}
			break;
		default:
			accMoveCursor(pKeyState, &AccCursor, ACC_CURSOR_TIMER_BLINK_ALL, ACC_CURSOR_TIMER_OFF_MINUTE);
			if (AccCursor < ACC_CURSOR_TIMER_ON_ENABLE)
			{
				isCancel = 1;
			}
			else if ((AccCursor == ACC_CURSOR_TIMER_ON_HOUR) &&
					 (AccOnTimerWork.enable == 0))
			{
				AccCursor = ACC_CURSOR_TIMER_OFF_ENABLE;
				accSetCursorBlinkOn();
			}
			break;
		}
		break;
	case ACC_CURSOR_TIMER_ON_HOUR:
	case ACC_CURSOR_TIMER_ON_MINUTE:
		/// Timer setting key operation.
		/// ON time input.
		switch(pKeyState->keycode)
		{
		case KEY_SETTING_UP:
		case KEY_SETTING_DN:
			switch(pKeyState->keyevent)
			{
			case KEY_EVENT_PRESS:
			case KEY_EVENT_LONG_PRESS:
			case KEY_EVENT_REPEAT:
				if (AccCursor == ACC_CURSOR_TIMER_ON_HOUR)
				{
					/// Increase or decrease the hour.
					util_upDnHour(pKeyState->keycode == KEY_TEMP_UP, &AccOnTimerWork.hour);
				}
				else
				{
					/// Increase or decrease the minute.
					util_upDnMinute(pKeyState->keycode == KEY_TEMP_UP, &AccOnTimerWork.minute);
				}
				/// Pause blinking of cursor.
				AccCursorBlinkEnable = ACC_CURSOR_BLINK_DISABLE;
				accSetCursorBlinkOn();
				break;
			case KEY_EVENT_RELEASE:
				/// Resume blinking of cursor.
				AccCursorBlinkEnable = ACC_CURSOR_BLINK_ENABLE;
				accSetCursorBlinkOn();
				break;
			}
			break;
		default:
			accMoveCursor(pKeyState, &AccCursor, ACC_CURSOR_TIMER_BLINK_ALL, ACC_CURSOR_TIMER_OFF_MINUTE);
			break;
		}
		break;
	case ACC_CURSOR_TIMER_OFF_ENABLE:
		/// Timer setting key operation.
		/// OFF timer enable/disable cursor.
		switch(pKeyState->keycode)
		{
		case KEY_SETTING_UP:
		case KEY_SETTING_DN:
			if(pKeyState->keyevent == KEY_EVENT_PRESS)
			{
				AccOffTimerWork.enable = AccOffTimerWork.enable ? 0: 1;
				accSetCursorBlinkOn();
			}
			break;
		default:
			accMoveCursor(pKeyState, &AccCursor, ACC_CURSOR_TIMER_BLINK_ALL, ACC_CURSOR_TIMER_OFF_MINUTE);
			if ((AccCursor == ACC_CURSOR_TIMER_ON_MINUTE) &&
					 (AccOnTimerWork.enable == 0))
			{
				AccCursor = ACC_CURSOR_TIMER_ON_ENABLE;
				accSetCursorBlinkOn();
			}
			else if ((AccCursor == ACC_CURSOR_TIMER_OFF_HOUR) &&
					 (AccOffTimerWork.enable == 0))
			{
				isComplete = 1;
			}
			break;
		}
		break;
	case ACC_CURSOR_TIMER_OFF_HOUR:
	case ACC_CURSOR_TIMER_OFF_MINUTE:
		/// Timer setting key operation.
		/// OFF time hour input.
		switch(pKeyState->keycode)
		{
		case KEY_SETTING_UP:
		case KEY_SETTING_DN:
			switch(pKeyState->keyevent)
			{
			case KEY_EVENT_PRESS:
			case KEY_EVENT_LONG_PRESS:
			case KEY_EVENT_REPEAT:
				if (AccCursor == ACC_CURSOR_TIMER_OFF_HOUR)
				{
					/// Increase or decrease the hour.
					util_upDnHour(pKeyState->keycode == KEY_TEMP_UP, &AccOffTimerWork.hour);
				}
				else
				{
					/// Increase or decrease the minute.
					util_upDnMinute(pKeyState->keycode == KEY_TEMP_UP, &AccOffTimerWork.minute);
				}
				/// Pause blinking of cursor.
				AccCursorBlinkEnable = ACC_CURSOR_BLINK_DISABLE;
				accSetCursorBlinkOn();
				break;
			case KEY_EVENT_RELEASE:
				/// Resume blinking of cursor.
				AccCursorBlinkEnable = ACC_CURSOR_BLINK_ENABLE;
				accSetCursorBlinkOn();
				break;
			}
			break;
		default:
			isComplete = accMoveCursor(pKeyState, &AccCursor, ACC_CURSOR_TIMER_BLINK_ALL, ACC_CURSOR_TIMER_OFF_MINUTE);
			break;
		}
		break;
	}

	if (isComplete)
	{
		/// Copy working timer setting to active variable.
		AccAcSettings.OnTimer  = AccOnTimerWork ;
		AccAcSettings.OffTimer = AccOffTimerWork;
		accReqIrSend();

		/// Return to normal control state.
		accChangeOperationState(ACC_OPERATION_STATE_NORMAL);
	}
	else if (isCancel)
	{
		/// Cancel the timer setting.
		accChangeOperationState(ACC_OPERATION_STATE_NORMAL);
	}
	else
	{
		mid_startTmr(MAIN_SETTING_TIMER);
	}
}

/************************************************************************//**
 * @brief  Request to send IR remote control code.
 *
 *
 * @retval  none
 ***************************************************************************/
static void accReqIrSend(void)
{
	/// Stop SVD.
	if (!AccIsIrSending)
	{
		AccIsIrSending     = 1;
		AccIsTramsIconDisp = 1;
		mid_dispLcdAcTransmission(1);
		mid_startTmr(MAIN_TRAMS_ICON_TIMER);
		mid_chkBatLowBattery(AccIsIrSending);
	}

	//Convert A/C settings to IR remote control code.
	mid_convAcCode(&AccIrCode, &AccAcSettings);

	//Request to send.
	mid_reqRemSndIrSendCode(ACC_IR_REPEAT_COUNT, AccIrCode.code, sizeof(AccIrCode.code));

}
/************************************************************************//**
 * @brief  Check sending progress of IR remote control code.
 *
 * @retval  0 Stopped
 * @retval  1 Sending
 ***************************************************************************/
int main_chkAccIrSendState(void)
{
	int result = mid_chkRemSndIrSendState();
	if (result == 0)
	{
		if (AccIsIrSending)
		{
			AccIsIrSending = 0;
		}
	} else {
		mid_startTmr(MAIN_TRAMS_ICON_TIMER);
	}
	if (AccIsTramsIconDisp)
	{
		if (mid_getTmr(MAIN_TRAMS_ICON_TIMER) > ACC_TRAMS_ICON_DISPTIME)
		{
			AccIsTramsIconDisp = 0;
			mid_stopTmr(MAIN_TRAMS_ICON_TIMER);
			mid_dispLcdAcTransmission(0);
		}
	}
	return result;
}
/************************************************************************//**
 * @brief  Update clock display area.
 *
 * @retval  none
 ***************************************************************************/
void main_updateClockDisp(void)
{
	if (!AccAcSettings.AcOnOff)
	{
		return;
	}

	if (AccOperationState == ACC_OPERATION_STATE_NORMAL)
	{
		if (AccClockPrevCount != c17rtcaIntCount)
		{
			accUpdateNormalClockDisp();
		}
		if (AccClockDispMode != ACC_CLOCK_DISP_TIME)
		{
			if (mid_getTmr(MAIN_CLOCK_DISP_TIMER) >= ACC_DATE_TIMER_DISP_TIME)
			{
				accSetClockDispMode(ACC_CLOCK_DISP_TIME);
				accUpdateNormalClockDisp();
			}
		}
	}
	else
	{
		if (mid_getTmr(MAIN_SETTING_TIMER) >= ACC_SETTING_TIMEOUT)
		{
			/// Cancel the timer setting.
			accChangeOperationState(ACC_OPERATION_STATE_NORMAL);
		}
		else if (AccCursorBlinkEnable == ACC_CURSOR_BLINK_ENABLE)
		{
			if (mid_getTmr(MAIN_CURSOR_BLINK_TIMER) >= ACC_CURSOR_BLINK_TIME)
			{
				if (AccCursorBlinkState == ACC_BLINK_STATE_ON)
				{
					accSetCursorBlinkOff();
				}
				else
				{
					accSetCursorBlinkOn();
				}
			}
		}

	}
}
/************************************************************************//**
 * @brief  Update normal clock display.
 *
 * @retval  none
 ***************************************************************************/
static void accUpdateNormalClockDisp(void)
{
	ST_RTCA_BCD cur;

	/// Clear Display area.
	mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 0, AccYearBlank);
	mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_LEFT, LCDAC_AMPM_DISP_NONE);
	mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_RIGHT, LCDAC_AMPM_DISP_NONE);
	mid_dispLcdAcTimerOn(0);
	mid_dispLcdAcTimerOff(0);
	mid_hideLcdAcAllWeek();
	

	/// Display current clock mode.
	c17getRtcaDate(&cur);
	switch(AccClockDispMode)
	{
	case ACC_CLOCK_DISP_TIME:
		// Time display mode.
		//
		mid_dispLcdAcTime(LCDAC_DAYTIMEPOS_RIGHT, c17getRtcaMode(), cur.ampm, cur.hour, cur.minute);
		break;
	case ACC_CLOCK_DISP_DATE:
		// Date display mode.
		//
		mid_dispLcdAcDate(cur.year, cur.month, cur.day, cur.week);
		break;
	case ACC_CLOCK_DISP_TIMER:
		// Timer display mode.
		//
		mid_dispLcdAcTimerOn(1);
		if (AccAcSettings.OnTimer.enable)
		{
			mid_dispLcdAcSerialTime(LCDAC_DAYTIMEPOS_LEFT, c17getRtcaMode(),
					(AccAcSettings.OnTimer.hour * 60) + AccAcSettings.OnTimer.minute);
		}
		else
		{
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 0, AccTimerDisableBlank);
		}
		mid_dispLcdAcTimerOff(1);
		if (AccAcSettings.OffTimer.enable)
		{
			mid_dispLcdAcSerialTime(LCDAC_DAYTIMEPOS_RIGHT, c17getRtcaMode(),
						(AccAcSettings.OffTimer.hour * 60) + AccAcSettings.OffTimer.minute);;
		}
		else
		{
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, AccTimerDisableBlank);
		}
		break;
	}
	AccClockPrevCount = c17rtcaIntCount;
}
/************************************************************************//**
 * @brief  Change operation state.
 *
 * @param   newState
 *  @li @c  ACC_KEYOPE_STATE_NORMAL
 *  @li @c  ACC_KEYOPE_STATE_SET_CLOCK
 *  @li @c  ACC_KEYOPE_STATE_SET_TIMER
 * @retval  none
 ***************************************************************************/
static void accChangeOperationState(int newState)
{
	ST_RTCA_BCD tmpDate;

	AccOperationState = newState;
	mid_cancelKeyCurKeyEvent();	/// Cancel key event until cancel key is released.
	accSetClockDispMode(ACC_CLOCK_DISP_TIME);
	switch(AccOperationState)
	{
	case ACC_OPERATION_STATE_NORMAL:
		/// Return to normal state.
		accUpdateNormalClockDisp();
		break;
	case ACC_OPERATION_STATE_SET_CLOCK:
		/// Enter to state of clock setting.
		c17getRtcaDate(&tmpDate);
		accRtcBcdToWork(c17getRtcaMode(), &tmpDate, &AccClockWork);
		AccCursor = ACC_CURSOR_CLOCK_YEAR;
		break;
	case ACC_OPERATION_STATE_SET_TIMER:
		/// Enter to state of timer setting.
		AccCursor = ACC_CURSOR_TIMER_BLINK_ALL;
		AccOnTimerWork  = AccAcSettings.OnTimer;
		AccOffTimerWork = AccAcSettings.OffTimer;
		break;
	}
	if (AccOperationState != ACC_OPERATION_STATE_NORMAL)
	{
		/// Display at blink on.
		AccCursorBlinkEnable = ACC_CURSOR_BLINK_ENABLE;
		accSetCursorBlinkOn();
		mid_startTmr(MAIN_SETTING_TIMER);
	}
	else
	{
		mid_stopTmr(MAIN_CURSOR_BLINK_TIMER);
		mid_stopTmr(MAIN_SETTING_TIMER);
	}
}
/************************************************************************//**
 * @brief  Show setting item on blink.
 *
 * @retval  none
 ***************************************************************************/
static void accSetCursorBlinkOn(void)
{
	if (AccOperationState == ACC_OPERATION_STATE_SET_CLOCK)
	{
		//
		// Clock setting.
		//
		switch(AccCursor)
		{
		case ACC_CURSOR_CLOCK_YEAR:
		case ACC_CURSOR_CLOCK_MONTH:
		case ACC_CURSOR_CLOCK_DAY:
			mid_dispLcdAcDate(
							util_convUShortNumToBcd(AccClockWork.year),
							util_convUShortNumToBcd(AccClockWork.month),
							util_convUShortNumToBcd(AccClockWork.day),
							AccClockWork.week);
			break;
		case ACC_CURSOR_CLOCK_12h_24h:
			mid_hideLcdAcAllWeek();
			mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_LEFT, LCDAC_AMPM_DISP_ALL);
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 0, AccYearBlank);
			mid_dispLcdAcSerialTime(LCDAC_DAYTIMEPOS_RIGHT,
							AccRtcModeWork,(AccClockWork.hour * 60) + AccClockWork.minute);
			break;
		case ACC_CURSOR_CLOCK_HOUR:
		case ACC_CURSOR_CLOCK_MINUTE:
			mid_hideLcdAcAllWeek();
			mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_LEFT, LCDAC_AMPM_DISP_NONE);
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 0, AccYearBlank);
			mid_dispLcdAcSerialTime(LCDAC_DAYTIMEPOS_RIGHT,
							AccRtcModeWork,(AccClockWork.hour * 60) + AccClockWork.minute);
			break;
		}
	}
	else if (AccOperationState == ACC_OPERATION_STATE_SET_TIMER)
	{
		//
		// Timer setting.
		//
		mid_showLcdAcAllWeek();
		mid_dispLcdAcTimerOn(1);
		mid_dispLcdAcTimerOff(1);

		if (AccOnTimerWork.enable)
		{
			mid_dispLcdAcSerialTime(LCDAC_DAYTIMEPOS_LEFT, c17getRtcaMode(),
					(AccOnTimerWork.hour * 60) + AccOnTimerWork.minute);
		}
		else
		{
			mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_LEFT, LCDAC_AMPM_DISP_NONE);
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 0, AccTimerDisableBlank);
		}
		if (AccOffTimerWork.enable)
		{
			mid_dispLcdAcSerialTime(LCDAC_DAYTIMEPOS_RIGHT, c17getRtcaMode(),
					(AccOffTimerWork.hour * 60) + AccOffTimerWork.minute);
		}
		else
		{
			mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_RIGHT, LCDAC_AMPM_DISP_NONE);
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, AccTimerDisableBlank);
		}
	}
	AccCursorBlinkState = ACC_BLINK_STATE_ON;
	mid_startTmr(MAIN_CURSOR_BLINK_TIMER);
}

/************************************************************************//**
 * @brief  Hide setting item on blink.
 *
 * @retval  none
 ***************************************************************************/
static void accSetCursorBlinkOff(void)
{
	if (AccOperationState == ACC_OPERATION_STATE_SET_CLOCK)
	{
		//
		// Clock setting.
		//
		switch(AccCursor)
		{
		case ACC_CURSOR_CLOCK_YEAR:
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 0, AccYearBlank);
			break;
		case ACC_CURSOR_CLOCK_MONTH:
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, "  ");
			break;
		case ACC_CURSOR_CLOCK_DAY:
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 3, "  ");
			break;
		case ACC_CURSOR_CLOCK_12h_24h:
			mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_LEFT, LCDAC_AMPM_DISP_NONE);
			break;
		case ACC_CURSOR_CLOCK_HOUR:
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, "  ");
			break;
		case ACC_CURSOR_CLOCK_MINUTE:
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 3, "  ");
			break;
		}
	}
	else if (AccOperationState == ACC_OPERATION_STATE_SET_TIMER)
	{
		//
		// Timer setting.
		//
		if (AccCursor == ACC_CURSOR_TIMER_BLINK_ALL)
		{
			/// Blink all item.
			mid_dispLcdAcTimerOn(0);
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 0, AccYearBlank);
			mid_dispLcdAcTimerOff(0);
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, AccYearBlank);
		}
		else
		{
			/// Blink item at cursor.
			switch(AccCursor)
			{
			case ACC_CURSOR_TIMER_ON_ENABLE:
				mid_dispLcdAcTimerOn(0);
				break;
			case ACC_CURSOR_TIMER_ON_HOUR:
				mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 0, "  ");
				break;
			case ACC_CURSOR_TIMER_ON_MINUTE:
				mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 3, "  ");
				break;
			case ACC_CURSOR_TIMER_OFF_ENABLE:
				mid_dispLcdAcTimerOff(0);
				break;
			case ACC_CURSOR_TIMER_OFF_HOUR:
				mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, "  ");
				break;
			case ACC_CURSOR_TIMER_OFF_MINUTE:
				mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 3, "  ");
				break;
			}
		}
	}
	AccCursorBlinkState = ACC_BLINK_STATE_OFF;
	mid_startTmr(MAIN_CURSOR_BLINK_TIMER);
}

static void accSetClockDispMode(int mode)
{
	AccClockDispMode = mode;

	mid_stopTmr(MAIN_CLOCK_DISP_TIMER);
	if (AccClockDispMode != ACC_CLOCK_DISP_TIME)
	{
		mid_startTmr(MAIN_CLOCK_DISP_TIMER);
	}
}

static void accRtcBcdToWork(unsigned short rtcaMode, const ST_RTCA_BCD *psrc, ACC_DATE_TIME_S *pdst)
{
	pdst->year   = util_convUShortBcdToNum(psrc->year);
	pdst->month  = util_convUShortBcdToNum(psrc->month);
	pdst->day    = util_convUShortBcdToNum(psrc->day);
	pdst->week   = psrc->week;
	pdst->hour   = util_convUShortBcdToNum(psrc->hour);
	pdst->minute = util_convUShortBcdToNum(psrc->minute);
	if ((rtcaMode == C17_RTCA_12H) && (psrc->ampm == C17_RTCA_PM))
	{
		pdst->hour += 12;
	}
}
static void accWorkToRtcBcd(unsigned short rtcaMode, const ACC_DATE_TIME_S *psrc, ST_RTCA_BCD *pdst)
{
	pdst->ampm = C17_RTCA_AM;
	if ((rtcaMode == C17_RTCA_12H) && (psrc->hour >= 12))
	{
		pdst->ampm = C17_RTCA_PM;
		pdst->hour -= 12;
	}
	pdst->year   = util_convUShortNumToBcd(psrc->year);
	pdst->month  = util_convUShortNumToBcd(psrc->month);
	pdst->day    = util_convUShortNumToBcd(psrc->day);
	pdst->week   = psrc->week;
	if ((rtcaMode == C17_RTCA_12H) && (psrc->hour >= 12))
	{
		pdst->ampm = C17_RTCA_PM;
		pdst->hour = util_convUShortNumToBcd(psrc->hour - 12);
	}
	else
	{
		pdst->hour   = util_convUShortNumToBcd(psrc->hour);
	}
	pdst->minute = util_convUShortNumToBcd(psrc->minute);
	pdst->second = 0;
}
static int accMoveCursor(KEY_STATE_S *pKeyState, unsigned short *pcursor, unsigned short min, unsigned short max)
{
	int cur = *pcursor;
	int isStop = 0;
	if (pKeyState->keyevent == KEY_EVENT_PRESS)
	{
		switch(pKeyState->keycode)
		{
		case KEY_OK:
			isStop = util_incUShortVal(pcursor, min, max, UTIL_WRAP_DISABLE);
			break;
		case KEY_CANCEL:
			isStop = util_decUShortVal(pcursor, min, max, UTIL_WRAP_DISABLE);
			break;
		}
	}
	if (cur != *pcursor)
	{
		accSetCursorBlinkOn();
	}

	return isStop;
}


