/************************************************************************//**
 *
 * @file    main_demo.c
 *
 * @brief   This is demonstration mode program for S1C17M33 IR Remote controller sample.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "c17_rtca_1.h"
#include "c17_wdt2_1.h"
#include "main_config.h"
#include "util.h"
#include "mid_key.h"
#include "mid_lcdac.h"
#include "mid_tmr.h"
#include "mid_accode.h"
#include "mid_remsnd.h"
#include "main_acc.h"
#include "mid_bat.h"

#ifdef MAIN_USE_DEMO_MODE
/* --- definition --- */
enum
{
	DEMO_CONTROL_NORMAL,
	DEMO_CONTROL_REMOTE,
};
enum
{
	DEMO_ITEM_DISP_ALL,
	DEMO_ITEM_DISP_NORMAL,
	DEMO_ITEM_DISP_TIME,
	DEMO_ITEM_DISP_DATE,
	DEMO_ITEM_DISP_ONTIME,
	DEMO_ITEM_DISP_OFFTIME,
	DEMO_ITEM_DISP_OPERATION,
	DEMO_ITEM_DISP_BATTERY,
	DEMO_ITEM_DISP_BAT_BLINK,
	DEMO_ITEM_DISP_ALL_CLEAR,
};

#define DEMO_OPERATION_SETP1	50
#define DEMO_OPERATION_SETP2	100

/* --- function --- */
static int demoDemonstrationMode(void);
static int demoRemoteControlMode(void);
static void demoSetItem(int nextItem);
static void demoExecDemoItem(void);
static void demoStartTimer(void);
static void demoStopTimer(void);
static unsigned int demoTimerElapse(unsigned int str, unsigned int prv);

/* --- constant variable --- */
#define DEMO_WIND_ORDER_COUNT		10
const unsigned char DemoWindHoriOrder[DEMO_WIND_ORDER_COUNT] = {0, 3, 2, 1, 2, 3, 4, 5, 4, 3};
const unsigned char DemoWindOrder[DEMO_WIND_ORDER_COUNT]     = {0, 1, 2, 3, 4, 5, 4, 3, 2, 1};


/* --- static variables ---*/
static int DemoControlMode = DEMO_CONTROL_NORMAL;
static int DemoItem = DEMO_ITEM_DISP_ALL;
static int DemoItemStep;
static unsigned short DemoWindIdx;
static unsigned int DemoItem100hz = 0;
static unsigned int DemoBlink100hz = 0;
static unsigned int Demo1hzTimer = 0;
static unsigned int DemoStepTime;
static unsigned int DemoTemp;

/************************************************************************//**
 * main function
 * @brief  Demonstration mode program.
 *
 * @retval  EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_runDemo(void)
{
	int activeState;

	/// Initialize the interfaces.
	mid_initRemSnd();
	mid_initLcdAc();
	mid_initKey();
	mid_initTmr();
	main_initAcc();

	/// Main loop
	activeState = ACC_ACTIVE_STATE_STANDBY;
	while (1)
	{
		if (activeState == ACC_ACTIVE_STATE_STANDBY)
		{
			/// Turn off module.
			main_offAcc();

			/// Enter to standby.
			mid_setKeyWakeup(KEY_ONOFF);
			while (1)
			{
				// Set sleep mode.
									// Nop is necessary when the main program is not executed after
									// sleep is	released, and it diverges to the server blade.
				asm ("slp");
				asm ("nop");

				/// Reset the WDT.
				c17resetWdt();

				/// Check the power voltage.
				mid_chkBatLowBattery(0);

				///@check if ON/OFF key is pressed.
				if (mid_chkKeyWakeup())
				{
					break;
				}
			}
			/// Turn on the module.
			activeState     = ACC_ACTIVE_STATE_ACTIVE;
			DemoControlMode = DEMO_CONTROL_NORMAL;
			mid_onLcdAc();
			demoStartTimer();
			demoSetItem(DEMO_ITEM_DISP_ALL);
		}
		else if (activeState == ACC_ACTIVE_STATE_ACTIVE)
		{
			/// Reset the WDT.
			c17resetWdt();

			/// Demonstration.
			if (DemoControlMode == DEMO_CONTROL_NORMAL)
			{
				activeState = demoDemonstrationMode();
			}

			/// Remote control.
			if (DemoControlMode == DEMO_CONTROL_REMOTE)
			{
				activeState = demoRemoteControlMode();
			}

			// Set halt mode.
					// Nop is necessary when the main program is not executed after
					// halt is	released, and it diverges to the server blade.
			asm ("halt");
			asm ("nop");
		}


	}
	return EXIT_SUCCESS;
}
/************************************************************************//**
 * @brief  Execute demonstration.
 *
 *
 * @retval  new state.
 ***************************************************************************/
static int demoDemonstrationMode(void)
{
	int activeState = ACC_ACTIVE_STATE_ACTIVE;
	KEY_STATE_S keystate;

	mid_getKeyState(&keystate);
	if (keystate.keycode == KEY_ONOFF)
	{
		if ((keystate.keyevent == KEY_EVENT_PRESS_LIMIT) ||
			(keystate.keyevent == KEY_EVENT_PRESS))
		{
			activeState = ACC_ACTIVE_STATE_STANDBY;
			return activeState;
		}
	}
	else if (keystate.keycode != KEY_NONE)
	{
		// Enter to remote control mode.
		DemoControlMode = DEMO_CONTROL_REMOTE;
		Demo1hzTimer = c17rtcaIntCount;
		demoStopTimer();
		main_onAcc();
	}
	else
	{
		demoExecDemoItem();
	}

	/// Check the power voltage.
	mid_chkBatLowBattery(0);

	return activeState;
}
/************************************************************************//**
 * @brief  Execute remote controller mode in demonstration mode.
 *
 * @param  None
 *
 * @retval  new state.
 ***************************************************************************/
static int demoRemoteControlMode(void)
{
	int activeState = ACC_ACTIVE_STATE_ACTIVE;
	KEY_STATE_S keystate;

	/// Get key state.
	mid_getKeyState(&keystate);
	if (keystate.keyevent != KEY_EVENT_NONE)
	{
		Demo1hzTimer = c17rtcaIntCount;
		main_execAccKeyInput(&keystate);
	}
	else
	{
		if (demoTimerElapse(c17rtcaIntCount, Demo1hzTimer) >= 60)
		{
			// Return to demonstration mode.
			demoStartTimer();
			DemoControlMode = DEMO_CONTROL_NORMAL;
			demoSetItem(DEMO_ITEM_DISP_ALL);
			return activeState;
		}
	}
	/// Check sending progress of IR remote control code.
	if (!main_chkAccIrSendState())
	{
		if (!AccAcSettings.AcOnOff)
		{
			activeState = ACC_ACTIVE_STATE_STANDBY;
			return activeState;
		}
	}

	/// Check the power voltage.
	if (mid_chkBatLowBattery(AccIsIrSending)) {
		mid_dispLcdAcBatteryLevel(LCDAC_BATTERY_LEVEL_LOW);
	} else {
		mid_dispLcdAcBatteryLevel(LCDAC_BATTERY_LEVEL_NO);
	}

	/// Update clock display.
	main_updateClockDisp();

	return activeState;
}
/************************************************************************//**
 * @brief  Set next demonstration item.
 *
 * @param  nextItem   Index of a next item.
 *
 * @retval  none
 ***************************************************************************/
static void demoSetItem(int nextItem)
{
	DemoItem = nextItem;
	DemoItemStep = 0;
	DemoItem100hz = c17swIntCount100;
	DemoBlink100hz = c17swIntCount100;

	switch(DemoItem)
	{
	case DEMO_ITEM_DISP_ALL:
		/// Set item
		/// LCD All display
		c17setLcd8aDisplayState(C17_LCD_STATE_ALL_ON);
		break;
	case DEMO_ITEM_DISP_NORMAL:
		// Set item
		// LCD normal display
		mid_dispLcdAcSeparator();
		mid_dispLcdAcMode(0);
		mid_dispLcdAcTemperature(AccAcSettings.TempUnit, 23);
		mid_dispLcdAcWindVert(0);
		mid_dispLcdAcWindHori(0);
		mid_dispLcdAcWindFan(0);
		c17setLcd8aDisplayState(C17_LCD_STATE_NORMAL);
		break;
	case DEMO_ITEM_DISP_TIME:
		// Set item
		// Time display
		mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_RIGHT, LCDAC_AMPM_DISP_PM);
		mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, " 1:23");
		break;
	case DEMO_ITEM_DISP_DATE:
		// Set item
		// Date display
		mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_RIGHT, LCDAC_AMPM_DISP_NONE);
		mid_dispLcdAcDate(0x18, 0x01, 0x23, C17_RTCA_THU);
		break;
	case DEMO_ITEM_DISP_ONTIME:
		// Set item
		// LCD on time display
		mid_hideLcdAcAllWeek();
		mid_dispLcdAcTimerOn(1);
		mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_LEFT, LCDAC_AMPM_DISP_AM);
		mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT, 0, " 7:00");
		mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, "     ");
		break;
	case DEMO_ITEM_DISP_OFFTIME:
		// Set item
		// LCD off time display
		mid_dispLcdAcTimerOff(1);
		mid_dispLcdAcAmPm(LCDAC_DAYTIMEPOS_RIGHT, LCDAC_AMPM_DISP_PM);
		mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, "11:00");
		break;
	case DEMO_ITEM_DISP_OPERATION:
		// Set item
		// LCD operation display
		DemoWindIdx = 0;
		mid_dispLcdAcWindVert(DemoWindIdx);
		mid_dispLcdAcWindHori(DemoWindIdx);
		mid_dispLcdAcWindFan(DemoWindIdx);
		//cooling step
		DemoTemp = 23;
		mid_dispLcdAcTemperature(AccAcSettings.TempUnit, 23);
		mid_dispLcdAcMode(1);
		DemoStepTime = DEMO_OPERATION_SETP1;
		break;
	case DEMO_ITEM_DISP_BATTERY:
		// Set item
		// LCD Battery display
		mid_dispLcdAcWindVert(0);
		mid_dispLcdAcWindHori(0);
		mid_dispLcdAcWindFan(0);
		mid_dispLcdAcBatteryLevel(LCDAC_BATTERY_LEVEL_LOW);
		break;
	case DEMO_ITEM_DISP_BAT_BLINK:
		// Set item
		// LCD Battery blink display
		mid_dispLcdAcBatteryLevel(LCDAC_BATTERY_LEVEL_NO);
		break;
	case DEMO_ITEM_DISP_ALL_CLEAR:
		// Set item
		// LCD All clear
		mid_clrLcdAc();
		break;
	}
}
/************************************************************************//**
 * @brief  Execute demonstration item.
 *
 * @retval  none
 ***************************************************************************/
static void demoExecDemoItem(void)
{
	switch(DemoItem)
	{
	case DEMO_ITEM_DISP_ALL:
	case DEMO_ITEM_DISP_NORMAL:
		/// Demo item
		/// LCD All display
		if (demoTimerElapse(c17swIntCount100, DemoItem100hz) >= 200)
		{
			demoSetItem(DemoItem + 1);
		}
		break;
	case DEMO_ITEM_DISP_TIME:
	case DEMO_ITEM_DISP_DATE:
	case DEMO_ITEM_DISP_ONTIME:
	case DEMO_ITEM_DISP_OFFTIME:
		/// Demo item
		/// LCD normal/on time/off time display
		if (demoTimerElapse(c17swIntCount100, DemoItem100hz) >= 100)
		{
			demoSetItem(DemoItem + 1);
		}
		break;
	case DEMO_ITEM_DISP_OPERATION:
		/// Demo item
		/// LCD operation display
		if (demoTimerElapse(c17swIntCount100, DemoBlink100hz) >= DEMO_OPERATION_SETP1)
		{
			util_upDnUShortVal(1, &DemoWindIdx, 0, DEMO_WIND_ORDER_COUNT - 1, UTIL_WRAP_ENABLE);
			mid_dispLcdAcWindVert(DemoWindOrder[DemoWindIdx]);
			mid_dispLcdAcWindHori(DemoWindHoriOrder[DemoWindIdx]);
			mid_dispLcdAcWindFan(DemoWindOrder[DemoWindIdx]);
			DemoBlink100hz = c17swIntCount100;
		}
		if (demoTimerElapse(c17swIntCount100, DemoItem100hz) >= DemoStepTime)
		{
			switch(DemoItemStep)
			{
			case 0:
				//Cooling step
				if (DemoTemp == 16) {
					DemoItemStep++;
					mid_dispLcdAcMode(2);
				} else {
					DemoTemp--;
					mid_dispLcdAcTemperature(AccAcSettings.TempUnit, DemoTemp);
				}
				break;
			case 1:
				//Heating step
				if (DemoTemp == 23) {
					DemoItemStep++;
					DemoStepTime = DEMO_OPERATION_SETP2;
					mid_dispLcdAcMode(3);
				} else {
					DemoTemp++;
					mid_dispLcdAcTemperature(AccAcSettings.TempUnit, DemoTemp);
				}
				break;
			case 2:
				//Blower step
				mid_dispLcdAcMode(4);
				DemoItemStep++;
				break;
			case 3:
				//Dehumidify step
				mid_dispLcdAcMode(3);
				DemoItemStep++;
				break;
			case 4:
				//Blower step
				mid_dispLcdAcMode(2);
				DemoItemStep++;
				DemoStepTime = DEMO_OPERATION_SETP1;
				break;
			case 5:
				//Heating step
				if (DemoTemp == 32) {
					DemoItemStep++;
					mid_dispLcdAcMode(1);
				} else {
					DemoTemp++;
					mid_dispLcdAcTemperature(AccAcSettings.TempUnit, DemoTemp);
				}
				break;
			case 6:
				//Cooling step
				if (DemoTemp == 23) {
					DemoItemStep++;
					mid_dispLcdAcMode(0);
				} else {
					DemoTemp--;
					mid_dispLcdAcTemperature(AccAcSettings.TempUnit, DemoTemp);
				}
				break;
			case 7:
				demoSetItem(DEMO_ITEM_DISP_BATTERY);
				break;
			}
			DemoItem100hz = c17swIntCount100;
		}
		break;
	case DEMO_ITEM_DISP_BATTERY:
		/// Demo item
		/// LCD Battery display
		if (demoTimerElapse(c17swIntCount100, DemoItem100hz) >= 100)
		{
			demoSetItem(DEMO_ITEM_DISP_BAT_BLINK);
		}
		break;
	case DEMO_ITEM_DISP_BAT_BLINK:
		/// Demo item
		/// LCD Battery blink display
		if (demoTimerElapse(c17swIntCount100, DemoItem100hz) >= 300)
		{
			demoSetItem(DEMO_ITEM_DISP_ALL_CLEAR);
		}
		else
		{
			unsigned int elap = demoTimerElapse(c17swIntCount100, DemoItem100hz);
			if ((elap > 0) && ((elap % 50) == 0))
			{
				if (DemoItemStep)
				{
					mid_dispLcdAcBatteryLevel(LCDAC_BATTERY_LEVEL_LOW);
					DemoItemStep = 0;
				}
				else
				{
					mid_dispLcdAcBatteryLevel(LCDAC_BATTERY_LEVEL_NO);
					DemoItemStep = 1;
				}
			}
		}
		break;
	case DEMO_ITEM_DISP_ALL_CLEAR:
		/// Demo item
		/// LCD All clear
		if (demoTimerElapse(c17swIntCount100, DemoItem100hz) >= 100)
		{
			demoSetItem(DEMO_ITEM_DISP_ALL);
		}
		break;
	}
}

static void demoStartTimer(void)
{
	c17startSw();
	c17enableIntSW();
}
static void demoStopTimer(void)
{
	c17stopSw();
	c17disableIntSW();
}
static unsigned int demoTimerElapse(unsigned int str, unsigned int prv)
{

	if (str >= prv)
	{
		return (str - prv);
	}
	return ((0xFFFF - prv) + str);
}

#endif
