/************************************************************************//**
 *
 * @file    main_term.c
 *
 * @brief   This is terminal mode program for S1C17M33 IR Remote controller sample.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "crt0.h"
#include "c17_init_config.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_uart3_1.h"
#include "c17_rtca_1.h"
#include "c17_remc3_1.h"
#include "c17_wdt2_1.h"
#include "main_config.h"
#include "mid_key.h"
#include "mid_host.h"
#include "mid_remrcv.h"
#include "mid_remsnd.h"
#include "mid_lcdac.h"
#include "mid_accode.h"
#include "mid_tmr.h"
#include "main_acc.h"
#include "mid_bat.h"
#include "util.h"

#ifdef MAIN_USE_TERM_MODE

/* --- definition --- */
enum
{
	TERM_RECV_STATE_START,
	TERM_RECV_STATE_RUN
};

#define TERM_RECV_TIMER MAIN_CURSOR_BLINK_TIMER
#define TERM_CMDARGS_MAX     16

/* --- structure --- */
typedef struct term_cmd_s
{
	const char *pname;
	int (*func)(char *parg[], int args);
}TERM_CMD_T;

/* --- function declaration --- */
static void termRecvIr(void);
static void termRecvCommand(void);
static void termCheckSendIrState(void);
static void termCheckReceiveTimeout(void);
static int termCheckKeyInput(void);
static void termStrToUpper(char *pstr);
static int termStringToBinary(int remcFormat, unsigned char *pdest, int *dstlen, const char *psrc);
static int termParseAugument(char *pCmd, char *pArg[], int maxarg);
static unsigned int termTimerElapse(unsigned int str, unsigned int prv);
static char* termCheckOption(char *parg, const char *poption);
static int termArgToUShort(unsigned short *pvalue, const char *parg, unsigned long min, unsigned long max);

static int termLedSetCmd(char *parg[], int args);
static int termLedSndCmd(char *parg[], int args);
static int termLedRcvCmd(char *parg[], int args);
static int termClockGetCmd(char *parg[], int args);
static int termClockSetCmd(char *parg[], int args);
static int termVersionCmd(char *parg[], int args);

/* --- const variable --- */
static const TERM_CMD_T	TermCmd[] = {
		{"LED_SET", termLedSetCmd},
		{"LED_SND", termLedSndCmd},
		{"LED_RCV", termLedRcvCmd},
		{"CLK_GET", termClockGetCmd},
		{"CLK_SET", termClockSetCmd},
		{"VER_GET", termVersionCmd},
};
const char *TermTitle = "LEd  ";

/* --- global variable --- */
char 					*TermCmdArgs[TERM_CMDARGS_MAX];
char					TermSendData[128];
char					TermOutSep = ',';
int						TermIsRecvOn = 0;
unsigned int			TermRecvTimeout = 0;
unsigned int			TermRecvTimer = 0;
unsigned char			TermIrCode[IRCODE_LENGTH_MAX];
static int				TermIsIrSending = 0;
static int				TermIsTramsIconDisp = 0;
static ST_REMC_PARAM	TermRemParam = {
		C17_REMC_FORMAT_AEHA, 38000, 425, 8, 4, 1, 1, 1, 3
};
/************************************************************************//**
 * main function
 * @brief  This is Terminal mode program.
 *
 * @retval  EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_runTerm(void)
{
	int activeState;

	/// Initialize the modules.
	mid_initHost();
	mid_initRemRcv();
	mid_initRemSnd();
	mid_initLcdAc();
	mid_initKey();
	mid_initTmr();

	/// Initialize the format of the IR remote control code.
	c17setFormatRemcDirect(&TermRemParam);
	mid_setRemRecvFormat(&TermRemParam);

	/// Main loop
	activeState = ACC_ACTIVE_STATE_STANDBY;
	while (1)
	{
		if (activeState == ACC_ACTIVE_STATE_STANDBY)
		{
			/// Turn off the interface.
			mid_offHost();
			mid_offRemSnd();
			mid_offRemRcv();
			mid_offLcdAc();
			mid_stopTmr(TERM_RECV_TIMER);
			TermIsTramsIconDisp = 0;
			TermIsIrSending     = 0;
			mid_stopTmr(MAIN_TRAMS_ICON_TIMER);

			/// Enter to standby.
			mid_setKeyWakeup(KEY_ONOFF);
			while (1)
			{
				// Set sleep mode.
									// Nop is necessary when the main program is not executed after
									// sleep is	released, and it diverges to the server blade.
				asm ("slp");
				asm ("nop");

				/// Reset the WDT.
				c17resetWdt();

				/// Check the power voltage.
				mid_chkBatLowBattery(0);

				///@check if ON/OFF key is pressed.
				if (mid_chkKeyWakeup())
				{
					break;
				}
			}
			/// Turn on the module.
			mid_onLcdAc();
			mid_onHost();
			mid_onRemSnd();
			TermIsRecvOn = 0;
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_LEFT,  0, TermTitle);
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, "     ");
			activeState = ACC_ACTIVE_STATE_ACTIVE;
		}
		else if (activeState == ACC_ACTIVE_STATE_ACTIVE)
		{
			/// Reset the WDT.
			c17resetWdt();

			/// Get key state.
			if (termCheckKeyInput())
			{
				activeState = ACC_ACTIVE_STATE_STANDBY;
				continue;
			}

			/// Receive the command via UART.
			termRecvCommand();

			/// Receive the IR remote control code.
			termRecvIr();

			/// Check sending progress of IR remote control code.
			termCheckSendIrState();

			/// check reception timeout.
			termCheckReceiveTimeout();

			/// Check the power voltage.
			mid_chkBatLowBattery(TermIsIrSending);

			// Set halt mode.
					// Nop is necessary when the main program is not executed after
					// halt is	released, and it diverges to the server blade.
			asm ("halt");
			asm ("nop");
		}
	}
	return EXIT_SUCCESS;
}
/************************************************************************//**
 * @brief  Receive the IR remote control code.
 *
 *
 * @retval  none
 ***************************************************************************/
static void termRecvIr(void)
{
	int		bitlen;

	/// Get received IR remote control code.
	if ((bitlen = mid_recvRemRcvCode(TermIrCode, sizeof(TermIrCode) * 8)) > 0)
	{
		int i;
		int len = 0;
		int	bytlen;
		char str[8];

		bytlen = bitlen / 8;
		bitlen = bitlen % 8;
		strcpy(TermSendData, "-D ");
		len = strlen(TermSendData);
		for (i = 0; i < bytlen; i++)
		{
			if ((TermOutSep != '\0') && (i > 0))
			{
				str[0] = TermOutSep;
				util_hexTo2DigitStr(&str[1], TermIrCode[i], '0');
			}
			else
			{
				util_hexTo2DigitStr(&str[0], TermIrCode[i], '0');
			}
			len += strlen(str);
			if (len >= (sizeof(TermSendData) - 1))
			{
				return;
			}
			strcat(TermSendData, str);
		}
		if (bitlen > 0)
		{
			if (TermOutSep != '\0')
			{
				str[0] = TermOutSep;
				util_hexTo2DigitStr(&str[1], TermIrCode[i] >> (8 - bitlen), '0');
			}
			else
			{
				util_hexTo2DigitStr(&str[0], TermIrCode[i] >> (8 - bitlen), '0');
			}
			len += strlen(str);
			if (len >= (sizeof(TermSendData) - 1))
			{
				return;
			}
			strcat(TermSendData, str);
		}
		mid_sendHostString(TermSendData);
		mid_sendHostString("\r\n");
	}
}
/************************************************************************//**
 * @brief  Receive the command via UART.
 *
 *
 * @retval  none
 ***************************************************************************/
static void termRecvCommand(void)
{
	char *pstr = mid_recvHostString();
	if (pstr != NULL)
	{
		if (strlen(pstr) > 0)
		{
			int		args = 0;
			int 	Result = EXIT_FAILURE;

			/// Separate  the command.
			termStrToUpper(pstr);
			args = termParseAugument(pstr, TermCmdArgs, TERM_CMDARGS_MAX);
			if (args >= 0)
			{
				int		i;
				/// Search a command.
				for (i = 0; i < (sizeof(TermCmd) / sizeof(TERM_CMD_T)); i++)
				{
					if (strcmp(pstr, TermCmd[i].pname)== 0)
					{
						/// Execute a command.
						Result = TermCmd[i].func(TermCmdArgs, args);
						break;
					}
				}
			}
			if (Result == EXIT_SUCCESS)
			{
				mid_sendHostString("Success\r\n");
			}
			else
			{
				mid_sendHostString("Command Error\r\n");
			}
		}
	}
}
/************************************************************************//**
 * @brief  Check sending progress of IR remote control code.
 *
 *
 * @retval  none
 ***************************************************************************/
static void termCheckSendIrState(void)
{
	if (!mid_chkRemSndIrSendState())
	{
		if (TermIsIrSending) {
			TermIsIrSending = 0;
		}
	}
	else
	{
		mid_startTmr(MAIN_TRAMS_ICON_TIMER);
	}
	if (TermIsTramsIconDisp)
	{
		if (mid_getTmr(MAIN_TRAMS_ICON_TIMER) > ACC_TRAMS_ICON_DISPTIME)
		{
			TermIsTramsIconDisp = 0;
			mid_stopTmr(MAIN_TRAMS_ICON_TIMER);
			mid_dispLcdAcTransmission(0);
		}

	}
}
/************************************************************************//**
 * @brief  Check reception timeout.
 *
 *
 * @retval  none
 ***************************************************************************/
static void termCheckReceiveTimeout(void)
{
	if (TermIsRecvOn && (TermRecvTimeout > 0))
	{
		if (termTimerElapse(c17rtcaIntCount, TermRecvTimer) > TermRecvTimeout)
		{
			TermIsRecvOn = 0;
			mid_offRemRcv();
			mid_stopTmr(TERM_RECV_TIMER);
			mid_sendHostString("Time out\r\n");
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 4, " ");
		}
	}
}

/************************************************************************//**
 * @brief  Execute the led_set command.
 *
 * @param   parg   Pointer to arguments.
 * @param   args   Numbers of arguments.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
static int termLedSetCmd(char *parg[], int args)
{
	int i;
	unsigned long setflg = 0;

	for (i = 0; i < args; i++)
	{
		char *pval;
		if ((pval = termCheckOption(parg[i], "FORMAT")) != NULL)
		{
			/// format
			if (strcmp(pval, "NOR") == 0)
			{
				TermRemParam.remcFormat = C17_REMC_FORMAT_NEC;
			}
			else if (strcmp(pval, "REV") == 0)
			{
				TermRemParam.remcFormat = C17_REMC_FORMAT_SONY;
			}
			else
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0001;
		}
		else if ((pval = termCheckOption(parg[i], "SUBCARRIER")) != NULL)
		{
			if (termArgToUShort(&TermRemParam.subcreer, pval, 0, 0xFFFF) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0002;
		}
		else if ((pval = termCheckOption(parg[i], "T")) != NULL)
		{
			if (termArgToUShort(&TermRemParam.t, pval, 0, 0xFFFF) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0004;
		}
		else if ((pval = termCheckOption(parg[i], "LH")) != NULL)
		{
			if (termArgToUShort(&TermRemParam.leader_h, pval, 0, 0xFFFF) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0008;
		}
		else if ((pval = termCheckOption(parg[i], "LL")) != NULL)
		{
			if (termArgToUShort(&TermRemParam.leader_l, pval, 0, 0xFFFF) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0010;
		}
		else if ((pval = termCheckOption(parg[i], "0H")) != NULL)
		{
			if (termArgToUShort(&TermRemParam.bit0_h, pval, 0, 0xFFFF) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0020;
		}
		else if ((pval = termCheckOption(parg[i], "0L")) != NULL)
		{
			if (termArgToUShort(&TermRemParam.bit0_l, pval, 0, 0xFFFF) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0040;
		}
		else if ((pval = termCheckOption(parg[i], "1H")) != NULL)
		{
			if (termArgToUShort(&TermRemParam.bit1_h, pval, 0, 0xFFFF) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0080;
		}
		else if ((pval = termCheckOption(parg[i], "1L")) != NULL)
		{
			if (termArgToUShort(&TermRemParam.bit1_l, pval, 0, 0xFFFF) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x00100;
		}
		else
		{
			return EXIT_FAILURE;
		}
	}
	if (setflg == 0x01FF)
	{
		/// update wave parameter.
		mid_setRemSndPreciedCode(0, 0);
		c17setFormatRemcDirect(&TermRemParam);
		mid_setRemRecvFormat(&TermRemParam);
		return EXIT_SUCCESS;
	}
	return EXIT_FAILURE;
}
/************************************************************************//**
 * @brief  Execute the led_snd command.
 *
 * @param   parg   Pointer to arguments.
 * @param   args   numbers of arguments.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
static int termLedSndCmd(char *parg[], int args)
{

	int i;
	int rlen = 0;
	unsigned short rep = 1;
	char *pval;

	/// Get data to send.
	mid_setRemSndPreciedCode(NULL, 0);
	for (i = 0; i < args; i++)
	{

		if ((pval = termCheckOption(parg[i], "P")) != NULL)
		{
			int tmplen = sizeof(TermIrCode);
			if (termStringToBinary(TermRemParam.remcFormat, TermIrCode, &tmplen, pval) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			if (mid_setRemSndPreciedCode(TermIrCode, tmplen) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
		}
		else if ((pval = termCheckOption(parg[i], "REP")) != NULL)
		{

		}
		else if ((pval = termCheckOption(parg[i], "D")) == NULL)
		{
			return EXIT_FAILURE;
		}
	}
	for (i = 0; i < args; i++)
	{
		if ((pval = termCheckOption(parg[i], "D")) != NULL)
		{
			if (rlen > 0)
			{
				return EXIT_FAILURE;
			}
			rlen = sizeof(TermIrCode);
			if (termStringToBinary(TermRemParam.remcFormat, TermIrCode, &rlen, pval) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			
		}
		else if ((pval = termCheckOption(parg[i], "REP")) != NULL)
		{
			if (termArgToUShort(&rep, pval, 0, 10) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
		}
		else if ((pval = termCheckOption(parg[i], "P")) == NULL)
		{
			return EXIT_FAILURE;
		}
	}
		
	/// Send the data.
	if (rlen > 0)
	{
		if (!TermIsIrSending)
		{
			TermIsIrSending     = 1;
			TermIsTramsIconDisp = 1;
			mid_dispLcdAcTransmission(1);
			mid_startTmr(MAIN_TRAMS_ICON_TIMER);
			mid_chkBatLowBattery(1);
		}
		return mid_reqRemSndIrSendCode(rep, TermIrCode, rlen);
	}

	return EXIT_FAILURE;
}
/************************************************************************//**
 * @brief  Execute the led_rcv command.
 *
 * @param   parg   Pointer to arguments.
 * @param   args   Numbers of arguments.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
static int termLedRcvCmd(char *parg[], int args)
{
	int i;
	unsigned short timeoutTmp = 0;
	unsigned short sepalator = 1;

	for (i = 0; i < args; i++)
	{
		char *pval;
		if ((pval = termCheckOption(parg[i], "TIMEOUT")) != NULL)
		{
			if (termArgToUShort(&timeoutTmp, pval, 0, 60000) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
		}
		else if ((pval = termCheckOption(parg[i], "SEPARATOR")) != NULL)
		{
			if (termArgToUShort(&sepalator, pval, 0, 1) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
		}
		else
		{
			return EXIT_FAILURE;
		}
	}
	TermOutSep      = sepalator ? ',' : '\0';
	TermIsRecvOn    = 1;
	TermRecvTimeout = timeoutTmp;
	TermRecvTimer   = c17rtcaIntCount;
	mid_startTmr(TERM_RECV_TIMER);
	mid_onRemRcv();
	mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 4, "R");
	return EXIT_SUCCESS;
}

/************************************************************************//**
 * @brief  Execute the clock_get command.
 *
 * @param   parg   Pointer to arguments.
 * @param   args   Numbers of arguments.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
static int termClockGetCmd(char *parg[], int args)
{
	if (args == 0)
	{
		ST_RTCA_BCD cur;
		unsigned short numhour;
		c17getRtcaDate(&cur);
		numhour = util_convUShortBcdToNum(cur.hour);
		if ((c17getRtcaMode() == C17_RTCA_12H) && (cur.ampm == C17_RTCA_PM))
		{
			numhour += 12;
			cur.hour = util_convUShortNumToBcd(numhour);
		}
		util_bcdTo2DigitStr(&TermSendData[0], cur.year, '0');
		TermSendData[2] = '/';
		util_bcdTo2DigitStr(&TermSendData[3], cur.month, '0');
		TermSendData[5] = '/';
		util_bcdTo2DigitStr(&TermSendData[6], cur.day, '0');
		TermSendData[8] = '_';
		util_hexTo2DigitStr(&TermSendData[9], cur.week, '0');
		TermSendData[11] = '_';
		util_bcdTo2DigitStr(&TermSendData[12], cur.hour, '0');
		TermSendData[14] = ':';
		util_bcdTo2DigitStr(&TermSendData[15], cur.minute, '0');
		TermSendData[17] = ':';
		util_bcdTo2DigitStr(&TermSendData[18], cur.second, '0');
		TermSendData[20] = ' ';
		TermSendData[21] = '\0';
		mid_sendHostString(TermSendData);
		return EXIT_SUCCESS;
	}
	return EXIT_FAILURE;
}
/************************************************************************//**
 * @brief  Execute the clock_set command.
 *
 * @param   parg   Pointer to arguments.
 * @param   args   Numbers of arguments.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
static int termClockSetCmd(char *parg[], int args)
{
	unsigned long setflg = 0;
	int i;
	unsigned short numyear;
	unsigned short nummonth;
	unsigned short numday;
	unsigned short numhour;
	unsigned short numminute;
	unsigned short numsecond;

	for (i = 0; i < args; i++)
	{
		char *pval;
		if ((pval = termCheckOption(parg[i], "YEAR")) != NULL)
		{
			if (termArgToUShort(&numyear, pval, 0, 99) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0001;
		}
		else if ((pval = termCheckOption(parg[i], "MONTH")) != NULL)
		{
			if (termArgToUShort(&nummonth, pval, 1, 12) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0002;
		}
		else if ((pval = termCheckOption(parg[i], "DAY")) != NULL)
		{
			if (termArgToUShort(&numday, pval, 1, 31) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0004;
		}
		else if ((pval = termCheckOption(parg[i], "HOUR")) != NULL)
		{
			if (termArgToUShort(&numhour, pval, 0, 23) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0008;
		}
		else if ((pval = termCheckOption(parg[i], "MINUTE")) != NULL)
		{
			if (termArgToUShort(&numminute, pval, 0, 59) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0010;
		}
		else if ((pval = termCheckOption(parg[i], "SECOND")) != NULL)
		{
			if (termArgToUShort(&numsecond, pval, 0, 59) == EXIT_FAILURE)
			{
				return EXIT_FAILURE;
			}
			setflg |= 0x0020;
		}
	}
	if (setflg == 0x3F)
	{
		ST_RTCA_BCD rtcval;

		if ((c17getRtcaMode() == C17_RTCA_12H) && (numhour >= 12))
		{
			numhour -= 12;
			rtcval.ampm   = C17_RTCA_PM;
		}
		else
		{
			rtcval.ampm   = C17_RTCA_AM;
		}
		rtcval.year   = util_convUShortNumToBcd(numyear);
		rtcval.month  = util_convUShortNumToBcd(nummonth);
		rtcval.day    = util_convUShortNumToBcd(numday);
		rtcval.week   = util_getWeek(numyear, nummonth, numday);
		rtcval.hour   = util_convUShortNumToBcd(numhour);
		rtcval.minute = util_convUShortNumToBcd(numminute);
		rtcval.second = util_convUShortNumToBcd(numsecond);
		c17setRtcaDate(&rtcval);
		return EXIT_SUCCESS;
	}
	return EXIT_FAILURE;
}

/************************************************************************//**
 * @brief  Execute the ver_get command.
 *
 * @param   parg   Pointer to arguments.
 * @param   args   numbers of arguments.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
static int termVersionCmd(char *parg[], int args)
{
	mid_sendHostString("-date ");
	mid_sendHostString(__DATE__);
	mid_sendHostString(" ");
	mid_sendHostString("-time ");
	mid_sendHostString(__TIME__);
	mid_sendHostString(" ");
	return EXIT_SUCCESS;
}

/************************************************************************//**
 * @brief  Check key input.
 *
 *
 * @retval  1  ON/OFF key is pushed.
 * @retval  0  ON/OFF key is not pushed.
 ***************************************************************************/
int termCheckKeyInput(void)
{
	KEY_STATE_S keystate;
	mid_getKeyState(&keystate);
	if (keystate.keycode == KEY_ONOFF)
	{
		if ((keystate.keyevent == KEY_EVENT_PRESS) ||
			(keystate.keyevent == KEY_EVENT_PRESS_LIMIT))
		{
			return 1;
		}
	}
	else if (keystate.keycode != KEY_NONE)
	{
		if (keystate.keyevent != KEY_EVENT_NONE)
		{
			mid_sendHostString("-key ");
			util_hexTo2DigitStr(&TermSendData[0], (keystate.keycode >>  8) & 0xFF, '0');
			util_hexTo2DigitStr(&TermSendData[2], keystate.keycode & 0xFF, '0');
			TermSendData[4] = ' ';
			TermSendData[5] = '\0';
			mid_sendHostString(TermSendData);
			switch(keystate.keyevent)
			{
			case KEY_EVENT_PRESS:
				mid_sendHostString("PRESS");
				break;
			case KEY_EVENT_LONG_PRESS:
				mid_sendHostString("LONG PRESS");
				break;
			case KEY_EVENT_REPEAT:
				mid_sendHostString("REPEAT");
				break;
			case KEY_EVENT_RELEASE:
				mid_sendHostString("RELEASE");
				break;
			case KEY_EVENT_PRESS_LIMIT:
				mid_sendHostString("PRESS LIMIT");
				break;
			}
			mid_sendHostString("\r\n");
		}
	}
	return 0;
}
/************************************************************************//**
 * @brief  Check the option for command.
 *
 * @param   parg     Pointer to argument in command.
 * @param   poption  Name of option.
 *
 * @retval  NULL	Failed.
 * @retval  other   Pointer to option value in the parg.
 ***************************************************************************/
static char* termCheckOption(char *parg, const char *poption)
{
	while (1)
	{
		if (*parg != *poption)
		{
			break;
		}
		if (*parg == '\0')
		{
			break;
		}
		parg++;
		poption++;
		if (*poption == '\0')
		{
			if (*parg == ' ')
			{
				return (parg + 1);
			}
			break;
		}
	}
	return NULL;
}

/************************************************************************//**
 * @brief  Parse argument in command.
 *
 * @param   pCmd   Pointer to command buffer.
 * @param   pArg   Pointer to argument pointer array.
 * @param   maxarg maximum numbers of argument pointer array.
 * @retval  -1            Failed.
 * @retval  0 and more    numbers of argument.
 ***************************************************************************/
int termParseAugument(char *pCmd, char *pArg[], int maxarg)
{
	int args = 0;
	char *pcur = pCmd;
	char *pprv = pCmd;
	int optpos = 0;

	/// get position of command name.
	while (1)
	{
		if (*pcur == ' ')
		{
			*pcur = '\0';
			pprv  = pcur;
			pcur++;
			break;
		}
		else if (*pcur == '\0')
		{
			return args;
		}
		pcur++;
	}


	/// get position of arguments.
	while (1)
	{
		if (optpos == 0)
		{
			/// wait for first option.
			if (*pcur == '-')
			{
				if (args >= maxarg)
				{
					break;
				}
				pArg[args++] = pprv + 1;
				optpos++;
			}
			else if (*pcur != ' ')
			{
				break;
			}
		}
		else if (optpos == 1)
		{
			/// get option name.
			*(++pprv) = *pcur;
			if (*pcur == '\0')
			{
				break;
			}
			else if (*pcur == ' ')
			{
				optpos++;
			}
		}
		else if (optpos == 2)
		{
			/// wait for value.
			if ((*pcur == '-') || (*pcur == '\0'))
			{
				break;
			}
			else if (*pcur != ' ')
			{
				*(++pprv) = *pcur;
				optpos++;
			}
		}
		else if (optpos == 3)
		{
			/// Get value.
			if (*pcur == '\0')
			{
				if ((*pprv == ' ') || (*pprv == ','))
				{
					break;
				}
				*(++pprv) = '\0';
				return args;
			}
			else if (*pcur == '-')
			{
				if (*pprv == ' ')
				{
					if (args >= maxarg)
					{
						break;
					}
					*(pprv++) = '\0';
					pArg[args++] = pprv + 1;
					optpos = 1;
				}
				else
				{
					break;
				}
			}
			else if (*pcur == ' ')
			{
				if (*pprv != ' ')
				{
					*(++pprv) = *pcur;
				}
			}
			else if (*pcur == ',')
			{
				if (*pprv != ',')
				{
					*(++pprv) = *pcur;
				}
			}
			else
			{
				*(++pprv) = *pcur;
			}
		}
		pcur++;
	}
	return -1;

}


static void termStrToUpper(char *pstr)
{
	while (*pstr != '\0')
	{
		if ((*pstr >='a') && (*pstr <= 'z'))
		{
			*pstr = (*pstr - 'a') + 'A';
		}
		pstr++;
	}
}

static int termStringToBinary(int remcFormat, unsigned char *pdest, int *dstlen, const char *psrc)
{
	char *plast;
	char str[4];
	char prv = ',';
	int length = 0;
	int bcnt = 0;
	int dstmax = *dstlen;

	str[2] = 0;
	while (*psrc != '\0')
	{
		if ((*psrc == ',') || (*psrc == ' '))
		{
			if ((bcnt > 0) || (prv == ',') || (prv == ' '))
			{
				return EXIT_FAILURE;
			}
		}
		else
		{
			str[bcnt] = *psrc;
			if (++bcnt >= 2) {
				bcnt = 0;
				if (length >= dstmax)
				{
					return EXIT_FAILURE;
				}
				pdest[length] = (unsigned char)strtoul(str, &plast, 16);
				if (*plast != '\0')
				{
					return EXIT_FAILURE;
				}
				length++;
			}
		}
		prv = *psrc;
		psrc++;
	}
	if (bcnt == 1) {
		str[1] = '\0';
		if (length >= dstmax)
		{
			return EXIT_FAILURE;
		}
		pdest[length] = (unsigned char)strtoul(str, &plast, 16);
		if (*plast != '\0')
		{
			return EXIT_FAILURE;
		}
		if ((remcFormat != C17_REMC_FORMAT_SONY) || (length == 0))
		{
			pdest[length] <<= 4;
		}

		length++;
	} else {
		if (remcFormat == C17_REMC_FORMAT_SONY)
		{
			if (length > 1)
			{
				pdest[length - 1] >>= 4;
			}
		}
	}
	
	if (remcFormat == C17_REMC_FORMAT_SONY)
	{
		if (length < 2)
		{
			pdest[1] = 0x00;
			length++;
		}
	}
	*dstlen = length;
	return EXIT_SUCCESS;
}

static unsigned int termTimerElapse(unsigned int str, unsigned int prv)
{

	if (str >= prv)
	{
		return (str - prv);
	}
	return ((0xFFFF - prv) + str);
}
static int termArgToUShort(unsigned short *pvalue, const char *parg, unsigned long min, unsigned long max)
{
	char *plast;
	unsigned long val;
	val = strtoul(parg, &plast, 10);
	if (*plast != '\0')
	{
		return EXIT_FAILURE;
	}
	if (val > max)
	{
		return EXIT_FAILURE;
	}
	if (val < min)
	{
		return EXIT_FAILURE;
	}
	*pvalue = (unsigned short)val;
	return EXIT_SUCCESS;
}
#endif
