/************************************************************************//**
 *
 * @file    main_test.c
 *
 * @brief   This is test mode program for S1C17M33 IR Remote controller sample.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "c17_clg.h"
#include "c17_rtca_1.h"
#include "c17_wdt2_1.h"
#include "main_config.h"
#include "util.h"
#include "mid_key.h"
#include "mid_lcdac.h"
#include "mid_tmr.h"
#include "main_acc.h"
#include "mid_accode.h"
#include "mid_bat.h"
#include "mid_remrcv.h"
#include "mid_remsnd.h"
#include "mid_host.h"
#include "mid_dipsw.h"

#ifdef MAIN_USE_TEST_MODE
/* --- definition --- */
enum
{
	TEST_ITEM_RTC,
	TEST_ITEM_DIPSW,
	TEST_ITEM_KEY,
	TEST_ITEM_REMOTE,
	TEST_ITEM_LCD,
	TEST_ITEM_UART,
	TEST_ITEM_SUCCESS,
	TEST_ITEM_SEND,
};
enum
{
	TEST_RECV_WAIT,
	TEST_RECV_OK,
	TEST_RECV_NG
};
#define TEST_REMOTE_LENGTH	12

/* --- structure --- */
typedef struct test_lcd_seg_s{
	const LCD_SEG_POS_S *pseg;
	int length;
}TEST_LCD_SEG_T;

/* --- function --- */
static void testSetItem(int newItem);
static int testExecItem(void);
static void testSwNumberDisp(int num);
static void testSendNumberDisp(int num);
static void testDipBitDisp(int num);
static void testLcdSegDisp(int item1, int item2);
static void testSendIrCode(void);
static int testCheckIrSendState(void);
static int testRecvData(void);
static void testSendUart(void);
static void testStartTimer(void);
static void testStopTimer(void);
static unsigned int testTimerElapse(unsigned int, unsigned int prv);

/* --- external declaration --- */
extern unsigned char TermIrCode[];

/* --- constant variables ---*/
#define TEST_KEY_COUNT		12
#define TEST_KEY_OFFSET		4
const char TestKeyOrder[TEST_KEY_COUNT] = {5, 6, 7, 4, 9, 11, 8, 10, 12, 13, 14, 15};
const unsigned char TestDip[] = {
					DIPSW_BIT_SW4 | DIPSW_BIT_SW3 | DIPSW_BIT_SW2,
					DIPSW_BIT_SW4 | DIPSW_BIT_SW3,
					DIPSW_BIT_SW4,
					0x0
};
const TEST_LCD_SEG_T TestLcdSeg[] = {
		{&LcdTransmissionIcon, 1},
		{LcdModeItemIcon, LCDAC_MODE_ITEM_COUNT},
		{&LcdSeparatorIcon,1},
		{LcdTempUpper7seg, LCDAC_7SEG_COUNTS - 1},
		{LcdTempLower7seg, LCDAC_7SEG_COUNTS - 1},
		{LcdTempUnitIcon, 2},
		{LcdBatteryIcon, 3},
		{&LcdWindHoriSymbolIcon, 1},
		{LcdWindHoriItemIcon, LCDAC_WIND_HORI_ITEM_COUNT},
		{&LcdWindVertSymbolIcon, 1},
		{LcdWindVertItemIcon, LCDAC_WIND_HORI_ITEM_COUNT},
		{LcdWindFanIcon, LCDAC_WIND_FAN_ITEM_COUNT},
		{LcdWeekIcon, 7},
		{&LcdTimerOnIcon, 1},
		{LcdLeftDayTimeAmPm, 2},
		{LcdLeftDayTime7seg[0], LCDAC_7SEG_COUNTS - 1},
		{LcdLeftDayTime7seg[1], LCDAC_7SEG_COUNTS - 1},
		{LcdLeftDayTime7seg[2], LCDAC_7SEG_COUNTS},
		{LcdLeftDayTime7seg[3], LCDAC_7SEG_COUNTS - 1},
		{LcdLeftDayTime7seg[4], LCDAC_7SEG_COUNTS - 1},
		{&LcdTimerOffIcon, 1},
		{LcdRightDayTimeAmPm, 2},
		{LcdRightDayTime7seg[0], LCDAC_7SEG_COUNTS - 1},
		{LcdRightDayTime7seg[1], LCDAC_7SEG_COUNTS - 1},
		{LcdRightDayTime7seg[2], LCDAC_7SEG_COUNTS},
		{LcdRightDayTime7seg[3], LCDAC_7SEG_COUNTS - 1},
		{LcdRightDayTime7seg[4], LCDAC_7SEG_COUNTS - 1},

};
const char *TestUartData = "0123456789ABCEF";

/* --- static variables ---*/
static int TestItem = TEST_ITEM_KEY;
static int TestItemStep;
static int TestItemStep2;
static unsigned int TestSend100hz;
static unsigned int TestLcd100hz;
static int TestLcdPause;
static int TestIsIrSending = 0;

/************************************************************************//**
 * main function
 * @brief  Test mode program.
 *
 * @retval  EXIT_SUCCESS    Success.
 ***************************************************************************/
int main_runTest(void)
{
	int activeState;

	/// Initialize the module.
	mid_initHost();
	mid_initRemSnd();
	mid_initRemRcv();
	mid_initLcdAc();
	mid_initKey();
	mid_initTmr();
	
	/// Main loop
	activeState = ACC_ACTIVE_STATE_STANDBY;
	while (1)
	{
		if (activeState == ACC_ACTIVE_STATE_STANDBY)
		{
			/// Turn off module.
			mid_offHost();
			mid_offRemSnd();
			mid_offRemRcv();
			mid_offLcdAc();
			testStopTimer();

			/// Enter to standby.
			mid_setKeyWakeup(KEY_ONOFF);
			while (1)
			{
				// Set sleep mode.
									// Nop is necessary when the main program is not executed after
									// sleep is	released, and it diverges to the server blade.
				asm ("slp");
				asm ("nop");

				/// Reset the WDT.
				c17resetWdt();

				/// Check the power voltage.
				mid_chkBatLowBattery(0);

				///@check if ON/OFF key is pressed.
				if (mid_chkKeyWakeup())
				{
					break;
				}
			}
			/// Turn on module
			activeState = ACC_ACTIVE_STATE_ACTIVE;
			mid_onLcdAc();
			mid_onHost();
			mid_onRemSnd();
			mid_onRemRcv();
			testStartTimer();
			testSetItem(TEST_ITEM_RTC);
		}
		else if (activeState == ACC_ACTIVE_STATE_ACTIVE)
		{
			/// Reset the WDT.
			c17resetWdt();

			/// Test Mode
			activeState = testExecItem();

			/// Check the power voltage.
			mid_chkBatLowBattery(TestIsIrSending);

			// Set halt mode.
					// Nop is necessary when the main program is not executed after
					// halt is	released, and it diverges to the server blade.
			asm ("halt");
			asm ("nop");
		}
	}
	return EXIT_SUCCESS;
}
/************************************************************************//**
 * @brief  Set next test item.
 *
 * @param  nextItem   Index of a next item.
 *
 * @retval  none
 ***************************************************************************/
static void testSetItem(int newItem)
{
	c17clearLcd8a();
	TestItem = newItem;
	TestItemStep = 0;
	TestItemStep2 = 0;
	mid_dispLcdAcTemperature(ACC_TEMP_UNIT_CELSIUS, newItem + 1);
	switch(newItem)
	{
	case TEST_ITEM_RTC:
		// Test item
		// RTC
		c17enableFout(C17_CLG_FOUT_OSC1, C17_CLG_FOUT_CLKDIV_1);
		main_updateClockDisp();
		break;
	case TEST_ITEM_DIPSW:
		// Test item
		// DIP SW
		testDipBitDisp(TestItemStep);
		break;
	case TEST_ITEM_KEY:
		// Test item
		// Key
		testSwNumberDisp(TestKeyOrder[0]);
		break;
	case TEST_ITEM_REMOTE:
		// Test item
		// Remote
		testSendIrCode();
		break;
	case TEST_ITEM_LCD:
		// Test item
		// LCD
		c17clearLcd8a();
		testLcdSegDisp(TestItemStep, TestItemStep2);
		TestLcdPause = 0;
		break;
	case TEST_ITEM_UART:
		// Test item
		// UART
		testSendUart();
		break;
	case TEST_ITEM_SUCCESS:
		c17setLcd8aDisplayState(C17_LCD_STATE_ALL_ON);
		break;
	case TEST_ITEM_SEND:
		c17clearLcd8a();
		c17setLcd8aDisplayState(C17_LCD_STATE_NORMAL);
		mid_setLcdAcSegPosition(LCDAC_DISPLAY_AREA, &TestLcdSeg[2].pseg[0], LCDAC_SEGMENT_ON);
		break;
	}
}
/************************************************************************//**
 * @brief  Execute test item.
 *
 * @retval  next state.
 ***************************************************************************/
static int testExecItem(void)
{
	KEY_STATE_S keystate;
	int nextState = ACC_ACTIVE_STATE_ACTIVE;

	mid_getKeyState(&keystate);
	switch(TestItem)
	{
	case TEST_ITEM_RTC:
		/// Test item
		/// RTC
		if (keystate.keyevent == KEY_EVENT_PRESS)
		{
			testSetItem(TEST_ITEM_DIPSW);
		}
		else
		{
			ST_RTCA_BCD cur;
			char str[12];

			c17getRtcaDate(&cur);
			mid_dispLcdAcTime(LCDAC_DAYTIMEPOS_LEFT, c17getRtcaMode(), cur.ampm, cur.hour, cur.minute);
			util_bcdTo2DigitStr(str,cur.second, '0');
			mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, str);
		}
		break;
	case TEST_ITEM_DIPSW:
		/// Test item
		/// DIP SW
		if (mid_getDipSwState() == TestDip[TestItemStep])
		{
			if (++TestItemStep >= sizeof(TestDip))
			{
				testSetItem(TEST_ITEM_KEY);
			}
			else
			{
				testDipBitDisp(TestItemStep);
			}
		}
		if (keystate.keyevent == KEY_EVENT_PRESS)
		{
			testSetItem(TEST_ITEM_KEY);
		}
		break;
	case TEST_ITEM_KEY:
		/// Test item
		/// Key input.
		if (keystate.keyevent == KEY_EVENT_PRESS)
		{
			unsigned long code = (unsigned long)1UL << (TestKeyOrder[TestItemStep] - TEST_KEY_OFFSET);
			if (code == keystate.keycode)
			{
				if (++TestItemStep >= TEST_KEY_COUNT)
				{
					testSetItem(TEST_ITEM_REMOTE);
				}
				else
				{
					testSwNumberDisp(TestKeyOrder[TestItemStep]);
				}
			}
		}
		break;
	case TEST_ITEM_REMOTE:
		/// Test item.
		/// Remote control.
		if (TestItemStep == 0) {
			int result = TEST_RECV_WAIT;
			/// check if transmit is done.
			if (!testCheckIrSendState())
			{
				/// check result of reception.
				if ((result = testRecvData()) == TEST_RECV_WAIT)
				{
					if (testTimerElapse(c17swIntCount100, TestSend100hz) >= 100)
					{
						if (++TestItemStep2 < 10) {
							testSendIrCode();
						} else {
							mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, "Error");
							TestItemStep = 1;
						}
					}
				}
				else if (result == TEST_RECV_OK)
				{
					testSetItem(TEST_ITEM_LCD);
				}
			}
			else
			{
				TestSend100hz = c17swIntCount100;
			}
		}
		break;
	case TEST_ITEM_LCD:
		/// Test item.
		/// LCD.
		if (!TestLcdPause)
		{
			if (testTimerElapse(c17swIntCount100, TestLcd100hz) >= 50)
			{
				if (++TestItemStep2 >= TestLcdSeg[TestItemStep].length )
				{
					TestItemStep2 = 0;
					if (++TestItemStep >= (sizeof(TestLcdSeg) / sizeof(TEST_LCD_SEG_T)))
					{
						testSetItem(TEST_ITEM_UART);
					}
					else
					{
						testLcdSegDisp(TestItemStep, TestItemStep2);
					}
				}
				else
				{
					testLcdSegDisp(TestItemStep, TestItemStep2);
				}

			}
		}
		if (keystate.keyevent == KEY_EVENT_PRESS)
		{
			if (keystate.keycode == KEY_OK)
			{
				testSetItem(TEST_ITEM_UART);
			}
			else if (keystate.keycode == KEY_OPT)
			{
				TestLcdPause ^= 1;
				testLcdSegDisp(TestItemStep, TestItemStep2);
			}
		}
		break;
	case TEST_ITEM_UART:
		/// Test item.
		/// UART.
		/// Check receive data.
		if (TestItemStep == 0) {
			char *pstr;
			pstr = mid_recvHostString();
			if (pstr != NULL)
			{
				if (strcmp(pstr, TestUartData) == 0)
				{
					testSetItem(TEST_ITEM_SUCCESS);
				}
			}
			else if (testTimerElapse(c17swIntCount100, TestSend100hz) >= 50)
			{
				if (++TestItemStep2 < 10) {
					testSendUart();
				} else {
					mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, "Error");
					TestItemStep = 1;
				}
			}

		} else {
			if (keystate.keyevent == KEY_EVENT_PRESS)
			{
				testSetItem(TEST_ITEM_SUCCESS);
			}
		}
		break;
	case TEST_ITEM_SUCCESS:
		/// Test item.
		/// Success.
		if ((keystate.keyevent == KEY_EVENT_PRESS) && (keystate.keycode == KEY_ONOFF))
		{
			nextState = ACC_ACTIVE_STATE_STANDBY;
		}
		else if ((keystate.keyevent == KEY_EVENT_PRESS) && (keystate.keycode == KEY_OPT))
		{
			testSetItem(TEST_ITEM_SEND);
		}
		break;
	case TEST_ITEM_SEND:
		if ((keystate.keyevent == KEY_EVENT_PRESS) && (keystate.keycode == KEY_MODE))
		{
			testSendIrCode();
			++TestItemStep;
		}
		testCheckIrSendState();
		break;
	}
	return nextState;
}


static void testSendIrCode(void)
{
	int i;
	unsigned char sum = 0;

	testSendNumberDisp(TestItemStep2 + 1);

	//Set testing data.
	for (i = 0; i < (TEST_REMOTE_LENGTH- 1); i++)
	{
		TermIrCode[i] = (unsigned char)i;
		sum ^= TermIrCode[i];
	}
	TermIrCode[i++] = sum;

	//Request to send.
	if (!TestIsIrSending)
	{
		mid_dispLcdAcTransmission(1);
		mid_chkBatLowBattery(1);
		TestIsIrSending = 1;
	}
	mid_reqRemSndIrSendCode(1, TermIrCode, i);
	memset(TermIrCode, 0x00, TEST_REMOTE_LENGTH);
	TestSend100hz = c17swIntCount100;
}

static int testCheckIrSendState(void)
{
	if (!mid_chkRemSndIrSendState())
	{
		if (TestIsIrSending)
		{
			mid_dispLcdAcTransmission(0);
			TestIsIrSending = 0;
		}
		return 0;
	}
	return 1;
}

static int testRecvData(void)
{
	int irlen;

	if ((irlen = mid_recvRemRcvCode(TermIrCode, (TEST_REMOTE_LENGTH + 2) * 8)) > 0)
	{
		irlen /= 8;
		if (irlen == TEST_REMOTE_LENGTH)
		{
			int i;
			unsigned char sum = 0;
			for (i = 0; i < TEST_REMOTE_LENGTH; i++)
			{
				if ((unsigned char)i != TermIrCode[i])
				{
					break;
				}
				sum ^= TermIrCode[i];
			}
			if ((sum == 0) && (i == TEST_REMOTE_LENGTH))
			{
				return TEST_RECV_OK;
			}
		}
		return TEST_RECV_NG;
	}
	return TEST_RECV_WAIT;
}
static void testSendUart(void)
{
	testSendNumberDisp(TestItemStep2 + 1);
	mid_sendHostString(TestUartData);
	mid_sendHostString("\r\n");
	TestSend100hz = c17swIntCount100;
}
static void testLcdSegDisp(int item1, int item2)
{
	mid_setLcdAcSegPosition(LCDAC_DISPLAY_AREA, &TestLcdSeg[item1].pseg[item2], LCDAC_SEGMENT_ON);
	TestLcd100hz = c17swIntCount100;
}

static void testSwNumberDisp(int num)
{
	char str[12];
	str[0] = ' ';
	str[1] = 'S';
	str[2] = ':';
	util_decTo2DigitStr(&str[3], num, '0');
	mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, str);
}

static void testSendNumberDisp(int num)
{
	char str[12];
	str[0] = ' ';
	str[1] = ' ';
	util_decTo3DigitStr(&str[2], num, '0');
	mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, str);
}

static void testDipBitDisp(int num)
{
	char str[12];
	int i;
	unsigned char mask = 0x08;
	str[0] = 'b';
	
	for (i = 1; i <= 4; i++, mask >>= 1)
	{
		str[i] = '0' + ((TestDip[num] & mask) ? 1 : 0);
	}
	str[i] = '\0';
	mid_dispLcdAcClockArea(LCDAC_DAYTIMEPOS_RIGHT, 0, str);
}

static void testStartTimer(void)
{
	c17startSw();
	c17enableIntSW();
}
static void testStopTimer(void)
{
	c17stopSw();
	c17disableIntSW();
}
static unsigned int testTimerElapse(unsigned int str, unsigned int prv)
{
	if (str >= prv)
	{
		return (str - prv);
	}
	return ((0xFFFF - prv) + str);
}

#endif
