/************************************************************************//**
 *
 * @file    mid_acccode.c
 *
 * @brief   This is IR remote control code transmission interface program for the A/C.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include "mid_accode.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "crt0.h"
#include "c17_misc.h"
#include "main_config.h"
#include "c17_init_config.h"
#include "c17_rtca_1.h"
#include "c17_remc3_1.h"
#include "mid_remsnd.h"
#include "mid_lcdac.h"
#include "util.h"





/* --- static function --- */
static void acCodeClearCode(REMAC_IRCODE_S *pcode);
static unsigned short acCodeTimeToSerial(unsigned short rtcMode, unsigned short amPm, unsigned short bcdHour, unsigned short bcdMinute);

/* --- constant variable --- */
const unsigned char AcCodePrecede[] = {
		0x02, 0x20, 0xE0, 0x04, 0x00, 0x00, 0x00, 0x06
};


/************************************************************************//**
 * @brief  Initialize this interface.
 *
 * @param  pCodeInfo      Pointer to union to initialize data.
 *
 * @retval  none
 ***************************************************************************/
void mid_initAcCode(REMAC_IRCODE_S *pIrCode)
{
	acCodeClearCode(pIrCode);
#if ACC_PRECEDE_CODE_ENABLE == 1
	mid_setRemSndPreciedCode(AcCodePrecede, sizeof(AcCodePrecede));
#endif

}

/************************************************************************//**
 * @brief  Convert the setting parameter for Air conditioner to IR remote control code.
 *
 * @param  pCodeInfo      Pointer to union to copy converted data.
 * @param  pAcSettings    Pointer to structure of A/C settings.
 *
 * @retval  none
 ***************************************************************************/
void mid_convAcCode(REMAC_IRCODE_S *pIrCode, const ACC_AC_SETTINGS_S *pAcSettings)
{
	ST_RTCA_BCD cur;
	const unsigned char mode[] = {0, 3, 4, 6, 2};
	const unsigned char hflap[] = {0xD, 0x9, 0xA, 0x6, 0xB, 0xC};
	unsigned short curTime;
	unsigned short onTime = (pAcSettings->OnTimer.hour * 60) + pAcSettings->OnTimer.minute;
	unsigned short offTime = (pAcSettings->OffTimer.hour * 60) + pAcSettings->OffTimer.minute;



	if (pAcSettings->OnTimer.enable || pAcSettings->OffTimer.enable)
	{
		c17getRtcaDate(&cur);
		curTime = acCodeTimeToSerial(
							c17getRtcaMode(), cur.ampm, cur.hour, cur.minute);
		onTime  = (pAcSettings->OnTimer.hour * 60) + pAcSettings->OnTimer.minute;
		offTime = (pAcSettings->OffTimer.hour * 60) + pAcSettings->OffTimer.minute;
	}
	else
	{
		curTime = 0x600;
		onTime  = 0x600;
		offTime = 0x600;
	}

	pIrCode->field.acOnOff        = pAcSettings->AcOnOff;
	pIrCode->field.tempriture     = pAcSettings->Temp * 2;
	pIrCode->field.windVert       = (pAcSettings->WindVert == 0) ? 0xF : pAcSettings->WindVert;
	pIrCode->field.windHori       = hflap[pAcSettings->WindHori];
	pIrCode->field.windFan        = (pAcSettings->WindFan == 0) ? 0xA : (pAcSettings->WindFan + 2);
	pIrCode->field.Mode           = mode[pAcSettings->Mode];
	pIrCode->field.onTimerEnable  = pAcSettings->OnTimer.enable;
	pIrCode->field.offTimerEnable = pAcSettings->OffTimer.enable;
	pIrCode->field.onTimerL       = onTime & 0xF;
	pIrCode->field.onTimerM       = (onTime >> 4) & 0xF;
	pIrCode->field.onTimerH       = ((onTime >> 8) & 0xF) | 0x8;
	pIrCode->field.offTimerL      = offTime & 0xF;
	pIrCode->field.offTimerM      = (offTime >> 4) & 0xF;
	pIrCode->field.offTimerH      = ((offTime >> 8) & 0xF) | 0x8;
	pIrCode->field.timeL          = curTime & 0xF;
	pIrCode->field.timeM          = (curTime >> 4) & 0xF;
	pIrCode->field.timeH          = (curTime >> 8) & 0xF;
	pIrCode->field.code16         = 0x80;
	pIrCode->field.bcc            = mid_calcRemSndSum(pIrCode->code, sizeof(pIrCode->code) - 1);


}


static void acCodeClearCode(REMAC_IRCODE_S *pcode)
{
	pcode->field.code1 = 0x02;
	pcode->field.code2 = 0x20;
	pcode->field.code3 = 0xE0;
	pcode->field.code4 = 0x04;
	pcode->field.code5 = 0;
	pcode->field.timerPad1 = 1;
	pcode->field.code8 = 0x80;
	pcode->field.windPad = 0;
	pcode->field.onTimerL = 0;
	pcode->field.onTimerM = 0;
	pcode->field.onTimerH = 0xE;
	pcode->field.onTimerL = 0;
	pcode->field.offTimerM = 0;
	pcode->field.offTimerH = 0xE;
	pcode->field.code14 = 0x00;
	pcode->field.code15 = 0x00;
	pcode->field.code16 = 0x89;
	pcode->field.timeL = 0;
	pcode->field.timeM = 0;
	pcode->field.timeH = 0;
	pcode->field.code18 = 0;
}

static unsigned short acCodeTimeToSerial(unsigned short rtcMode, unsigned short amPm, unsigned short bcdHour, unsigned short bcdMinute)
{
	unsigned short numHour = util_convUShortBcdToNum(bcdHour);
	if ((rtcMode == C17_RTCA_12H) && (amPm == C17_RTCA_PM))
	{
		numHour += 12;
	}
	return ((numHour * 60) + util_convUShortBcdToNum(bcdMinute));

}

