/************************************************************************//**
 *
 * @file    mid_bat.c
 *
 * @brief   This is battery level monitoring interface program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include "mid_bat.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_port_1.h"
#include "main_config.h"
#include "c17_init_config.h"
#include "c17_svd3_1.h"
#include "c17_wdt2_1.h"
#include "mid_tmr.h"

/* --- definition --- */
enum
{
	BAT_STATE_STOP,
	BAT_STATE_ANALYZING,
	BAT_STATE_COMPLETE,
};

/* --- static function --- */
static int batAnalyzeLobattery(void);
static void batStartSvd(void);
static void batStopSvd(void);

/* --- external declaration --- */
extern volatile unsigned int c17rtcaIntCount;

/* --- static variable --- */
int BatLowBatDetect = 0;
static int BatDetectCount = 0;
static int BatIsAnalysing = 0;
static unsigned int BatLowBattPrevTime = 0;


/************************************************************************//**
 * @brief  Initialize this interface.
 *
 * @retval  none
 ***************************************************************************/
void mid_initBat(void)
{
	c17initSvd();
	BatIsAnalysing     = 0;
	BatLowBattPrevTime = c17rtcaIntCount;
}

/************************************************************************//**
 * @brief  Check battery level.
 *
 * @retval  0 - Low Battery is not detected.
 * @retval  1 - Low Battery is detected.
 ***************************************************************************/
int mid_chkBatLowBattery(int isSending)
{
	if (isSending)
	{
		batStopSvd();
		return BatLowBatDetect;
	}

	switch(batAnalyzeLobattery())
	{
	case BAT_STATE_STOP:
		if (BatLowBattPrevTime != c17rtcaIntCount)
		{
			batStartSvd();
		}
		break;
	case BAT_STATE_ANALYZING:
		break;
	case BAT_STATE_COMPLETE:
	default:
		batStopSvd();
		BatLowBattPrevTime = c17rtcaIntCount;
		break;
	}
	return BatLowBatDetect;
}

static int batAnalyzeLobattery(void)
{
	if (BatIsAnalysing)
	{
		if (mid_getTmr(MAIN_SVD3_TIMER) >= 2)
		{
			if (BatLowBatDetect)
			{
				//low battery is detecting.
				if (c17getSvdVoltageDetection() == C17_SVD_SUPPLY_HIGH)
				{
					if (++BatDetectCount >= 2)
					{
						BatDetectCount = 1;
						c17forceResetWdt();
						while(1);
					}
				}
				else
				{
					//Reset the detection counter.
					BatDetectCount = 0;
				}
			}
			else
			{

				if (c17getSvdVoltageDetection() == C17_SVD_SUPPLY_LOW)
				{
					//Count up the detection counter.
					if (++BatDetectCount >= 2)
					{
						BatLowBatDetect = 1;
						BatDetectCount  = 0;
						c17setSvdComparisonVoltage(C17_SVDC_2500MV);
					}
				}
				else
				{
					//Reset the detection counter.
					BatDetectCount = 0;
				}
			}
			return BAT_STATE_COMPLETE;
		}
		else
		{
			return BAT_STATE_ANALYZING;
		}
	}
	return BAT_STATE_STOP;
}


static void batStartSvd(void)
{
	c17startSvd();
	mid_startTmr(MAIN_SVD3_TIMER);
	BatIsAnalysing = 1;
}

static void batStopSvd(void)
{
	mid_stopTmr(MAIN_SVD3_TIMER);
	c17stopSvd();
	BatIsAnalysing = 0;
}
