/************************************************************************//**
 *
 * @file    mid_host.c
 *
 * @brief   This is host interface program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "crt0.h"
#include "c17_init_config.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_uart3_1.h"
#include "mid_host.h"
#include "main_config.h"

#ifdef MAIN_USE_TERM_MODE
/* --- definition --- */
#define HOST_RECV_MAX		128
#define HOST_SEND_MAX		64

/* --- static function --- */
static void hostStartRecv(void);

/* --- static variable --- */
static char HostRecvBuff[2][HOST_RECV_MAX];
static char HostSendData[HOST_SEND_MAX];
static int  HostIsReceving = 0;
static int  HostIsSending = 0;
static char *HostRecvInBuff  = HostRecvBuff[0];
static char *HostRecvOutBuff = HostRecvBuff[1];

/************************************************************************//**
 * @brief  Initialize this interface.
 *
 * @retval  none
 ***************************************************************************/
void mid_initHost(void)
{
	/// Initialize UART3.
	c17initUart();
}

/************************************************************************//**
 * @brief  Turn on this interface.
 *
 * @return  none
 ***************************************************************************/
void mid_onHost(void)
{
	c17startUart();
	c17enableIntUart(C17_UART_RCV_1BYTE);
	hostStartRecv();
}

/************************************************************************//**
 * @brief  Turn off this interface.
 *
 * @return  none
 ***************************************************************************/
void mid_offHost(void)
{
	mid_waitHostSendComplete();
	HostIsReceving = 0;
	c17disableIntUart();
	c17startUart();
}

/************************************************************************//**
 * @brief  Receive string.
 *
 * @retval  NULL    There is no reception string.
 * @retval  other   Pointer to receive buffer.
 ***************************************************************************/
char* mid_recvHostString(void)
{
	if (HostIsReceving)
	{
		if (c17rcvUartDataWait() == EXIT_SUCCESS)
		{
			char *cTmp = HostRecvOutBuff;
			HostRecvOutBuff = HostRecvInBuff;
			HostRecvInBuff  = cTmp;
			HostIsReceving  = 0;
			hostStartRecv();
			return HostRecvOutBuff;
		}
	}
	return NULL;
}
/************************************************************************//**
 * @brief  Send string.
 *
 * @param   pstr   Pointer to string buffer.
 *
 * @retval  none
 ***************************************************************************/
void mid_sendHostString(const char *pstr)
{
	int remain = strlen(pstr);
	while (remain > 0)
	{
		int dlen = (remain > sizeof(HostSendData)) ? sizeof(HostSendData) : remain;
		mid_waitHostSendComplete();
		memcpy(HostSendData, pstr, sizeof(HostSendData));
		c17sendUartData(HostSendData, dlen);
		HostIsSending = 1;
		remain -= dlen;
		pstr += dlen;
	}
}
/************************************************************************//**
 * @brief  wait for completion of transmit.
 *
 * @param   pstr   Pointer to string buffer.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
void mid_waitHostSendComplete(void)
{
	if (HostIsSending)
	{
		while (c17sendUartDataWait() != EXIT_SUCCESS);
		HostIsSending = 0;
	}
}
/************************************************************************//**
 * @brief  Starts reception of UART.
 *
 * @retval  None.
 ***************************************************************************/
static void hostStartRecv(void)
{
	memset(HostRecvInBuff, '\0', HOST_RECV_MAX);
	c17rcvUartaData(HostRecvInBuff, HOST_RECV_MAX - 1, C17_UART_RCV_1BYTE);
	HostIsReceving = 1;
}
#endif

