/************************************************************************//**
 *
 * @file    mid_key.c
 *
 * @brief   This is key input interface program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/

#include <stdlib.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_init_config.h"
#include "main_config.h"
#include "mid_key.h"

#include "c17_port_1.h"
#include "c17_t16_1.h"
#include "main_config.h"

/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector09_handler, c17intT16Ch0);		// T16 Ch.0 interrupt
#endif /* CRT0_LINKED */
void c17intT16Ch0(void)	__attribute__ ((interrupt_handler));


/* --- definition --- */
enum
{
	KEY_INP_STATE_RELEASE,
	KEY_INP_STATE_PRESS,
	KEY_INP_STATE_LONG_PRESS
};

enum
{
	KEY_LEVEL_OFF,	/* input level for key matrix port at key off(low)*/
	KEY_LEVEL_ON	/* input level for key matrix port at key on (high) */
};

enum
{
	KEY_ACT_STATE_OFF,
	KEY_ACT_STATE_ON,
};

/* --- macro --- */
#define KEY_T16_CH0		0


/**
 * Macro changed according to user environment
 * Please change the argument or value if necessary.
 */
#define KEY_TIME_LONG_PRESS_DEF		1000	/* Default waiting time until the key long press event occurred. (ms)*/
#define KEY_TIME_REP_INTERVAL_DEF	200	/* Default Interval for key repeat event. (ms)*/
#define KEY_TIME_PRESS_LIMIT		10000
#define KEY_DEBOUNCE_COUNT			12		/* Debounce count*/
#define KEY_SCAN_STOP_TIMER			5000	/* Time ount to pause keyscan. */


/* --- Structure  ---*/
typedef struct port_s
{
	unsigned short grpNo;
	unsigned short portNo;
}KEY_PORT_SEL_S;

/* --- static function --- */
static int keyCheckInterrupt(void);
static inline void updateMatriOutPort(void);

/// --- Key matrix settings. ---
/**
 * Internal definition macro
 * Do not change.
 */
/// Matrix output line number.
#define KEY_MATLIX_ROW_1		(0)
#define KEY_MATLIX_ROW_2		(1)
#define KEY_MATLIX_ROW_3		(2)
/// Matrix input line number.
#define KEY_MATLIX_COL_1		(0)
#define KEY_MATLIX_COL_2		(1)
#define KEY_MATLIX_COL_3		(2)
#define KEY_MATLIX_COL_4		(3)

/* Configuration for key matrix.
 * Please change the argument or value if necessary.
 * */
#define KEY_MATRIX_ROW_PORTS	(3)	/* Number of output ports for key matrix */
#define KEY_MATRIX_COL_PORTS	(4)   /* Number of input ports for key matrix */
/// output ports for key matrix.
const KEY_PORT_SEL_S KeyRowPorts[KEY_MATRIX_ROW_PORTS] = {
		{C17_PORT_GRP_P2, 4},/* R1 = P24 */
		{C17_PORT_GRP_P2, 6},/* R2 = P26 */
		{C17_PORT_GRP_P2, 7},/* R3 = P27 */
};
/// input ports for key matrix.
const KEY_PORT_SEL_S KeyColPorts[KEY_MATRIX_COL_PORTS] = {
		{C17_PORT_GRP_P2, 1},/* C1 = P21 */
		{C17_PORT_GRP_P2, 2},/* C2 = P22 */
		{C17_PORT_GRP_P2, 3},/* C3 = P23 */
		{C17_PORT_GRP_P2, 5},/* C4 = P25 */
};

/* --- static variable --- */
static short KeyActivityState = KEY_ACT_STATE_OFF;
static short KeyInputState = KEY_INP_STATE_RELEASE;
static volatile unsigned short KeyScanRow = 0;
static volatile unsigned short KeyBounceCount = 0;
static volatile unsigned long KeyIntCodeOut = 0x00000000;
static volatile unsigned long KeyIntCodeWork = 0x00000000;
static volatile unsigned long KeyCurCode = KEY_NONE;
static unsigned long KeyWakeCode = 0;
static unsigned short KeyPressStart;
static unsigned short KeyRepStart;
static unsigned short KeyRepInterval = KEY_TIME_REP_INTERVAL_DEF;
static unsigned short KeyLongPressTime = KEY_TIME_LONG_PRESS_DEF;
static unsigned short KeyPresLimitOcurred;
static short KeyCancelCurEvent = 0;
static volatile unsigned short KeyInputTimer = 0;
static volatile unsigned short KeyScanStopTimer = 0;
static volatile unsigned short KeyWakeupTimer;



/************************************************************************//**
 * @brief  Initialize this interface.
 *
 * @retval  none
 ***************************************************************************/
void mid_initKey(void)
{
	int i;
	const KEY_PORT_SEL_S *pmrow = KeyRowPorts;
	const KEY_PORT_SEL_S *pmcol = KeyColPorts;

    //set up timer.
	c17setCountT16(KEY_T16_CH0, T16CH0_COUNT);
    c17enableIntT16(KEY_T16_CH0);

	//setup the output ports.
	for (i = 0; i < KEY_MATRIX_ROW_PORTS; i++, pmrow++)
	{
		c17setDirectionPortGrp(pmrow->grpNo, pmrow->portNo, C17_PORT_DIR_INPUT, 0, C17_PORT_PULL_DOWN);
		c17setOutputPortGrp(pmrow->grpNo, pmrow->portNo, KEY_LEVEL_ON);
	}
	//setup the input port.
	for (i = 0; i < KEY_MATRIX_COL_PORTS; i++, pmcol++)
	{
		c17setDirectionPortGrp(pmcol->grpNo, pmcol->portNo, C17_PORT_DIR_INPUT, 0, C17_PORT_PULL_DOWN);
	}
	
}
/************************************************************************//**
 * @brief  Start key scan.
 *
 * @return  none
 ***************************************************************************/
void mid_startKeyScan(void)
{
	/// Turn off the module.
    mid_stopKeyScan();

	/// reset key state.
	KeyActivityState = KEY_ACT_STATE_ON;
	KeyScanRow       = 0;
	updateMatriOutPort();

	/// resume timer for key scan.
	c17startT16(KEY_T16_CH0);
	c17enableIntT16(KEY_T16_CH0);
}
/************************************************************************//**
 * @brief  Stop key scan.
 *
 * @return  none
 ***************************************************************************/
void mid_stopKeyScan(void)
{
	int i;
	const KEY_PORT_SEL_S *pmrow = KeyRowPorts;
	const KEY_PORT_SEL_S *pmcol = KeyColPorts;

	/// stop the timer.
	c17stopT16(KEY_T16_CH0);

	/// clear key state.
	KeyActivityState = KEY_ACT_STATE_OFF;
	KeyInputState    = KEY_INP_STATE_RELEASE;
	KeyIntCodeOut    = 0x00000000UL;
	KeyIntCodeWork   = 0x00000000UL;
    KeyCurCode       = KEY_NONE;
    KeyScanStopTimer = 0;
    
	/// Turn off all output ports.
	for (i = 0; i < KEY_MATRIX_ROW_PORTS; i++, pmrow++)
	{
		c17setDirectionPortGrp(pmrow->grpNo, pmrow->portNo, C17_PORT_DIR_INPUT, 0, C17_PORT_PULL_DOWN);
	}

	/// turn of all input ports.
	for (i = 0; i < KEY_MATRIX_COL_PORTS; i++, pmcol++)
	{
		c17disableIntPortGrp(pmcol->grpNo, pmcol->portNo);
		c17clrIntFlagGrp(pmcol->grpNo, pmcol->portNo);
	}
	
}
/************************************************************************//**
 * @brief  Set wake-up key.
 *
 * @param  keyCode   key code .
 *
 * @return  none
 ***************************************************************************/
void mid_setKeyWakeup(unsigned long keyCode)
{
	int row, col;
	const KEY_PORT_SEL_S *pmrow;
	const KEY_PORT_SEL_S *pmcol;

	//search input and output port in key matrix circuit.
	row = 0;
	col = 0;
	for (row = 0; row < KEY_MATRIX_ROW_PORTS; row++)
	{
		unsigned long code;
		for (col = 0; col < KEY_MATRIX_COL_PORTS; col++)
		{
			code = (unsigned long)1UL << ((row * KEY_MATRIX_COL_PORTS) + col);
			if (code == keyCode)
			{
				break;
			}
		}
		if (code == keyCode)
		{
			break;
		}
	}
	KeyWakeCode = keyCode;
	pmrow = &KeyRowPorts[row];
	pmcol = &KeyColPorts[col];

	/// turn off the module.
	mid_stopKeyScan();

	/// Turn on output port of the wake-up key.
	c17setOutputPortGrp(pmrow->grpNo, pmrow->portNo, KEY_LEVEL_ON);
	c17setDirectionPortGrp(pmrow->grpNo, pmrow->portNo, C17_PORT_DIR_OUTPUT, 0, C17_PORT_PULL_DISABLE);

	/// Enable interrupt of input port for the wake-up key.
	c17enableIntPortGrp(pmcol->grpNo, pmcol->portNo, C17_PORT_INT_EDGE_RISING);
}
/************************************************************************//**
 * @brief  Check if wake-up key is pushed.
 *
 * @retval  0 wake up key is not pushed.
 * @retval  1 wake up key is bing pushed.
 ***************************************************************************/
int mid_chkKeyWakeup(void)
{
	KEY_STATE_S keystate;

	KeyWakeupTimer = 0;
	mid_getKeyState(&keystate);
	if (KeyActivityState == KEY_ACT_STATE_ON)
	{
		while (KeyWakeupTimer < 100) {
			mid_getKeyState(&keystate);
			if ((keystate.keycode == KeyWakeCode) && (keystate.keyevent == KEY_EVENT_PRESS))
			{
				return 1;
			}
		}
		mid_setKeyWakeup(KeyWakeCode);
	}
	return 0;
}
/************************************************************************//**
 * @brief  Pause the key scan.
 *
 * @return  none
 ***************************************************************************/
void mid_pauseKeyScan(void)
{
	int i;
	const KEY_PORT_SEL_S *pmrow = KeyRowPorts;
	const KEY_PORT_SEL_S *pmcol = KeyColPorts;

	/// clear key state.
	mid_stopKeyScan();

	/// Turn on all output ports.
	for (i = 0; i < KEY_MATRIX_ROW_PORTS; i++, pmrow++)
	{
		c17setOutputPortGrp(pmrow->grpNo, pmrow->portNo, KEY_LEVEL_ON);
		c17setDirectionPortGrp(pmrow->grpNo, pmrow->portNo, C17_PORT_DIR_OUTPUT, 0, C17_PORT_PULL_DISABLE);
	}

	/// Enable interrupt of all input ports.
	for (i = 0; i < KEY_MATRIX_COL_PORTS; i++, pmcol++)
	{
		c17enableIntPortGrp(pmcol->grpNo, pmcol->portNo, C17_PORT_INT_EDGE_RISING);
	}
}


/************************************************************************//**
 * @brief  Get key input state.
 *
 * @param pkeyState   Pointer to key state structure to store the key state.
 *
 * @return  none.
 ***************************************************************************/
void mid_getKeyState(KEY_STATE_S *pkeyState)
{
	pkeyState->keyevent = KEY_EVENT_NONE;
	pkeyState->keycode  = KeyCurCode;

	if (KeyActivityState == KEY_ACT_STATE_OFF)
	{
		if (keyCheckInterrupt())
		{
			mid_startKeyScan();
		}
		pkeyState->keycode = KEY_NONE;
		return;
	}
	
	if (KeyInputState == KEY_INP_STATE_RELEASE)
	{
		/// Key Released state
		if (KeyIntCodeOut == 0)
		{
			if (KeyScanStopTimer >= KEY_SCAN_STOP_TIMER)
			{
				mid_pauseKeyScan();
			}
		}
		else
		{
			KeyScanStopTimer = 0;
			if (KeyCurCode != KeyIntCodeOut)
			{
				/// Transition to Press state.
				pkeyState->keyevent = KEY_EVENT_PRESS;
				pkeyState->keycode  = KeyIntCodeOut;
				KeyInputState       = KEY_INP_STATE_PRESS;
				KeyCurCode          = KeyIntCodeOut;
				KeyInputTimer       = 0;
				KeyPressStart       = KeyInputTimer;
				KeyPresLimitOcurred = 0;
				KeyCancelCurEvent   = 0;
			}
		}
	}
	else
	{
		/// Key press or long press state.
		if (KeyCurCode != KeyIntCodeOut)
		{
			/// Transition to Release state.
			KeyInputState       = KEY_INP_STATE_RELEASE;
			pkeyState->keyevent = KEY_EVENT_RELEASE;
			pkeyState->keycode  = KeyCurCode;
			KeyCurCode          = KEY_NONE;
		}
		else if (KeyInputState == KEY_INP_STATE_PRESS)
		{
			/// Key press state.
			if ((KeyInputTimer - KeyPressStart) >= KeyLongPressTime)
			{
				/// Transition to Long press state.
				KeyInputState       = KEY_INP_STATE_LONG_PRESS;
				pkeyState->keyevent = KEY_EVENT_LONG_PRESS;
				pkeyState->keycode  = KeyCurCode;
				KeyRepStart         = KeyInputTimer;
			}
		}
		else
		{
			/// Key long press state.
			if ((KeyInputTimer - KeyRepStart) >= KeyRepInterval)
			{
				/// Notify key repeat.
				pkeyState->keyevent = KEY_EVENT_REPEAT;
				pkeyState->keycode  = KeyCurCode;
				KeyRepStart         = KeyInputTimer;
			}
			else if ((!KeyPresLimitOcurred) &&
							((KeyInputTimer - KeyPressStart) >= KEY_TIME_PRESS_LIMIT))
			{
				/// Notify key press limit.
				pkeyState->keyevent = KEY_EVENT_PRESS_LIMIT;
				pkeyState->keycode  = KeyCurCode;
				KeyPresLimitOcurred = 1;
			}
		}

		if (KeyCancelCurEvent)
		{
			pkeyState->keyevent = KEY_EVENT_NONE;
			pkeyState->keycode  = 0;
		}
		KeyScanStopTimer = 0;
	}
}

/************************************************************************//**
 * @brief  Set waiting time in order to occur the long press event.
 *
 * @param   WaitTime  Waiting time(ms unit)
 *
 * @return  none.
 ***************************************************************************/
void mid_setKeyLongPressTime(unsigned short waitTime)
{
	KeyLongPressTime = waitTime;
}
/************************************************************************//**
 * @brief  Set interval for key repeat event.
 *
 * @param   interval  Interval(ms unit)
 *
 * @return  none.
 ***************************************************************************/
void mid_setKeyRepeateTime(unsigned short interval)
{
	KeyRepInterval = interval;
}
/************************************************************************//**
 * @brief  Cancel subsequent key event until current key is released.
 *
 * @return  none.
 ***************************************************************************/
void mid_cancelKeyCurKeyEvent(void)
{
	KeyCancelCurEvent = 1;
}

/************************************************************************//**
 * @brief  Check if input port interrupt occurred.
 *
 * @return  0  Input port interrupt did not occur.
 * @return  1  Input port interrupt occurred.
 ***************************************************************************/
static int keyCheckInterrupt(void)
{
	int col;
	int state = 0;
	const KEY_PORT_SEL_S *pmcol = KeyColPorts;

	for (col = 0; col < KEY_MATRIX_COL_PORTS; col++, pmcol++)
	{
		state |= c17checkIntFlagGrp(pmcol->grpNo, pmcol->portNo);
		c17clrIntFlagGrp(pmcol->grpNo, pmcol->portNo);
	}
	return (state ? 1 : 0);
}


/************************************************************************//**
 * @brief  Execute the key scan.
 *
 * @return  none
 ***************************************************************************/
inline void KeyScan(void)
{
	int i;
	const KEY_PORT_SEL_S *pmcol = KeyColPorts;

	/// Scan the input ports.
	for (i = 0; i < KEY_MATRIX_COL_PORTS; i++, pmcol++)
	{
		unsigned long mask = (unsigned long)1UL << ((KeyScanRow * KEY_MATRIX_COL_PORTS) + i);
		if (c17getInputPortGrp(pmcol->grpNo, pmcol->portNo) == KEY_LEVEL_ON)
		{
			KeyIntCodeWork |= mask;
		}
		else
		{
			KeyIntCodeWork &= ~mask;
		}
	}

	/// Change the scanning line.
	if (++KeyScanRow >= KEY_MATRIX_ROW_PORTS)
	{
		KeyScanRow = 0;

		if (KeyIntCodeWork == 0)
		{
			KeyScanStopTimer += KEY_MATRIX_ROW_PORTS;
		}

		/// Debounce processing
		if (KeyIntCodeOut != KeyIntCodeWork)
		{
			if (++KeyBounceCount >= KEY_DEBOUNCE_COUNT)
			{
				KeyIntCodeOut  = KeyIntCodeWork;
				KeyBounceCount = 0;
			}
		}
		else
		{
			KeyBounceCount = 0;
		}
		KeyIntCodeWork = 0x00000000;

	}
	if (KeyInputTimer < 0xFFFF)
	{
		KeyInputTimer++;
	}
	KeyWakeupTimer++;
	updateMatriOutPort();
}


/************************************************************************//**
 * @brief  Update I/O output ports for key matrix.
 *
 * @return  none
 ***************************************************************************/
static inline void updateMatriOutPort(void)
{
	int i;
	const KEY_PORT_SEL_S *pmrow = KeyRowPorts;
	for (i = 0; i < KEY_MATRIX_ROW_PORTS; i++, pmrow++)
	{
		if (i != KeyScanRow)
		{
			c17setDirectionPortGrp(pmrow->grpNo, pmrow->portNo, C17_PORT_DIR_INPUT, 0, C17_PORT_PULL_DOWN);
		}
		else
		{
			c17setOutputPortGrp(pmrow->grpNo, pmrow->portNo, KEY_LEVEL_ON);
			c17setDirectionPortGrp(pmrow->grpNo, pmrow->portNo, C17_PORT_DIR_OUTPUT, 0, C17_PORT_PULL_DISABLE);
		}
	}
}

/************************************************************************//**
 * @brief   T16(Ch.0) interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intT16Ch0(void)
{
	T16_NINTF_REG	t16_0intf_set;

	/* T16_0INTF  T16 Ch.0  Interrupt Flag Register read & clear */
	{
		t16_0intf_set.reg = T16_0INTF;
		T16_0INTF = t16_0intf_set.reg;
	}

	/// Check interrupt cause occurrence status.
	if(t16_0intf_set.bit.UFIF == 1) {
		/// Increment the tick counter.
		if (KeyActivityState == KEY_ACT_STATE_ON)
		{
			KeyScan();
		}
	}
}
