/************************************************************************//**
 *
 * @file    mid_remrcv.c
 *
 * @brief   This is IR Remote code reception interface program.
 *
 * @par     Copyright:
 *          Copyright (C) SEIKO EPSON CORPORATION 2018. All rights reserved.
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "crt0.h"
#include "c17_misc.h"
#include "c17_clg.h"
#include "c17_t16b.h"
#include "c17_init_config.h"
#include <c17_mcu_select.h>
#include "c17_port_1.h"
#include "c17_remc3_1.h"
#include "main_config.h"

#ifdef MAIN_USE_TERM_MODE
/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector14_handler, c17intT16bCh0);		// T16B Ch.0 interrupt
#endif /* CRT0_LINKED */
void c17intT16bCh0(void)	__attribute__ ((interrupt_handler));

/* --- macro --- */
#define REMRCV_INPLEV_ON	0
#define REMRCV_INPLEV_OFF	1
#define REMRCV_T16B_CH0		0

/* --- definition --- */
enum
{
	STATUS_NONE,
	STATUS_LEADER_1,
	STATUS_LEADER_2,
	STATUS_DATA,
	STATUS_END
};

/* --- structure --- */
typedef struct st_remc_recv_s {
	unsigned short	Status;
	unsigned short	remcFormat;
	unsigned short	bitCount;
	unsigned short	workData;
	unsigned short	dataCount;
	unsigned short  blank;
	unsigned char	data[IRCODE_LENGTH_MAX];
} ST_REMC_RECV_S;

typedef struct st_remc_wave {
	unsigned short remcFormat;
	unsigned short leader_on_lo;		/// Leader Lower limit.
	unsigned short leader_on_hi;		/// Leader Upper limit.
	unsigned short leader_off_lo;		/// Leader Lower limit.
	unsigned short leader_off_hi;		/// Leader Upper limit.
	unsigned short bit1_lo;				/// Data = 1 Lower limit.
	unsigned short bit1_hi;				/// Data = 1 Upper limit.
	unsigned short bit0_lo;				/// Data = 0 Lower limit.
	unsigned short bit0_hi;				/// Data = 0 Upper limit.
} ST_REMC_RECV_WAVE_S;

/* --- global variable --- */
static ST_REMC_RECV_WAVE_S  RemRcvWave = {

	/* AEHA  pulse width */
	C17_REMC_FORMAT_AEHA,
	100, 123,	/* READER ON  8T(T=0.000425) */
	 50,  61,	/* READER ON  4T(T=0.000425) */
	 34,  49,	/* DATA1      3T(T=0.000425) */
	  9,  19,	/* DATA0      1T(T=0.000425) */
};

/* --- global variable --- */
static volatile ST_REMC_RECV_S remc_recv;
static volatile unsigned short startCapture;
static volatile unsigned short endCapture;
static volatile unsigned short RemRcvOnLevel = 0;

/* --- function --- */
static void setCAP(void);
static unsigned short remRcvCalcPulseCount(unsigned short t, unsigned short width);

/************************************************************************//**
 * @brief  Initialize this interface.
 *
 * @return  none
 ***************************************************************************/
void mid_initRemRcv(void)
{
	/// T16B initial
	/* clock source = OSC1
	   division = 1/1
	   count =UP
	   repeat mode
	 */
	c17setT16b(REMRCV_T16B_CH0,C17_T16B_CLKSRC_OSC1,C17_T16B_CLKDIV_1,C17_T16B_UP,C17_T16B_REPEAT);
	c17setT16bCh0MaxCount(0xFFFF);

	/// P00 = CAP0
	setCAP();
}

/************************************************************************//**
 * @brief  Turn on this interface.
 *
 * @return  none
 ***************************************************************************/
void mid_onRemRcv(void)
{
	remc_recv.Status = STATUS_NONE;
	c17startT16b(REMRCV_T16B_CH0);
}
/************************************************************************//**
 * @brief  Turn off this interface.
 *
 * @return  none
 ***************************************************************************/
void mid_offRemRcv(void)
{
	c17stopT16b(REMRCV_T16B_CH0);
}

/************************************************************************//**
 * @brief  Set the data format.
 *
 * @param   param     Pointer to wave parameter.
 *
 * @return  none
 ***************************************************************************/
void mid_setRemRecvFormat(ST_REMC_PARAM *param)
{
	unsigned short wtmp = param->leader_h * param->t;
	wtmp = remRcvCalcPulseCount(param->t, param->leader_h);
	RemRcvWave.leader_on_hi = (unsigned short)((wtmp * 13) / 10);
	RemRcvWave.leader_on_lo = (unsigned short)((wtmp * 7) / 10);
	wtmp = remRcvCalcPulseCount(param->t, param->leader_l);
	RemRcvWave.leader_off_hi = (unsigned short)((wtmp * 13) / 10);
	RemRcvWave.leader_off_lo = (unsigned short)((wtmp * 7) / 10);
	wtmp = remRcvCalcPulseCount(param->t, param->bit0_l);
	RemRcvWave.bit0_hi = (unsigned short)((wtmp * 13) / 10);
	RemRcvWave.bit0_lo = (unsigned short)((wtmp * 7) / 10);
	wtmp = remRcvCalcPulseCount(param->t, param->bit1_l);
	RemRcvWave.bit1_hi = (unsigned short)((wtmp * 13) / 10);
	RemRcvWave.bit1_lo = (unsigned short)((wtmp * 7) / 10);
	RemRcvWave.remcFormat = param->remcFormat;
}

/************************************************************************//**
 * @brief  Get received IR remote control code.
 *
 * @param   pData         Pointer to buffer to copy received data.
 * @param   MaxBitLength  Acceptable maximum bit length.
 *
 * @retval  0     There is no reception data.
 * @retval  other Bit length of reception data.
 ***************************************************************************/
int mid_recvRemRcvCode(unsigned char *pData, int MaxBitLength)
{
	int rlen = 0;
	int maxByteLength = MaxBitLength / 8;

	if(remc_recv.Status == STATUS_DATA)
	{
		unsigned short pulseWidth;
		
		endCapture = c17getT16bCh0TimerCount();
		pulseWidth = (endCapture >= startCapture) ?
							(endCapture - startCapture) : ((0xFFFF - startCapture) + endCapture);
		if (pulseWidth >= remc_recv.blank)
		{
			remc_recv.Status = STATUS_END;
		}
	}

	if(remc_recv.Status == STATUS_END)
	{
		int datlen = (remc_recv.dataCount > maxByteLength) ? maxByteLength : remc_recv.dataCount;
		for (rlen = 0; rlen < datlen; rlen++, pData++)
		{
			*pData = remc_recv.data[rlen];
		}
		if ((rlen > 0) && (rlen < maxByteLength) && (remc_recv.bitCount > 0))
		{
			*pData = remc_recv.workData;
			rlen = (rlen * 8) + remc_recv.bitCount;
		} else {
			rlen *= 8;
		}
		remc_recv.Status = STATUS_NONE;
	}
	return rlen;
}

/************************************************************************//**
 * @brief   Set CAP.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
static void setCAP(void)
{
	/**
	 * @todo P05=CAP0 Change port
	 */
	P0UPMUX2_REG	p0upmux_set;
	P0MODSEL_REG	p0modsel_set;
	P0FNCSEL_REG	p0fncsel_set;

	/* P0MODSEL:P0 Port Mode Select Register */
	{
		p0modsel_set.reg = P0MODSEL;
		p0modsel_set.bit.P05SEL = 1;		/// - P05 : Enable peripheral I/O function.
		P0MODSEL = p0modsel_set.reg;
	}

	/* P0FNCSEL:P0 Port Function Select Register. */
	{
		p0fncsel_set.reg = P0FNCSEL;
		p0fncsel_set.bit.P05MUX = 1;		/// - P05 : UPMUX.
		P0FNCSEL = p0fncsel_set.reg;
	}

	/*	P04-05 Universal Port Multiplexer Setting Register */
	{
		p0upmux_set.reg = P0UPMUX2;

		p0upmux_set.bit.P05PERICH  = 0;		/// - P05 : CH.0
		p0upmux_set.bit.P05PERISEL = 4;		/// - P05 : T16B.
		p0upmux_set.bit.P05PPFNC   = 1;		/// - P05 : TOUT0/CAP0
		P0UPMUX2 = p0upmux_set.reg;
	}
}
/************************************************************************//**
 * @brief   T16B(Ch.0) interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intT16bCh0(void)
{
	T16BNINTF_REG	t16b0intf_set;

	/* Capture interrupt flag */
	t16b0intf_set.reg = T16B0INTF;

	/* Clear T16B Ch.0 Interrupt flag. */
	T16B0INTF = t16b0intf_set.reg;

	/// Check compare/capture 0 interrupt flag.
	if(t16b0intf_set.bit.CMPCAP0IF == 1)
	{
		unsigned short pulseWidth;
		unsigned short workCapture;

		workCapture = T16B0CCR0;
		switch(remc_recv.Status)
		{
		case STATUS_NONE:
			if(T16B0CS_CAPI0 == REMRCV_INPLEV_ON)
			{
				startCapture = workCapture;
				remc_recv.Status = STATUS_LEADER_1;
			}
			break;
		case STATUS_LEADER_1:
			if (T16B0CS_CAPI0 != REMRCV_INPLEV_ON)
			{
				pulseWidth = (workCapture >= startCapture) ?
											(workCapture - startCapture) : ((0xFFFF - startCapture) + workCapture);
				if((pulseWidth >= (RemRcvWave.leader_on_lo)) &&
						(pulseWidth <= (RemRcvWave.leader_on_hi)))
				{
					remc_recv.remcFormat = RemRcvWave.remcFormat;
					remc_recv.bitCount   = 0;
					remc_recv.dataCount  = 0;
					remc_recv.workData   = 0;
					remc_recv.blank      = RemRcvWave.leader_on_hi;
					if (remc_recv.remcFormat == C17_REMC_FORMAT_SONY )
					{
						RemRcvOnLevel = REMRCV_INPLEV_OFF;
						remc_recv.Status = STATUS_DATA;
					}
					else
					{
						RemRcvOnLevel = REMRCV_INPLEV_ON;
						remc_recv.Status = STATUS_LEADER_2;
					}
					startCapture = workCapture;
				}
				else
				{
					remc_recv.Status = STATUS_NONE;
				}
			}
			else
			{
				startCapture = workCapture;
			}
			break;
		case STATUS_LEADER_2:
			if (T16B0CS_CAPI0 == RemRcvOnLevel)
			{
				remc_recv.Status = STATUS_DATA;
				pulseWidth = (workCapture >= startCapture) ?
									(workCapture - startCapture) : ((0xFFFF - startCapture) + workCapture);
				startCapture = workCapture;
			}
			else
			{
				remc_recv.Status = STATUS_NONE;
			}
			break;
		case STATUS_DATA:
			if (T16B0CS_CAPI0 == RemRcvOnLevel)
			{
				pulseWidth = (workCapture >= startCapture) ?
								(workCapture - startCapture) : ((0xFFFF - startCapture) + workCapture);
				if((pulseWidth >= (RemRcvWave.bit0_lo)) &&
						(pulseWidth <= (RemRcvWave.bit0_hi)))
				{
					//Bit = 0
					remc_recv.bitCount++;
#if REMC_BITORDER == REMC_BITORDER_MSBFIRST
					remc_recv.workData <<= 1;
#else
					remc_recv.workData >>= 1;
#endif
				}
				else if((pulseWidth >= (RemRcvWave.bit1_lo)) &&
						(pulseWidth <= (RemRcvWave.bit1_hi)))
				{
					//Bit = 1
					remc_recv.bitCount++;
#if REMC_BITORDER == REMC_BITORDER_MSBFIRST
					remc_recv.workData <<= 1;
					remc_recv.workData |= 0x01;
#else
					remc_recv.workData >>= 1;
					remc_recv.workData |= 0x80;
#endif
				}
				else
				{
					remc_recv.Status = STATUS_END;
				}
				//Byte data save
				if(remc_recv.bitCount == 8)
				{
					remc_recv.data[remc_recv.dataCount] = remc_recv.workData;
					remc_recv.bitCount = 0;
					remc_recv.workData = 0;
					remc_recv.dataCount++;

					if(remc_recv.dataCount >= IRCODE_LENGTH_MAX)
					{
						remc_recv.Status = STATUS_END;
					}
				}
				startCapture = workCapture;
			}
			else
			{
				startCapture = workCapture;
			}
			break;
		case STATUS_END:
			break;
		}
	}

	if(t16b0intf_set.bit.CAPOW0IF == 1)
	{
		/// over write error
		asm("nop");
	}
}

static unsigned short remRcvCalcPulseCount(unsigned short t, unsigned short width)
{
	unsigned long wval = t * width;
	wval *= 32768;
	wval /= 1000000;
	return wval;
}
#endif


