/************************************************************************//**
 *
 * @file    mid_remsnd.c
 *
 * @brief   This is IR remote control code transmission interface program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "crt0.h"
#include "c17_misc.h"
#include "main_config.h"
#include "c17_init_config.h"
#include "mid_remsnd.h"

#include "c17_remc3_1.h"
#include "c17_t16_1.h"


/* --- definition --- */
enum
{
	REMSND_SEND_PHASE_PRECEDE,
	REMSND_SEND_PHASE_PRECEDE_TR,
	REMSND_SEND_PHASE_DATA,
	REMSND_SEND_PHASE_DATA_TR,
};


enum
{
	REMSEND_BUFF_EMPTY,
	REMSEND_BUFF_FULL,
};

/* --- macro --- */
#define REMSND_T16_CH1	1

/* --- structure --- */
typedef struct remac_sendbuff_s
{
	unsigned char		code[IRCODE_LENGTH_MAX];
	unsigned short		codelen;
	unsigned short		state;
	unsigned short		phase;
	int					repcnt;
}REMSND_SENDBUFF_S;

/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector11_handler, c17intT16Ch1);		// T16 Ch.1 interrupt
#endif /* CRT0_LINKED */


/* Configuration for format for IR remote control code.
 * Please change the argument or value if necessary.
 * */
static const ST_REMC_PARAM	RemSndIrFormat = {
		C17_REMC_FORMAT_AEHA, 36700, 436, 8, 4, 1, 1, 1, 3
};

/* --- function --- */
static void RemSndSendStart(void);
static void RemSndStartTimer(void);
static void RemSndStopTimer(void);

/* --- static variable --- */
static REMSND_SENDBUFF_S		RemSndSendBuff[REMSND_SENDBUF_COUNT];
static unsigned char			RemSndPrecedeCode[REMSND_PRECEDE_MAX];
static int						RemSndPrecedeSize = 0;
static volatile unsigned short	RemSndDownCounter = 0;

/* --- global variable --- */
REMSND_SENDBUFF_S		*RemSndActBuff  = &RemSndSendBuff[0];
REMSND_SENDBUFF_S		*RemSndNextBuff = &RemSndSendBuff[1];

/************************************************************************//**
 * @brief  Initialize this interface.
 *
 * @retval  none
 ***************************************************************************/
void mid_initRemSnd(void)
{
	int i;

	/// Initialize REMC.
	c17initRemc();

	/// Initialize format
	c17setFormatRemcDirect(&RemSndIrFormat);

	/// Initialize timer.
	c17setCountT16(REMSND_T16_CH1, T16CH1_COUNT);

	/// Initialize sending buffer.
	for (i = 0; i < REMSND_SENDBUF_COUNT; i++)
	{
		RemSndSendBuff[i].state = REMSEND_BUFF_EMPTY;
		RemSndSendBuff[i].phase = 0;
	}
}
/************************************************************************//**
 * @brief  Turn on this interface.
 *
 * @retval  none
 ***************************************************************************/
void mid_onRemSnd(void)
{
	c17enableRemc();
	c17enableIntRemc();
}
/************************************************************************//**
 * @brief  Turn off this interface.
 *
 * @retval  none
 ***************************************************************************/
void mid_offRemSnd(void)
{
	/// wait for completion of transmission.
	while (mid_chkRemSndIrSendState());
	/// stop the module.
	RemSndStopTimer();
	c17disableRemc();
}

/************************************************************************//**
 * @brief  Request transmission IR remote control code.
 *
 * @param  repcnt    Repeat count
 *                   (if set to 1(0), IR remote control code is sent only one time.)
 * @param  pcode     Pointer to buffer that is stored IR remote control code.
 * @param  length    Length of IR remote control code.
 *
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int mid_reqRemSndIrSendCode(int repcnt, const unsigned char *pcode, unsigned short length)
{
	/// Check length of code.
	if ((length <= 0) || (length > IRCODE_LENGTH_MAX))
	{
		return EXIT_FAILURE;
	}

	/// Copy data to next buffer.
	memcpy(RemSndNextBuff->code, pcode, length);
	RemSndNextBuff->codelen = length;
	RemSndNextBuff->repcnt  = (repcnt == 0) ? 1 : repcnt;
	RemSndNextBuff->state   = REMSEND_BUFF_FULL;

	/// Check if data in next buffer can be send.
	if (RemSndActBuff->state == REMSEND_BUFF_EMPTY)
	{
		RemSndSendStart();
	}
	return EXIT_SUCCESS;
}



/************************************************************************//**
 * @brief  Check transmission state of the IR remote control code.
 *
 *
 * @retval  0   stopped.
 * @retval  1   sending
 ***************************************************************************/
int mid_chkRemSndIrSendState(void)
{
	if (RemSndActBuff->state == REMSEND_BUFF_FULL)
	{
		/// Check state of active buffer.
		switch(RemSndActBuff->phase)
		{
		case REMSND_SEND_PHASE_PRECEDE:
			if (c17sendRemcDataWait() == EXIT_SUCCESS)
			{
				RemSndDownCounter = REMSND_PRECEDE_TR_BLANK;
				RemSndActBuff->phase++;
			}
			break;
		case REMSND_SEND_PHASE_PRECEDE_TR:
			if (RemSndDownCounter == 0)
			{
				c17sendRemc((unsigned char*)&RemSndActBuff->code, RemSndActBuff->codelen);
				RemSndActBuff->phase++;
			}
			break;
		case REMSND_SEND_PHASE_DATA:
			if (c17sendRemcDataWait() == EXIT_SUCCESS)
			{
				RemSndDownCounter = REMSND_CODE_TR_BLANK;
				RemSndActBuff->phase++;
			}
			break;
		case REMSND_SEND_PHASE_DATA_TR:
			if (RemSndDownCounter == 0)
			{
				if (RemSndActBuff->repcnt <= 1)
				{
					RemSndStopTimer();
					RemSndActBuff->state = REMSEND_BUFF_EMPTY;
				}
				else
				{
					if (RemSndPrecedeSize > 0)
					{
						RemSndActBuff->phase = REMSND_SEND_PHASE_PRECEDE;
						c17sendRemc(RemSndPrecedeCode, RemSndPrecedeSize);
					}
					else
					{
						RemSndActBuff->phase = REMSND_SEND_PHASE_DATA;
						c17sendRemc((unsigned char*)&RemSndActBuff->code, RemSndActBuff->codelen);
					}
					RemSndActBuff->repcnt--;
				}
			}
			break;
		}
	}
	/// Check if need to switch the sending buffer.
	if ((RemSndActBuff->state == REMSEND_BUFF_EMPTY) && (RemSndNextBuff->state == REMSEND_BUFF_FULL))
	{
		RemSndSendStart();
	}
	/// return current state.
	if ((RemSndActBuff->state == REMSEND_BUFF_FULL) ||
		(RemSndNextBuff->state == REMSEND_BUFF_FULL))
	{
		return 1;
	}
	return 0;
}
/************************************************************************//**
 * @brief  set the preceded code.
 *
 * @param  pcode
 *  @li @c 0        Preceded code is unnecessary.
 *  @li @c other    Pointer to buffer that is stored data.
 * @param  length
 *  @l1 @c 0          Preceded code is unnecessary.
 *  oli @c 1 or more  Length of preceded code.
 * @retval  EXIT_SUCCESS    Success.
 * @retval  EXIT_FAILURE    Failed.
 ***************************************************************************/
int mid_setRemSndPreciedCode(const unsigned char *pcode, int length)
{
	if ((pcode == 0) || (length == 0))
	{
		RemSndPrecedeSize = 0;
	}
	else
	{
		if (length <= REMSND_PRECEDE_MAX)
		{
			RemSndPrecedeSize = length;
			memcpy(RemSndPrecedeCode, pcode, length);
		}
		else
		{
			return EXIT_FAILURE;
		}
	}
	return EXIT_SUCCESS;
}
/************************************************************************//**
 * @brief  Calculate the BCC(XOR each byte).
 *
 * @param   pdata     Pointer to buffer that is stored data.
 * @param   length    Length of data.
 * @retval  BCC
 ***************************************************************************/
unsigned char mid_calcRemSndBcc(unsigned char *pdata, int length)
{
	int i;
	unsigned char bcc = 0;
	for (i = 0; i < length; i++, pdata++)
	{
		bcc ^= *pdata;
	}
	return bcc;
}
/************************************************************************//**
 * @brief  Calculate the checksum.
 *
 * @param   pdata     pointer to buffer that is stored data.
 * @param   length    length of data.
 * @retval  Checksum
 ***************************************************************************/
unsigned char mid_calcRemSndSum(unsigned char *pdata, int length)
{
	int i;
	unsigned char sum = 0;
	for (i = 0; i < length; i++, pdata++)
	{
		sum += *pdata;
	}
	return sum;
}
static void RemSndSendStart(void)
{
	REMSND_SENDBUFF_S *pTmp;

	/// Swap the activate buffer and the next buffer.
	pTmp           = RemSndActBuff;
	RemSndActBuff  = RemSndNextBuff;
	RemSndNextBuff = pTmp;

	/// start sending code.
	RemSndActBuff->state  = REMSEND_BUFF_FULL;
	if (RemSndPrecedeSize > 0)
	{
		RemSndActBuff->phase = REMSND_SEND_PHASE_PRECEDE;
		c17sendRemc(RemSndPrecedeCode, RemSndPrecedeSize);
	}
	else
	{
		RemSndActBuff->phase = REMSND_SEND_PHASE_DATA;
		c17sendRemc((unsigned char*)&RemSndActBuff->code, RemSndActBuff->codelen);
	}
	RemSndStartTimer();
}

static void RemSndStartTimer(void)
{
	RemSndDownCounter = 0;
	c17startT16(REMSND_T16_CH1);
	c17enableIntT16(REMSND_T16_CH1);
}
static void RemSndStopTimer(void)
{
	c17disableIntT16(REMSND_T16_CH1);
	c17stopT16(REMSND_T16_CH1);
}



/************************************************************************//**
 * @brief   T16 CH.1 Interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intT16Ch1(void)
{
	T16_NINTF_REG	t16_1intf_set;

	/* T16_1INTF  T16 Ch.1  Interrupt Flag Register read & clear */
	{
		t16_1intf_set.reg = T16_1INTF;
		T16_1INTF = t16_1intf_set.reg;
	}

	/// Check interrupt cause occurrence status.
	if(t16_1intf_set.bit.UFIF == 1) {

	   /**
		* @todo add interrupt function of T16 ch1
		*/
		if (RemSndDownCounter > 0)
		{
			RemSndDownCounter--;
		}
	}
}
