/************************************************************************//**
 *
 * @file    mid_tmr.c
 *
 * @brief   This is 1ms timer counter interface program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "crt0.h"
#include "c17_misc.h"
#include "main_config.h"
#include "c17_init_config.h"
#include "c17_t16_1.h"

/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector22_handler, c17intT16Ch2);		// T16 Ch.2 interrupt
#endif /* CRT0_LINKED */

/* --- macro --- */
#define TMR_T16_CH2		2

/* --- static variable --- */
static volatile unsigned long  TmrEnable = 0UL;
static volatile unsigned short TmrCount[TMR_ENTRIES + 1];

/************************************************************************//**
 * @brief  Initialize this interface.
 *
 * @retval  none
 ***************************************************************************/
void mid_initTmr(void)
{
	c17setCountT16(TMR_T16_CH2, T16CH2_COUNT);
}
/************************************************************************//**
 * @brief   Start the timer count specified by index.
 *          Counter specified by index to be 0 before start counting.
 *
 * @param   index   index of timer.
 *
 * @retval  none
 ***************************************************************************/
void mid_startTmr(TMR_TIMER_ENUM index)
{
	if (index < TMR_ENTRIES)
	{
		unsigned long mask;
		unsigned long pren = TmrEnable;

		mask = (unsigned long)1UL << index;
		TmrEnable &= (~mask);
		TmrCount[index] = 0;
		TmrEnable |= mask;
		if (pren == 0UL)
		{
			c17startT16(TMR_T16_CH2);
			c17enableIntT16(TMR_T16_CH2);
		}
	}
}
/************************************************************************//**
 * @brief   Stop the timer count specified by index.
 *
 * @param   index   index of timer.
 *
 * @retval  none
 ***************************************************************************/
void mid_stopTmr(TMR_TIMER_ENUM index)
{
	if (index < TMR_ENTRIES)
	{
		unsigned long mask;
		mask = (unsigned long)1UL << index;
		TmrEnable &= (~mask);
		if (TmrEnable == 0UL)
		{
			c17disableIntT16(TMR_T16_CH2);
			c17stopT16(TMR_T16_CH2);
		}
		TmrCount[index] = 0;
	}
}
/************************************************************************//**
 * @brief   Get counter value specified by index.
 *
 * @param   index   index of timer.
 *
 * @retval  none
 ***************************************************************************/
unsigned short mid_getTmr(TMR_TIMER_ENUM index)
{
	if (index < TMR_ENTRIES)
	{
		return TmrCount[index];
	}
	return 0;
}
/************************************************************************//**
 * @brief   T16 CH.2 Interrupt function.
 *
 * @param   none
 *
 * @retval  none
 ***************************************************************************/
void c17intT16Ch2(void)
{
	T16_NINTF_REG	t16_2intf_set;

	/* T16_2INTF  T16 Ch.2  Interrupt Flag Register read & clear */
	{
		t16_2intf_set.reg = T16_2INTF;
		T16_2INTF = t16_2intf_set.reg;
	}

	/// Check interrupt cause occurrence status.
	if (t16_2intf_set.bit.UFIF == 1) {

	   /**
		* @todo add interrupt function of T16 ch2
		*/
		int i;
		unsigned long mask = 1;
		for (i = 0; i < TMR_ENTRIES; i++, mask <<= 1)
		{
			if (TmrEnable & mask)
			{
				TmrCount[i]++;
			}
		}
	}
}

