/************************************************************************//**
 *
 * @file    util.c
 *
 * @brief   This is utility function program.
 *
 * @par     Copyright:
 *          Copyright(C) SEIKO EPSON CORPORATION 2018. All rights reserved
 *
 * @warning -
 *
 * @par     History:
 *          - 16 Mar. 2018    1.00
 *              - First revision.
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "util.h"
#include "c17_rtca_1.h"



/************************************************************************//**
 * @brief   Up or down unsigned short value.
 *
 * @param   isUp
 *  @li @c  0        down
 *  @li @c  1        up
 * @param   pnumVal     pointer to unsigned short value.
 * @param   numMax      Numeric maximum value
 * @param   numMin      Numeric minimum value
 * @param   wrap
 *  @li @c  UTIL_WRAP_DISABLE    maintain maximum or minimum if overflow or underflow occurs.
 *  @li @1  UTIL_WRAP_ENABLE     wrap to maximum or minimum if overflow or underflow occurs.
 * @retval  0        overflow or underflow didn't occur.
 * @retval  1        overflow or underflow occurred.
 ***************************************************************************/
int util_upDnUShortVal(int isUp, unsigned short *pnumVal, unsigned short numMin, unsigned short numMax, int wrap)
{
	if (isUp)
	{
		return util_incUShortVal(pnumVal, numMin, numMax, wrap);
	}
	return util_decUShortVal(pnumVal, numMin, numMax, wrap);
}

/************************************************************************//**
 * @brief   Up or down hour value.
 *
 * @param   isUp
 *  @li @c  0        down
 *  @li @c  1        up
 * @param   pnumHour Pointer to variable that is stored hour (numeric).
 * @retval  0        overflow or underflow didn't occur.
 * @retval  1        overflow or underflow occurred.
 ***************************************************************************/
int util_upDnHour(int isUp, unsigned short *pnumHour)
{
	if (isUp)
	{
		return util_incUShortVal(pnumHour, 0, 23, UTIL_WRAP_ENABLE);
	}
	return util_decUShortVal(pnumHour, 0, 23, UTIL_WRAP_ENABLE);
}
/************************************************************************//**
 * @brief   Up or down minute value.
 *
 * @param   isUp
 *  @li @c  0          down
 *  @li @c  1          up
 * @param   pnumMinute Pointer to variable that is stored minute (numeric).
 * @retval  0          overflow or underflow didn't occur.
 * @retval  1          overflow or underflow occurred.
 ***************************************************************************/
int util_upDnMinute(int isUp, unsigned short *pnumMinute)
{
	if (isUp)
	{
		return util_incUShortVal(pnumMinute, 0, 59, UTIL_WRAP_ENABLE);
	}
	return util_decUShortVal(pnumMinute, 0, 59, UTIL_WRAP_ENABLE);
}
/************************************************************************//**
 * @brief   Increment unsigned short value.
 *
 * @param   pnumVal  pointer to unsigned short value.
 * @param   numMax   maximum value
 * @param   numMin   minimum value
 * @param   wrap
 *  @li @c  UTIL_WRAP_DISABLE   maintain maximum if overflow occurs.
 *  @li @1  UTIL_WRAP_ENABLE    wrap to minimum if overflow occurs.
 * @retval  0        overflow flow didn't occur.
 * @retval  1        overflow flow occurred.
 ***************************************************************************/
int util_incUShortVal(unsigned short *pnumVal, unsigned short numMin, unsigned short numMax, int wrap)
{
	int IsOverflow = 0;

	if (*pnumVal >= numMax)
	{
		IsOverflow = 1;
		*pnumVal = (wrap == UTIL_WRAP_DISABLE) ? numMax : numMin;
	}
	else
	{
		*pnumVal = *pnumVal + 1;
	}
	return IsOverflow;
}
/************************************************************************//**
 * @brief   Decrement unsigned short value.
 *
 * @param   pnumVal  pointer to unsigned short value.
 * @param   numMax   maximum value
 * @param   numMin   minimum value
 * @param   wrap
 *  @li @c  UTIL_WRAP_DISABLE   maintain minimum if underflow occurs.
 *  @li @1  UTIL_WRAP_ENABLE    wrap to maximum if underflow occurs.
 * @retval  0        underflow didn't occur.
 * @retval  1        underflow occurred.
 ***************************************************************************/
int util_decUShortVal(unsigned short *pnumVal, unsigned short numMin, unsigned short numMax, int wrap)
{
	int IsUnderflow = 0;
	if (*pnumVal <= numMin)
	{
		IsUnderflow = 1;
		*pnumVal = (wrap == UTIL_WRAP_DISABLE) ? numMin : numMax;
	}
	else
	{
		*pnumVal = *pnumVal - 1;
	}
	return IsUnderflow;
}


/************************************************************************//**
 * @brief   Get days of a month.
 *
 * @param   numYear     Year
 * @param   numMonth    Month.
 *
 * @retval  days.
 ***************************************************************************/
unsigned short util_getDaysOfMonth(unsigned short numYear, unsigned short numMonth)
{
	unsigned short days = 31;

	switch(numMonth)
	{
	case 2:
		if ((numYear == 0) || ((numYear % 4) == 0))
		{
			days = 	29;
		}
		else
		{
			days = 28;
		}
		break;
	case 4:
	case 6:
	case 9:
	case 11:
		days = 30;
		break;
	}
	return days;
}

/************************************************************************//**
 * @brief   Get day of week.
 *
 * @param   numYear     Year.
 * @param   numMonth    Month.
 * @param   numDay      Day.
 *
 * @retval  day of week.
 ***************************************************************************/
unsigned short util_getWeek(unsigned short numYear, unsigned short numMonth, unsigned short numDay)
{
	unsigned short i;
	unsigned short days = 0;

	for (i = 0; i < numYear; i++)
	{
		if ((i == 0) || ((i % 4) == 0))
		{
			days += 366;
		}
		else
		{
			days += 365;
		}
	}
	for (i = 1; i < numMonth; i++)
	{
		days += util_getDaysOfMonth(numYear, i);
	}
	return ((days + numDay + 5) % 7);
}
/************************************************************************//**
 * @brief   Convert time to a serial value
 *
 * @param  rtcaMode
 *  @li @c C17_RTCA_12H           12h
 *  @li @c C17_RTCA_24H           24h
 * @param  amPm
 *  @li @c C17_RTCA_AM            AM
 *  @li @c C17_RTCA_PM            PM
 * @param   numMinute  Minute.
 * @param   numHour    Hour.
 * @retval  serial value.
 ***************************************************************************/
unsigned short util_TimeToSerial(unsigned short rtcMode, unsigned short amPm, unsigned short numHour, unsigned short numMinute)
{

	if ((rtcMode == C17_RTCA_12H) && (amPm == C17_RTCA_PM))
	{
		numHour += 12;
	}
	return ((numHour * 60) + numMinute);

}
/************************************************************************//**
 * @brief   Convert numeric to BCD.
 *
 * @param   numVal   numeric value.
 *
 * @retval  BCD value.
 ***************************************************************************/
unsigned short util_convUShortNumToBcd(unsigned short numVal)
{
	int i;
	unsigned short bcd;
	unsigned short sht;

	for (i = 0, bcd = 0, sht = 0; i < 4; i++, sht += 4, numVal /= 10)
	{
		bcd += (numVal % 10) << sht;
	}
	return bcd;
}
/************************************************************************//**
 * @brief   Convert BCD to numeric.
 *
 * @param   numVal   BCD value.
 *
 * @retval  numeric value.
 ***************************************************************************/
unsigned short util_convUShortBcdToNum(unsigned short bcdVal)
{
	int i;
	unsigned short num;
	unsigned short mul;

	for (i = 0, num = 0, mul = 1; i < 4; i++, mul *= 10, bcdVal >>= 4)
	{
		num += (bcdVal & 0xF) * mul;
	}
	return num;
}
/************************************************************************//**
 * @brief   Convert BCD to string of 2 digits.
 *
 * @param   pbuf     pointer to string buffer.
 * @param   bcdVal   BCD value.
 * @param   upChar   Alignment character.
 *
 * @retval  none
 ***************************************************************************/
void util_bcdTo2DigitStr(char *pbuf, unsigned short bcdNum, char upChar)
{
	unsigned short high = (bcdNum >> 4) & 0xF;
	pbuf[0] = (high == 0) ? upChar : (high + '0');
	pbuf[1] = (bcdNum & 0xF) + '0';
	pbuf[2] = '\0';
}
/************************************************************************//**
 * @brief   Convert decimal number to string of 2 digits.
 *
 * @param   pbuf     pointer to string buffer.
 * @param   decNum   decimal number value.
 * @param   upChar   Alignment character.
 *
 * @retval  none
 ***************************************************************************/
void util_decTo2DigitStr(char *pbuf, unsigned short decNum, char upChar)
{
	unsigned short high = (decNum / 10) % 10;
	pbuf[0] = (high == 0) ? upChar : (high + '0');
	pbuf[1] = (decNum % 10) + '0';
	pbuf[2] = '\0';
}
/************************************************************************//**
 * @brief   Convert decimal number to string of 3 digits.
 *
 * @param   pbuf     pointer to string buffer.
 * @param   decNum   decimal number value.
 * @param   upChar   Alignment character.
 *
 * @retval  none.
 ***************************************************************************/
void util_decTo3DigitStr(char *pbuf, unsigned short decNum, char upChar)
{
	unsigned short high = (decNum / 100) % 10;
	pbuf[0] = (high == 0) ? upChar : (high + '0');
	pbuf[1] = ((decNum / 10) % 10) + '0';
	pbuf[2] = (decNum % 10) + '0';
	pbuf[3] = '\0';
}
/************************************************************************//**
 * @brief   Convert hex number to string of 2 digits.
 *
 * @param   pbuf     pointer to string buffer.
 * @param   hexNum   hex number value.
 * @param   upChar   Alignment character.
 *
 * @retval  none.
 ***************************************************************************/
void util_hexTo2DigitStr(char *pbuf, unsigned short hexNum, char upChar)
{
	unsigned short high = (hexNum >> 4) & 0xF;
	pbuf[0] = (high == 0) ? upChar : util_HexToChar(high);
	pbuf[1] = util_HexToChar(hexNum & 0xF);
	pbuf[2] = '\0';
}
/************************************************************************//**
 * @brief   Convert hex number to character.
 *
 * @param   num   hex number value.
 *
 * @retval  character of hex number.
 ***************************************************************************/
char util_HexToChar(unsigned char num)
{
	if (num >= 0xA)
	{
		return ('A' + (num - 0xA));
	}
	return ('0' + num);
}

/************************************************************************//**
 * @brief   Convert Celsius to fahrenheits.
 *
 * @param   cel   Celsius value.
 *
 * @retval  days.
 ***************************************************************************/
unsigned short util_convC2F(unsigned short cel)
{
	return ((unsigned short)((((cel * 90) / 5) + 5) / 10) + 32);
}

